#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define NMAX 360

unsigned int bit[32], b1mask[32];

void ComputeBits() {
	bit[0] = 1;
	b1mask[0] = 0;
	for (int i = 1; i < 32; i++) {
		bit[i] = bit[i - 1] << 1;
		b1mask[i] = b1mask[i - 1] | bit[i - 1];
	}
}

/* BIGNUM functions (start) */

#define MAX_DIGITS_10 200
typedef unsigned char BIGNUM10[MAX_DIGITS_10];

#define BITS_PER_DIGIT 30
#define MAX_DIGIT_VALUE 1073741823

typedef unsigned int BIGNUM2[13];

BIGNUM10 TwoNBIGNUM10[NMAX + 1];
BIGNUM2 TwoNBIGNUM2[NMAX + 1];

void ConvertStringToBIGNUM10(char s[], BIGNUM10 NUM) {
	int i, L = strlen(s);

	memset(NUM, 0, sizeof(BIGNUM10));
	NUM[0] = L;
	for (i = 0; i < L; i++)
		NUM[L - i] = s[i] - '0';
	//fprintf(stderr, "NUM[0]=%d\n", NUM[0]);
}

inline void AddBIGNUM2(BIGNUM2 x, BIGNUM2 y, BIGNUM2 z) {
	int i;
	for (i = 0; i < 13; i++)
		z[i] = x[i] + y[i];

	for (i = 0; i < 12; i++) {
		z[i + 1] += (z[i] >> BITS_PER_DIGIT);
		z[i] &= MAX_DIGIT_VALUE;
	}
	z[12] &= MAX_DIGIT_VALUE;
}

void AddBIGNUM10(BIGNUM10 x, BIGNUM10 y, BIGNUM10 z) {
	int i;
	memset(z, 0, sizeof(BIGNUM10));
	z[0] = x[0];
	if (y[0] > z[0])
		z[0] = y[0];
	for (i = 1; i <= z[0]; i++) {
		z[i] += x[i] + y[i];
		z[i + 1] += (z[i] / 10);
		z[i] %= 10;
	}
	if (z[z[0] + 1] > 0)
		z[0]++;
}

void DivideByXBIGNUM10(BIGNUM10 n, int x, BIGNUM10 quotient, int* remainder) {
	int i, j, ndigits = 0, v;
 
	memset(quotient, 0, sizeof(BIGNUM10));
	v = 0, i = n[0];
	while (v < x && i) {
		v = v * 10 +n[i];
		i--;
	}
 
	ndigits = 1;
	quotient[1] = v / x;
	v %= x;
 
	for (; i; i--) {
		v = v * 10 + n[i];
		quotient[++ndigits] = v / x;
		v %= x;
	}

	*remainder = v;
	
	for (v = ndigits >> 1, i = 1; i <= v; i++) {
		j = quotient[i];
		quotient[i] = quotient[ndigits - i + 1];
		quotient[ndigits - i + 1]=j;
	}
 
	quotient[0] = ndigits;
}

BIGNUM10 tmp_bignum10, tmp2_bignum10;

void ConvertBIGNUM10ToBIGNUM2(BIGNUM10 x, BIGNUM2 y) {
	int q = 0, r;
	memset(y, 0, sizeof(BIGNUM2));

	while (x[0] > 1 || x[1] > 0) {
		DivideByXBIGNUM10(x, 2, tmp_bignum10, &r);
		memcpy(x, tmp_bignum10, sizeof(BIGNUM10));
		if (r > 0)
			y[(q / BITS_PER_DIGIT)] |= bit[q % BITS_PER_DIGIT];
		q++;
	}
}

void ConvertBIGNUM2ToBIGNUM10(BIGNUM2 x, BIGNUM10 y) {
	unsigned int i, j;
	memset(y, 0, sizeof(BIGNUM10));
	for (i = 0; i < 12; i++) {
		for (j = 0; j < BITS_PER_DIGIT; j++)
			if ((x[i] & bit[j]) > 0) {
				AddBIGNUM10(y, TwoNBIGNUM10[i * BITS_PER_DIGIT + j], tmp_bignum10);
				memcpy(y, tmp_bignum10, sizeof(BIGNUM10));
			}
	}
}

void SubtractBIGNUM2(BIGNUM2 x, BIGNUM2 y, BIGNUM2 z) {
	int i, j;

	memset(z, 0, sizeof(BIGNUM2));
	for (i = 12; i >= 0; i--) {
		if (x[i] >= y[i])
			z[i] = x[i] - y[i];
		else {
			for (j = i + 1; j < 13; j++)
				if (z[j] > 0)
					break;
			z[j]--;
			j--;
			while (j > i) {
				z[j] = MAX_DIGIT_VALUE;
				j--;
			}
			
			z[i] = (x[i] + MAX_DIGIT_VALUE + 1) - y[i];
		}
	}
}

int CompareBIGNUM2(BIGNUM2 a, BIGNUM2 b) {
	for (int i = 12; i >= 0; i--) 
		if (a[i] < b[i])
			return -1;
		else if (a[i] > b[i])
			return +1;
	return 0;
}

void ComputePowersOfTwo() {
	int i;
	memset(TwoNBIGNUM10[0], 0, sizeof(BIGNUM10));
	memset(TwoNBIGNUM2[0], 0, sizeof(BIGNUM2));

	TwoNBIGNUM10[0][0] = TwoNBIGNUM10[0][1] = 1;
	TwoNBIGNUM2[0][0] = 1;

	for (i = 1; i < NMAX; i++) {
		AddBIGNUM10(TwoNBIGNUM10[i - 1], TwoNBIGNUM10[i - 1], TwoNBIGNUM10[i]);
		AddBIGNUM2(TwoNBIGNUM2[i - 1], TwoNBIGNUM2[i - 1], TwoNBIGNUM2[i]);
	}
}

void PrintBIGNUM10(BIGNUM10 x) {
	fprintf(stderr, "%d: ", x[0]);
	if (x[0] == 0)
		printf("0");
	else {
		for (int i = x[0]; i >= 1; i--)
			printf("%d", x[i]);
	}
	printf("\n");
}

BIGNUM10 tmp_conv;

void PrintBIGNUM2(BIGNUM2 x) {
	ConvertBIGNUM2ToBIGNUM10(x, tmp_conv);
	PrintBIGNUM10(tmp_conv);
}

#define NNMAX 351

BIGNUM2 C[NNMAX][NNMAX], M;

void ComputeC() {
	int i, j;

	memset(C[0][0], 0, sizeof(BIGNUM2));
	C[0][0][0] = 1;
	for (i = 1; i < NNMAX; i++) {
		memset(C[i][0], 0, sizeof(BIGNUM2));
		C[i][0][0] = 1;
		for (j = 1; j <= i; j++)
			AddBIGNUM2(C[i - 1][j], C[i - 1][j - 1], C[i][j]);
			//C[i][j] = C[i - 1][j] + C[i - 1][j - 1];
	}
}

char SM[10000];
BIGNUM10 M10;
BIGNUM2 M2, TMP;

int main() {
	ComputeBits();
	ComputePowersOfTwo();
	ComputeC();

	//PrintBIGNUM2(C[350][175]);
	
	int T, N, K, i;
	
	scanf("%d", &T);
	while (T--) {
		scanf("%d %d %s", &N, &K, SM);
		if (N > 350)
			exit(888);
		
		ConvertStringToBIGNUM10(SM, M10);
		ConvertBIGNUM10ToBIGNUM2(M10, M2);

		if (CompareBIGNUM2(M2, C[N][K]) > 0)
			printf("-1\n");
		else {
			for (i = N; i >= 1; i--)
				if (CompareBIGNUM2(M2, C[i - 1][K]) <= 0)
					printf("0");
				else {
					SubtractBIGNUM2(M2, C[i - 1][K], TMP);
					memcpy(M2, TMP, sizeof(BIGNUM2));
					printf("1");
					K--;
				}
			printf("\n");
		}
	}
	
	return 0;
}
