#include <stdio.h>
#include <stdlib.h>
#include <algorithm>
#include <vector>

using namespace std;

#define DEBUG 0
#define NMAX 131072
#define QMAX 100001
#define MAXNODES 10000000

vector<int> vec[NMAX];
int h[NMAX];
int N, Q;

char visited[NMAX];
int dfsmin[NMAX], dfsmax[NMAX], stk[NMAX], vecidx[NMAX], parent[NMAX], o[NMAX];

void IterativeDFS() {
	int i, j, niv, dfsindex = 0;

	niv = 1;
	stk[1] = 0;
	visited[0] = 1;
	parent[0] = -1;
	dfsindex = 1;
	dfsmin[0] = 1;
	o[1] = 0;

	while (niv >= 1) {
		i = stk[niv];
		
		if (vecidx[i] < vec[i].size()) {
			j = vec[i][vecidx[i]];
			if (!visited[j]) {
				niv++;
				stk[niv] = j;
				visited[j] = 1;
				vecidx[j] = 0;
				dfsindex++;
				dfsmin[j] = dfsindex;
				o[dfsmin[j]] = j;
				parent[j] = i;
			}
			vecidx[i]++;
		} else {
			dfsmax[i] = dfsindex;
			niv--;
		}
	}

	if (DEBUG) {
		for (i = 1; i <= N + 1; i++)
			fprintf(stderr, "i=%d: dfsmin=%d dfsmax=%d parent=%d\n", o[i], dfsmin[o[i]], dfsmax[o[i]], parent[o[i]]);
	}
}

int root[QMAX];
int li[MAXNODES], ls[MAXNODES], u[MAXNODES], lson[MAXNODES], rson[MAXNODES], nnodes;
int cnt[2 * NMAX];

void BuildTree() {
	int i, j;

	root[0] = 1;
	li[1] = 1; ls[1] = NMAX;

	for (i = 1; i < NMAX; i++) {
		j = i << 1;
		li[j] = li[i];
		ls[j] = (li[i] + ls[i]) >> 1;
		li[j + 1] = ls[j] + 1;
		ls[j + 1] = ls[i];
		lson[i] = j;
		rson[i] = j + 1;
	}

	nnodes = 2 * NMAX - 1;
}

void DuplicateNode(int node) {
	nnodes++;
	if (nnodes >= MAXNODES)
		exit(1);
	li[nnodes] = li[node];
	ls[nnodes] = ls[node];
	u[nnodes] = u[node];
	lson[nnodes] = lson[node];
	rson[nnodes] = rson[node];
}

int qmin, qmax, qupd;

void UpdateWithDuplication(int node) {
	if (qmin <= li[node] && ls[node] <= qmax) {
		u[node] += qupd;
		return;
	}

	if (qmin <= ls[lson[node]]) {
		DuplicateNode(lson[node]);
		lson[node] = nnodes;
		UpdateWithDuplication(lson[node]);
	}

	if (qmax >= li[rson[node]]) {
		DuplicateNode(rson[node]);
		rson[node] = nnodes;
		UpdateWithDuplication(rson[node]);
	}
}

int qans, qpoz;

void QueryU(int node) {
	qans += u[node];
	if (li[node] == qpoz && ls[node] == qpoz)
		return;
	if (qpoz <= ls[lson[node]])
		QueryU(lson[node]);
	else
		QueryU(rson[node]);
}

void UpdateCnt(int p) {
	p = NMAX + p - 1;
	while (p >= 1) {
		cnt[p]++;
		p >>= 1;
	}
}

void QueryCnt(int node) {
	if (qmin <= li[node] && ls[node] <= qmax) {
		qans += cnt[node];
		return;
	}

	int lson = node << 1;
	int rson = lson + 1;
	
	if (qmin <= ls[lson])
		QueryCnt(lson);
	if (qmax >= li[rson])
		QueryCnt(rson);
}

int q[QMAX][2], nq, t;

void PreprocessInput() {
	int i, j, op;

	//freopen("x.txt", "r", stdin);
	scanf("%d", &N);
	for (i = 1; i <= N; i++) {
		scanf("%d %d", &h[i], &j);
		vec[i].push_back(j);
		vec[j].push_back(i);
	}

	IterativeDFS();
	
	scanf("%d", &Q);
	t = nq = 0;

	while (Q--) {
		scanf("%d", &op);
		if (op == 1) {
			scanf("%d %d", &i, &qupd);
			t++;
			DuplicateNode(root[t - 1]);
			root[t] = nnodes;
			qmin = dfsmin[i] + 1;
			qmax = dfsmax[i];
			if (DEBUG)
				fprintf(stderr, "t=%d: update %d-%d qupd=%d\n", t, qmin, qmax, qupd);
			UpdateWithDuplication(root[t]);
		} else {
			scanf("%d", &i);
			nq++;
			q[nq][0] = i;
			q[nq][1] = t;
		}
	}
}

vector<pair<int, int> > vp;
int tmax[NMAX];

void ComputeTmax() {
	int i, li, ls, mid;

	for (i = 1; i <= N; i++) {
		qpoz = dfsmin[i];
		tmax[i] = t + 1;
		li = 0; ls = t;
		while (li <= ls) {
			mid = (li + ls) >> 1;
			qans = 0;
			QueryU(root[mid]);
			if (DEBUG)
				fprintf(stderr, "  Q(i=%d,qpoz=%d): time=%d qans=%d\n", i, qpoz, mid, qans);
			if (qans >= h[i]) {
				tmax[i] = mid;
				ls = mid - 1;
			} else
				li = mid + 1;
		}

		if (DEBUG)
			fprintf(stderr, "i=%d: tmax=%d\n", i, tmax[i]);

		vp.push_back(make_pair(tmax[i], i));
	}

	sort(vp.begin(), vp.end());
}

void ProcessQueries() {
	int i, vpidx = 0, qidx = 1;

	while (qidx <= nq) {
		while (vpidx < vp.size() && vp[vpidx].first <= q[qidx][1]) {
			UpdateCnt(dfsmin[vp[vpidx].second]);
			vpidx++;
		}
	
		i = q[qidx][0];
		qmin = dfsmin[i] + 1; qmax = dfsmax[i];
		qans = 0;
		QueryCnt(1);
		if (DEBUG)
			fprintf(stderr, "t=%d cnt[%d-%d]=%d\n", q[qidx][1], qmin, qmax, qans);
		printf("%d\n", dfsmax[i] - dfsmin[i] - qans);
		qidx++;
	}
}

int main() {
	BuildTree();
	PreprocessInput();
	ComputeTmax();
	ProcessQueries();
	return 0;
}
