#include <stdio.h>
#include <map>
#include <vector>

using namespace std;

#define DEBUG 0
#define NMAX 111111
#define LOGMAX 18
#define MOD 1000000009

int h[NMAX], S[NMAX];
int N, answer;

int bit[LOGMAX];

void ComputeBits() {
	int i;
	bit[0] = 1;
	for (i = 1; i < LOGMAX; i++)
		bit[i] = bit[i - 1] * 2;
}

int group[NMAX][LOGMAX + 1], vs[NMAX], nbefore[NMAX];
vector<int> g[NMAX], gtmp[NMAX];
map<int, vector<int> > smap;
map<int, vector<int> >::iterator it;

void SuffixArray() {
	int i, j, r, ng = 0;
	char c;

	if (DEBUG) {
		fprintf(stderr, "S:");
		for (i = 1; i <= N; i++)
			fprintf(stderr, " %d", S[i]);
		fprintf(stderr, "\n");
	}
	
	for (i = 0; i < NMAX; i++)
		g[i].clear();

	smap.clear();
	for (i = 1; i <= N; i++)
		smap[S[i]].push_back(i);

	for (ng = 0, it = smap.begin(); it != smap.end(); it++) {
		for (j = 0; j < (it->second).size(); j++) {
			group[it->second[j]][0] = ng;
			g[ng].push_back((it->second)[j]);
		}
		ng++;
	}

	for (r = 0; r < LOGMAX; r++) {
		nbefore[0] = 0;
		for (i = 0; i < ng; i++) {
			nbefore[i + 1] = nbefore[i] + g[i].size();
			for (j = 0; j < g[i].size(); j++) {
				if (g[i][j] + bit[r] <= N)
					gtmp[group[g[i][j] + bit[r]][r]].push_back(g[i][j]);
				else
					gtmp[ng].push_back(g[i][j]);
				//fprintf(stderr, "r=%d nbefore=%d i=%d/%d elem=%d\n", r, nbefore[i], i, ng, g[i][j]);
			}
			g[i].clear();
		}
		
		for (i = 0; i <= ng; i++) {
			for (j = 0; j < gtmp[i].size(); j++) {
				vs[nbefore[group[gtmp[i][j]][r]] + 1] = gtmp[i][j];
				nbefore[group[gtmp[i][j]][r]]++;
			}
			gtmp[i].clear();
		}

		ng = 0;
		g[0].push_back(vs[1]);
		group[vs[1]][r + 1] = 0;

		for (i = 2; i <= N; i++) {
			if (group[vs[i]][r] != group[vs[i - 1]][r] ||
				vs[i] + bit[r] > N ||
				vs[i - 1] + bit[r] > N ||
				group[vs[i] + bit[r]][r] != group[vs[i - 1] + bit[r]][r])
				ng++;
			g[ng].push_back(vs[i]);
			group[vs[i]][r + 1] = ng;
		}

		ng++;
	}

	if (DEBUG) {
		fprintf(stderr, "ng=%d\nSuffix order:", ng);
		for (i = 0; i < ng; i++)
			for (j = 0; j < g[i].size(); j++)
				fprintf(stderr, " %d", g[i][j]);
		fprintf(stderr, "\n");
	}
}

int lcp[NMAX];

void ComputeLCPs() {
	int i, j, p, q;

	answer = N - vs[1] + 1;

	for (i = 2; i <= N; i++) {
		p = vs[i];
		q = vs[i - 1];
		lcp[i] = 0;

		for (j = LOGMAX - 1; j >= 0 && p <= N && q <= N; j--) {
			if (group[p][j] == group[q][j]) {
				lcp[i] += bit[j];
				p += bit[j];
				q += bit[j];
			}
		}
		
		answer = (answer + (N - vs[i] + 1 - lcp[i])) % MOD;
	}

	if (DEBUG) {
		for (i = 2; i <= N; i++)
			fprintf(stderr, "i=%d: LCP(%d,%d)=%d\n", i, vs[i - 1], vs[i], lcp[i]);
	}
}

int main() {
	ComputeBits();

	//freopen("x.txt", "r", stdin);
	int T, i;
	scanf("%d", &T);
	
	while (T--) {
		scanf("%d", &N);
		for (i = 0; i < N; i++)
			scanf("%d", &h[i]);
		for (i = 1; i < N; i++)
			S[i] = h[i] - h[i - 1];
		N--;
		if (N == 0)
			answer = 0;
		else {
			SuffixArray();
			ComputeLCPs();
		}
		printf("%d\n", answer);
	}
	
	return 0;
}
