#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <algorithm>
#include <vector>

using namespace std;

#define DEBUG 0
#define NMAX 55
#define MMAX 11111
#define MSMALL 0

int xp[NMAX], yp[NMAX], xa[MMAX], ya[MMAX], xb[MMAX], yb[MMAX];
int N, M, MM;

void ReadInput() {
	int i;
	scanf("%d %d", &N, &M);
	for (i = 0; i < N; i++)
		scanf("%d %d", &xp[i], &yp[i]);
	for (i = 0; i < M; i++)
		scanf("%d %d %d %d", &xa[i], &ya[i], &xb[i], &yb[i]);
}

#define XRANGE 1100000
#define ZERO 550000

int yminv[XRANGE], ymaxv[XRANGE], *ymin, *ymax;
int lcidx[NMAX], nlcidx, ucidx[NMAX], nucidx;
int xmin, xmax;

#define eps 1e-7

void GetPolygonChain(int sidx, int dir, int cidx[], int *ncidx) {
	int i;
	*ncidx = 1;
	cidx[0] = sidx;

	for (i = (sidx + dir + N) % N; 1; i = (i + dir + N) % N) {
		cidx[*ncidx] = i;
		(*ncidx)++;
		if (xp[i] == xmax)
			break;
	}
}

void GetExtremeY(int cidx[], int ncidx, int yextreme[], int type) {
	int x, i, dx, dy, ycint;
	double yc;

	x = xp[cidx[0]];
	yextreme[x] = yp[cidx[0]] + type;
	
	for (i = 1; i < ncidx; i++) {
		dx = xp[cidx[i]] - xp[cidx[i - 1]];
		dy = yp[cidx[i]] - yp[cidx[i - 1]];
		
		for (x++; x < xp[cidx[i]]; x++) {
			yc = yp[cidx[i - 1]] + (double) (x - xp[cidx[i - 1]]) / (double) (dx) * (double) (dy);
			ycint = (int) (yc + 2 * type);
			if (type > 0) {
				while (ycint - 1 > yc && fabs(ycint - 1 - yc) > eps)
					ycint--;
			} else {
				while (ycint + 1 < yc && fabs(ycint + 1 - yc) > eps)
					ycint++;
			}

			yextreme[x] = ycint;
		}

		yextreme[x] = yp[cidx[i]] + type;
	}
}

int gcd(int a, int b) {
	if (a == 0)
		return b;
	if (b == 0)
		return a;
	return gcd(b, a % b);
}

long long llgcd(long long a, long long b) {
	if (a == 0)
		return b;
	if (b == 0)
		return a;
	return llgcd(b, a % b);
}

int num_points_on_polygon;
long long num_points_inside_polygon;

void ComputeNumPointsOnPolygon() {
	int i, dx, dy, d;
	num_points_on_polygon = 0;

	for (i = 0; i < N; i++) {
		dx = abs(xp[(i + 1) % N] - xp[i]);
		dy = abs(yp[(i + 1) % N] - yp[i]);
		if (dx == 0)
			num_points_on_polygon += dy;
		else if (dy == 0)
			num_points_on_polygon += dx;
		else {
			d = gcd(dx, dy);
			num_points_on_polygon += d;
		}
	}
}

void ProcessPolygon() {
	int i, imin, iuc, ilc, lcdir, ucdir;
	double u1, u2;

	ymin = yminv + ZERO;
	ymax = ymaxv + ZERO;

	xmin = xmax = xp[imin = 0];
	for (i = 1; i < N; i++) {
		if (xp[i] < xmin) {
			xmin = xp[i];
			imin = i;
		}
		if (xp[i] > xmax)
			xmax = xp[i];
	}

	u1 = atan2(yp[(imin - 1 + N) % N] - yp[imin], xp[(imin - 1 + N) % N] - xp[imin]);
	u2 = atan2(yp[(imin + 1) % N] - yp[imin], xp[(imin + 1) % N] - xp[imin]);

	if (u1 < u2) {
		lcdir = -1;
		ucdir = +1;
	} else {
		lcdir = +1;
		ucdir = -1;
	}

	iuc = ilc = imin;
	while (xp[ilc] == xp[(ilc + lcdir + N) % N])
		ilc = (ilc + lcdir + N) % N;
	while (xp[iuc] == xp[(iuc + ucdir + N) % N])
		iuc = (iuc + ucdir + N) % N;
	
	GetPolygonChain(ilc, lcdir, lcidx, &nlcidx);
	GetPolygonChain(iuc, ucdir, ucidx, &nucidx);
	GetExtremeY(lcidx, nlcidx, ymin, +1);
	GetExtremeY(ucidx, nucidx, ymax, -1);

	ymin[xmin] = ymin[xmax] = 0;
	ymax[xmin] = ymax[xmax] = -1;
	ComputeNumPointsOnPolygon();

	num_points_inside_polygon = 0;
	for (i = xmin; i <= xmax; i++)
		if (ymin[i] <= ymax[i])
			num_points_inside_polygon += (ymax[i] - ymin[i] + 1);
	
	if (DEBUG) {
		fprintf(stderr, "Lower chain:");
		for (i = 0; i < nlcidx; i++)
			fprintf(stderr, " %d(%d,%d)", lcidx[i], xp[lcidx[i]], yp[lcidx[i]]);
		fprintf(stderr, "\nUpper chain:");
		for (i = 0; i < nucidx; i++)
			fprintf(stderr, " %d(%d,%d)", ucidx[i], xp[ucidx[i]], yp[ucidx[i]]);
		fprintf(stderr, "\n Extreme Ys:\n");
		for (i = xmin; i <= xmax; i++)
			fprintf(stderr, " x=%d: ymin=%d ymax=%d\n", i, ymin[i], ymax[i]);
		fprintf(stderr, "Number of points on polygon=%d\n", num_points_on_polygon);
		fprintf(stderr, "Number of points inside polygon=%lld\n", num_points_inside_polygon);
	}
}

inline int InsidePolygon(int x, int y) {
	if (x < xmin || x > xmax)
		return 0;
	if (ymin[x] <= y && y <= ymax[x])
		return 1;
	return 0;
}

long long ap[NMAX], bp[NMAX], cp[NMAX];

void ClipLines() {
	int i, j, x, y, xx, yy, dx, dy, d, stepx, stepy, idxmin, idxmax, idx, li, ls, mid;
	long long tmp1, tmp2, tmp3, tmp4, a, b, c;
	int insign = 0, ok1, ok2;
	
	for (i = 0; i < N; i++) {
		ap[i] = yp[i] - yp[(i + 1) % N];
		bp[i] = xp[(i + 1) % N] - xp[i];
		cp[i] = (long long) xp[i] * (long long) yp[(i + 1) % N] - (long long) xp[(i + 1) % N] * (long long) yp[i];
		tmp1 = ap[i] * xp[(i + 2) % N] + bp[i] * yp[(i + 2) % N] + cp[i];
		if (tmp1 > 0)
			insign = 1;
		else if (tmp1 < 0)
			insign = -1;
	}

	if (insign == 0)
		exit(1);
	
	MM = 0;
	
	for (i = 0; i < M; i++) {
		if (xa[i] == xb[i] && ya[i] == yb[i]) {
			if (InsidePolygon(xa[i], ya[i])) {
				xa[MM] = xb[MM] = xa[i];
				ya[MM] = yb[MM] = ya[i];
				MM++;
			}
		} else {
			dx = xb[i] - xa[i];
			dy = yb[i] - ya[i];
			d = gcd(abs(dx), abs(dy));
			stepx = dx / d;
			stepy = dy / d;

			if (InsidePolygon(xa[i], ya[i])) {
				idxmax = 0;
				li = 1; ls = d;
				while (li <= ls) {
					mid = (li + ls) >> 1;
					x = xa[i] + mid * stepx;
					y = ya[i] + mid * stepy;
					if (InsidePolygon(x, y)) {
						idxmax = mid;
						li = mid + 1;
					} else
						ls = mid - 1;
				}

				xa[MM] = xa[i]; ya[MM] = ya[i];
				xb[MM] = xa[i] + idxmax * stepx;
				yb[MM] = ya[i] + idxmax * stepy;
				MM++;
			} else if (InsidePolygon(xb[i], yb[i])) {
				idxmin = d;
				li = 0; ls = d - 1;
				while (li <= ls) {
					mid = (li + ls) >> 1;
					x = xa[i] + mid * stepx;
					y = ya[i] + mid * stepy;
					if (InsidePolygon(x, y)) {
						idxmin = mid;
						ls = mid - 1;
					} else
						li = mid + 1;
				}

				xa[MM] = xa[i] + idxmin * stepx;
				ya[MM] = ya[i] + idxmin * stepy;
				xb[MM] = xb[i]; yb[MM] = yb[i];
				MM++;
			} else {
				a = ya[i] - yb[i];
				b = xb[i] - xa[i];
				c = (long long) xa[i] * (long long) yb[i] - (long long) xb[i] * (long long) ya[i];

				for (j = 0; j < N; j++) {
					tmp1 = ap[j] * xa[i] + bp[j] * ya[i] + cp[j];
					tmp2 = ap[j] * xb[i] + bp[j] * yb[i] + cp[j];
					tmp3 = a * xp[j] + b * yp[j] + c;
					tmp4 = a * xp[(j + 1) % N] + b * yp[(j + 1) % N] + c;
					if ((tmp1 == 0 && tmp2 != 0) ||
						(tmp1 > 0 && tmp2 <= 0) ||
						(tmp1 < 0 && tmp2 >= 0))
						ok1 = 1;
					else
						ok1 = 0;
					
					if ((tmp3 == 0 && tmp4 != 0) ||
						(tmp3 > 0 && tmp4 <= 0) ||
						(tmp3 < 0 && tmp4 >= 0))
						ok2 = 1;
					else
						ok2 = 0;
					
					if (ok1 && ok2)
						break;
				}

				if (j == N)
					continue;
				
				if (tmp1 == 0) {
					x = xa[i] + stepx;
					y = ya[i] + stepy;
					if (!InsidePolygon(x, y))
						continue;
					idx = 1;
				} else if (tmp2 == 0) {
					x = xb[i] - stepx;
					y = yb[i] - stepy;
					if (!InsidePolygon(x, y))
						continue;
					idx = d - 1;
				} else {
					if ((tmp1 > 0 && insign > 0) ||
						(tmp1 < 0 && insign < 0)) {
						li = 1; ls = d; idx = 0;
						while (li <= ls) {
							mid = (li + ls) >> 1;
							x = xa[i] + mid * stepx;
							y = ya[i] + mid * stepy;
							tmp1 = ap[j] * x + bp[j] * y + cp[j];
							if ((tmp1 > 0 && insign > 0) ||
								(tmp1 < 0 && insign < 0)) {
								idx = mid;
								li = mid + 1;
							} else
								ls = mid - 1;
						}
						
						x = xa[i] + idx * stepx;
						y = ya[i] + idx * stepy;
						if (!InsidePolygon(x, y))
							continue;
					} else {
						li = 0; ls = d - 1; idx = d;
						while (li <= ls) {
							mid = (li + ls) >> 1;
							x = xa[i] + mid * stepx;
							y = ya[i] + mid * stepy;
							tmp1 = ap[j] * x + bp[j] * y + cp[j];
							if ((tmp1 > 0 && insign > 0) ||
								(tmp1 < 0 && insign < 0)) {
								idx = mid;
								ls = mid - 1;
							} else
								li = mid + 1;
						}

						x = xa[i] + idx * stepx;
						y = ya[i] + idx * stepy;
						if (!InsidePolygon(x, y))
							continue;
					}
				}

				idxmin = idxmax = idx;
				
				li = 0; ls = idx - 1;
				while (li <= ls) {
					mid = (li + ls) >> 1;
					x = xa[i] + mid * stepx;
					y = ya[i] + mid * stepy;
					if (InsidePolygon(x, y)) {
						idxmin = mid;
						ls = mid - 1;
					} else
						li = mid + 1;
				}

				li = idx + 1; ls = d;
				while (li <= ls) {
					mid = (li + ls) >> 1;
					x = xa[i] + mid * stepx;
					y = ya[i] + mid * stepy;
					if (InsidePolygon(x, y)) {
						idxmax = mid;
						li = mid + 1;
					} else
						ls = mid - 1;
				}

				x = xa[i]; y = ya[i];
				xa[MM] = x + idxmin * stepx;
				ya[MM] = y + idxmin * stepy;
				xb[MM] = x + idxmax * stepx;
				yb[MM] = y + idxmax * stepy;
				MM++;
				
				//fprintf(stderr, "i=%d idxmin=%d idxmax=%d: (%d,%d) - (%d,%d)\n", i, idxmin, idxmax, xa[i] + idxmin * stepx, ya[i] + idxmin * stepy, xa[i] + idxmax * stepx, ya[i] + idxmax * stepy);

				/*
				for (idxmin = 0; idxmin <= d && !InsidePolygon(xa[i], ya[i]); idxmin++, xa[i] += stepx, ya[i] += stepy);
				if (idxmin > d)
					continue;
				for (idxmax = d; idxmax >= 0 && !InsidePolygon(xb[i], yb[i]); idxmax--, xb[i] -= stepx, yb[i] -= stepy);
				if (idxmax < idxmin)
					continue;
				
				xa[MM] = xa[i]; ya[MM] = ya[i];
				xb[MM] = xb[i]; yb[MM] = yb[i];
				fprintf(stderr, "  idxmin=%d idxmax=%d: (%d,%d) - (%d,%d)\n", idxmin, idxmax, xa[MM], ya[MM], xb[MM], yb[MM]);
				MM++;
				*/
			}
		}
	}

	M = MM;
	
	if (DEBUG) {
		fprintf(stderr, "Clipped lines: M=%d\n", M);
		for (i = 0; i < M; i++)
			fprintf(stderr, "%d: (%d,%d) - (%d,%d)\n", i, xa[i], ya[i], xb[i], yb[i]);
	}
}

vector<pair<pair<int, int>, pair<long long, pair<pair<int, int>, int> > > > vlines;
int xam[MMAX], yam[MMAX], xbm[MMAX], ybm[MMAX];

void CompactOverlappingLines() {
	int i, j, dx, dy, d, tmp, stepx, stepy, xaa, yaa, xbb, ybb;
	long long a, b, c, lld;

	if (M == 0)
		return;
	
	vlines.clear();
	
	for (i = 0; i < M; i++) {
		if (xa[i] == xb[i] && ya[i] == yb[i]) {
			vlines.push_back(make_pair(make_pair(0, 0), make_pair(0, make_pair(make_pair(xa[i], ya[i]), i))));
		} else {
			dx = xb[i] - xa[i];
			dy = yb[i] - ya[i];
			d = gcd(abs(dx), abs(dy));
			stepx = dx / d;
			stepy = dy / d;
			if (stepx < 0 || (stepx == 0 && stepy < 0)) {
				tmp = xa[i]; xa[i] = xb[i]; xb[i] = tmp;
				tmp = ya[i]; ya[i] = yb[i]; yb[i] = tmp;
				stepx = -stepx;
				stepy = -stepy;
			}

			a = ya[i] - yb[i];
			b = xb[i] - xa[i];
			c = (long long) xa[i] * (long long) yb[i] - (long long) xb[i] * (long long) ya[i];
			lld = llgcd((a > 0) ? a : (-a), (b > 0) ? b : (-b));
			lld = llgcd(lld, (c > 0) ? c : (-c));
			c /= lld;
			vlines.push_back(make_pair(make_pair(stepx, stepy), make_pair(c, make_pair(make_pair(xa[i], ya[i]), i))));
		}
	}
	
	sort(vlines.begin(), vlines.end());

	if (DEBUG) {
		fprintf(stderr, "Sorted vlines: M=%d\n", M);
		for (i = 0; i < M; i++)
			fprintf(stderr, " %d %d %lld %d\n", vlines[i].first.first, vlines[i].first.second, vlines[i].second.first, vlines[i].second.second.second);
	}
	
	stepx = vlines[0].first.first;
	stepy = vlines[0].first.second;
	c = vlines[0].second.first;
	i = vlines[0].second.second.second;
	xaa = xa[i]; yaa = ya[i]; xbb = xb[i]; ybb = yb[i];
	MM = 0;
	
	for (i = 1; i < M; i++) {
		if (vlines[i].first.first == stepx &&
			vlines[i].first.second == stepy &&
			vlines[i].second.first == c) {
			j = vlines[i].second.second.second;
			if (xa[j] > xbb ||
				(xa[j] == xbb && ((stepy >= 0 && ya[j] > ybb) ||
								  (stepy < 0 && ya[j] < ybb)))) {
				xam[MM] = xaa; yam[MM] = yaa; xbm[MM] = xbb; ybm[MM] = ybb;
				MM++;
				xaa = xa[j]; yaa = ya[j]; xbb = xb[j]; ybb = yb[j];
		    } else {
				if (xb[j] > xbb ||
					(xb[j] == xbb && ((stepy >= 0 && yb[j] > ybb) ||
									  (stepy < 0 && yb[j] < ybb)))) {
					xbb = xb[j];
					ybb = yb[j];
				}
			}
		} else {
			xam[MM] = xaa; yam[MM] = yaa; xbm[MM] = xbb; ybm[MM] = ybb;
			MM++;
			stepx = vlines[i].first.first;
			stepy = vlines[i].first.second;
			c = vlines[i].second.first;
			j = vlines[i].second.second.second;
			xaa = xa[j]; yaa = ya[j]; xbb = xb[j]; ybb = yb[j];
		}
	}

	xam[MM] = xaa; yam[MM] = yaa; xbm[MM] = xbb; ybm[MM] = ybb;
	MM++;
	
	M = MM;
	for (i = 0; i < M; i++) {
		xa[i] = xam[i]; ya[i] = yam[i]; xb[i] = xbm[i]; yb[i] = ybm[i];
	}
	
	if (DEBUG) {
		fprintf(stderr, "Compacted lines: M=%d\n", M);
		for (i = 0; i < M; i++)
			fprintf(stderr, "%d: (%d,%d) - (%d,%d)\n", i, xa[i], ya[i], xb[i], yb[i]);
	}
}

int ldx[MMAX], ldy[MMAX], ld[MMAX], lstepx[MMAX], lstepy[MMAX], yymin[MMAX], yymax[MMAX];
int ldxm[MMAX], ldym[MMAX], ldm[MMAX], lstepxm[MMAX], lstepym[MMAX];
long long a[MMAX], b[MMAX], c[MMAX], txym[MMAX], t, u, v, tmp1, tmp2;
int num_points_on_lines, idx, dx1, dx2, dy1, dy2, p, q, ncalls, poz, neg;

inline int IntersectsRectangle(int idx, int xmin, int xmax, int ymin, int ymax) {
	//if (idx == 2)
	//	fprintf(stderr, " !!! a=%lld b=%lld c=%lld %d %d %d %d\n", a[2], b[2], c[2], xmin, xmax, ymin, ymax);

	if (xa[idx] > xmax || xb[idx] < xmin || yymin[idx] > ymax || yymax[idx] < ymin)
		return 0;

	//if (idx == 2)
	//	fprintf(stderr, "*\n");
	
	if (xa[idx] >= xmin && xb[idx] <= xmax && yymin[idx] >= ymin && yymax[idx] <= ymax)
		return 1;
	
	if (ld[idx] == 0)
		return 0;

	poz = neg = 0;

	t = a[idx] * xmin + b[idx] * ymin + c[idx];
	if (t == 0)
		return 1;
	if (t > 0)
		poz = 1;
	else
		neg = 1;

	t = a[idx] * xmin + b[idx] * ymax + c[idx];
	if (t == 0)
		return 1;
	if (t > 0)
		poz = 1;
	else
		neg = 1;

	t = a[idx] * xmax + b[idx] * ymin + c[idx];
	if (t == 0)
		return 1;
	if (t > 0)
		poz = 1;
	else
		neg = 1;
	
	t = a[idx] * xmax + b[idx] * ymax + c[idx];
	if (t == 0)
		return 1;
	if (t > 0)
		poz = 1;
	else
		neg = 1;

	//if (idx == 2)
	//	fprintf(stderr, "  %d %d\n", poz, neg);
	
	if (poz && neg)
		return 1;
	return 0;
}

vector<pair<int, int> > vint;
int xint, yint;

void DecomposeSpace(const vector<int>& vidx, int xmin, int xmax, int ymin, int ymax) {	
	int i, j, xmid, ymid, dx, dy;

	ncalls++;
	
	//if (DEBUG && xmin <= 3 && xmax >= 3 && ymin <= 8 && ymax >= 8) {
	//	fprintf(stderr, "[DecomposeSpace xmin=%d xmax=%d ymin=%d ymax=%d nseg=%u]:", xmin, xmax, ymin, ymax, vidx.size());
	//	for (i = 0; i < vidx.size(); i++)
	//		fprintf(stderr, " %d((%d,%d)-(%d,%d))", vidx[i], xa[vidx[i]], ya[vidx[i]], xb[vidx[i]], yb[vidx[i]]);
	//	fprintf(stderr, "\n");
	//}
	
	if (vidx.size() <= 200) {
		MM = 0;
		
		for (i = 0; i < vidx.size(); i++) {
			idx = vidx[i];

			if (xa[idx] >= xmin || lstepx[idx] == 0)
				dx1 = 0;
			else {
				dx1 = (xmin - xa[idx]) / lstepx[idx];
				while (xa[idx] + dx1 * lstepx[idx] < xmin)
					dx1++;
			}
			
			if (xb[idx] <= xmax || lstepx[idx] == 0)
				dx2 = ld[idx];
			else
				dx2 = (xmax - xa[idx]) / lstepx[idx];
	
			if (lstepy[idx] == 0) {
				dy1 = 0; dy2 = ld[idx];
			} else if (lstepy[idx] > 0) {
				if (ya[idx] >= ymin)
					dy1 = 0;
				else {
					dy1 = (ymin - ya[idx]) / lstepy[idx];
					while (ya[idx] + dy1 * lstepy[idx] < ymin)
						dy1++;
				}
				
				if (yb[idx] <= ymax)
					dy2 = ld[idx];
				else
					dy2 = (ymax - ya[idx]) / lstepy[idx];
			} else {
				if (ya[idx] <= ymax)
					dy1 = 0;
				else {
					dy1 = (ya[idx] - ymax) / lstepy[idx];
					while (ya[idx] + dy1 * lstepy[idx] > ymax)
						dy1++;
				}
				
				if (yb[idx] >= ymin)
					dy2 = ld[idx];
				else {
					dy2 = (ymin - ya[idx]) / lstepy[idx];
					while (ya[idx] + dy2 * lstepy[idx] < ymin)
						dy2--;
				}	
			}
	
			//fprintf(stderr, "idx=%d (xa=%d xb=%d ya=%d yb=%d) | xmin=%d xmax=%d ymin=%d ymax=%d: dx1=%d dx2=%d dy1=%d dy2=%d\n",
			//		idx, xa[idx], xb[idx], ya[idx], yb[idx], xmin, xmax, ymin, ymax, dx1, dx2, dy1, dy2);
			dx1 = max(dx1, dy1);
			dx2 = min(dx2, dy2);
			if (dx1 <= dx2) {
				xam[MM] = xa[idx] + dx1 * lstepx[idx];
				xbm[MM] = xa[idx] + dx2 * lstepx[idx];
				yam[MM] = ya[idx] + dx1 * lstepy[idx];
				ybm[MM] = ya[idx] + dx2 * lstepy[idx];
				ldxm[MM] = xbm[MM] - xam[MM];
				ldym[MM] = ybm[MM] - yam[MM];
				if (ldxm[MM] == 0 && ldym[MM] == 0)
					ldm[MM] = lstepxm[MM] = lstepym[MM] = 0;
				else {
					ldm[MM] = gcd(ldxm[MM], abs(ldym[MM]));
					lstepxm[MM] = ldxm[MM] / ldm[MM];
					lstepym[MM] = ldym[MM] / ldm[MM];
				}
				txym[MM] = (long long) xam[MM] * (long long) lstepym[MM] -
						   (long long) yam[MM] * (long long) lstepxm[MM];
				MM++;
			}
		}

		for (i = 0; i < MM; i++) {
			if (ldm[i] == 0) {
				num_points_on_lines++;
				//for (j = 0; j < i; j++) {
				for (j = i - 1; j >= 0; j--) {
					if (ldxm[j] != 0 || ldym[j] != 0) {
						dx = xam[i] - xam[j];
						dy = yam[i] - yam[j];
						if (((lstepxm[j] == 0 && dx == 0) ||
							 (lstepxm[j] != 0 && (dx % lstepxm[j]) == 0)) &&
							((lstepym[j] == 0 && dy == 0) ||
							 (lstepym[j] != 0 && (dy % lstepym[j]) == 0))) {
							if (lstepxm[j] != 0)
								p = dx / lstepxm[j];
							else
								p = 0;
							if (lstepym[j] != 0)
								q = dy / lstepym[j];
							else q = 0;
							if ((lstepxm[j] == 0 || lstepym[j] == 0 || p == q) && p >= 0 && p <= ldm[j] && q >= 0 && q <= ldm[j]) {
								num_points_on_lines--;
								break;
							}
						}
					}
				}
			} else {
				num_points_on_lines += ldm[i] + 1;			
				vint.clear();
				for (j = i - 1; j >= 0; j--) {
					if (ld[j] == 0) {
						dx = xam[j] - xam[i];
						dy = yam[j] - yam[i];
						if (((lstepxm[i] == 0 && dx == 0) ||
							 (lstepxm[i] != 0 && (dx % lstepxm[i]) == 0)) &&
							((lstepym[i] == 0 && dy == 0) ||
							 (lstepym[i] != 0 && (dy % lstepym[i]) == 0))) {
							if (lstepxm[i] != 0)
								p = dx / lstepxm[i];
							else
								p = 0;
							if (lstepym[i] != 0)
								q = dy / lstepym[i];
							else q = 0;
							if ((lstepxm[i] == 0 || lstepym[i] == 0 || p == q) && p >= 0 && p <= ldm[i] && q >= 0 && q <= ldm[i]) {
								vint.push_back(make_pair(xam[j], yam[j]));
							}
						}
					} else {
						tmp1 = (long long) xam[j] * (long long) lstepym[i] -
							   (long long) yam[j] * (long long) lstepxm[i] -
							   (long long) txym[i];
						tmp2 = (long long) lstepym[j] * (long long) lstepxm[i] -
							   (long long) lstepxm[j] * (long long) lstepym[i];
						if (tmp2 == 0 || (tmp1 % tmp2) != 0)
							continue;
						v = tmp1 / tmp2;
						if (v < 0 || v > ldm[j])
							continue;
						tmp1 = (long long) xam[i] * (long long) lstepym[j] -
							   (long long) yam[i] * (long long) lstepxm[j] -
							   (long long) txym[j];
						tmp2 = (long long) lstepym[i] * (long long) lstepxm[j] -
							   (long long) lstepxm[i] * (long long) lstepym[j];
						if (tmp2 == 0 || (tmp1 % tmp2) != 0)
							continue;
						u = tmp1 / tmp2;
						if (u < 0 || u > ldm[i])
							continue;
						p = (int) u;
						xint = xam[i] + p * lstepxm[i];
						yint = yam[i] + p * lstepym[i];
						vint.push_back(make_pair(xint, yint));	
					}
				}

				if (vint.size() > 0) {
					sort(vint.begin(), vint.end());
					num_points_on_lines--;
					for (j = 1; j < vint.size(); j++)
						if (vint[j].first != vint[j - 1].first ||
							vint[j].second != vint[j - 1].second)
							num_points_on_lines--;
				}
			}
		}

		return;
	}

	if (xmin == xmax && ymin == ymax) {
		for (i = 0; i < vidx.size(); i++) {
			idx = vidx[i];
			p = xmin - xa[idx];
			q = ymin - ya[idx];
			if (lstepx[idx] == 0 && p != 0)
				continue;
			if (lstepy[idx] == 0 && q != 0)
				continue;
			if (lstepx[idx] != 0) {
				if ((p % lstepx[idx]) != 0)
					continue;
				p /= lstepx[idx];
				if (p < 0 || p > ld[idx])
					continue;
			}
			if (lstepy[idx] != 0) {
				if ((q % lstepy[idx]) != 0)
					continue;
				q /= lstepy[idx];
				if (q < 0 || q > ld[idx])
					continue;
			}
			if (lstepx[idx] != 0 && lstepy[idx] != 0 && p != q)
				continue;
			
			//fprintf(stderr, " [x=%d y=%d] ncand=%u: on line %d\n", xmin, ymin, vidx.size(), idx);
			num_points_on_lines++;
			return;
		}
		return;
	}
	
	vector<int> v[4];

	xmid = (xmin + xmax) >> 1;
	ymid = (ymin + ymax) >> 1;
	
	for (i = 0; i < vidx.size(); i++) {
		if (IntersectsRectangle(vidx[i], xmin, xmid, ymin, ymid))
			v[0].push_back(vidx[i]);
		if (IntersectsRectangle(vidx[i], xmin, xmid, ymid + 1, ymax))
			v[1].push_back(vidx[i]);
		if (IntersectsRectangle(vidx[i], xmid + 1, xmax, ymin, ymid))
			v[2].push_back(vidx[i]);
		if (IntersectsRectangle(vidx[i], xmid + 1, xmax, ymid + 1, ymax))
			v[3].push_back(vidx[i]);
	}

	if (v[0].size() > 0)
		DecomposeSpace(v[0], xmin, xmid, ymin, ymid);
	if (v[1].size() > 0)
		DecomposeSpace(v[1], xmin, xmid, ymid + 1, ymax);
	if (v[2].size() > 0)
		DecomposeSpace(v[2], xmid + 1, xmax, ymin, ymid);
	if (v[3].size() > 0)
		DecomposeSpace(v[3], xmid + 1, xmax, ymid + 1, ymax);
}

void FindPointsOnLines() {
	int i;
	int xrmin, xrmax, yrmin, yrmax;
	long long lld;
	
	num_points_on_lines = 0;
	if (M == 0)
		return;

	for (i = 0; i < M; i++) {
		ldx[i] = xb[i] - xa[i];
		ldy[i] = yb[i] - ya[i];
		yymin[i] = min(ya[i], yb[i]);
		yymax[i] = max(ya[i], yb[i]);

		if (ldx[i] == 0 && ldy[i] == 0)
			ld[i] = lstepx[i] = lstepy[i] = 0;
		else {
			ld[i] = gcd(abs(ldx[i]), abs(ldy[i]));
			lstepx[i] = ldx[i] / ld[i];
			lstepy[i] = ldy[i] / ld[i];
			a[i] = ya[i] - yb[i];
			b[i] = xb[i] - xa[i];
			c[i] = (long long) xa[i] * (long long) yb[i] - (long long) xb[i] * (long long) ya[i]; 
			lld = llgcd((a[i] > 0) ? a[i] : (-a[i]), (b[i] > 0) ? b[i] : (-b[i]));
			lld = llgcd(lld, (c[i] > 0) ? c[i] : (-c[i]));
			a[i] /= lld;
			b[i] /= lld;
			c[i] /= lld;
		}
	}

	xrmin = yrmin = -524288;
	xrmax = yrmax = 524287;
	
	vector<int> vidx;
	for (i = 0; i < M; i++)
		vidx.push_back(i);
	
	ncalls = 0;
	DecomposeSpace(vidx, xrmin, xrmax, yrmin, yrmax);
	//if (DEBUG)
		fprintf(stderr, "Number of points on lines=%d ncalls=%d\n", num_points_on_lines, ncalls);
}

int main() {
	if (DEBUG)
		freopen("x.txt", "r", stdin);
	int T, bt = 0;
	scanf("%d", &T);
	
	while (T--) {
		ReadInput();
		if (M > 8000) {
			//exit(1);
			bt++;
		}
		ProcessPolygon();
		ClipLines();
		CompactOverlappingLines();
		FindPointsOnLines();
		printf("%d %lld\n", num_points_on_polygon + num_points_on_lines, num_points_inside_polygon - num_points_on_lines);
	}

	//if (bt == 5)
	//	return 1;
	
	return 0;
}
