#include <stdio.h>
#include <string.h>
#include <time.h>
#include <algorithm>
#include <vector>
#include <queue>

using namespace std;

#define DEBUG 0
#define NMAX 251
#define INF 1000000000

int X[NMAX], Y[NMAX], R[NMAX];
int W;
long long D;
int C[NMAX];
int N, M, i, j, k, p, q, cost;

vector<pair<long long, int> > v;
int Rstk[NMAX], Rtmp;
int Cstk[NMAX], Ctmp, niv;

void ProcessDiskTypes() {
	v.clear();
	for (i = 1; i <= M; i++)
		v.push_back(make_pair(R[i], -C[i]));
	sort(v.begin(), v.end());
	niv = 0;
	
	for (i = 0; i < M; i++) {
		Rtmp = v[i].first;
		Ctmp = -v[i].second;
		while (niv > 0 && Cstk[niv] >= Ctmp)
			niv--;
		niv++;
		Rstk[niv] = Rtmp;
		Cstk[niv] = Ctmp;
	}

	M = niv;
	for (i = 1; i <= niv; i++) {
		R[i] = Rstk[i];
		C[i] = Cstk[i];
	}

	if (DEBUG) {
		fprintf(stderr, "M=%d\n", M);
		for (i = 1; i <= M; i++)
			fprintf(stderr, "  R=%lld C=%d\n", R[i], C[i]);
	}
}

int next[NMAX][NMAX][NMAX], oknext[NMAX][NMAX][NMAX], noknext[NMAX][NMAX];

void ConstructGraph() {
	for (i = 1; i <= N; i++) {
		for (j = 1; j <= M; j++)
			next[i][j][i] = j + 1;
		
		for (p = 1; p <= N; p++)
			if (p != i) {
				D = (long long) (X[i] - X[p]) * (long long) (X[i] - X[p]) +
					(long long) (Y[i] - Y[p]) * (long long) (Y[i] - Y[p]);
				for (j = 1, k = M + 1; j <= M; j++) {
					while (k > 1 && (long long) (R[j] + R[k - 1]) * (long long) (R[j] + R[k - 1]) >= D)
						k--;
					next[i][j][p] = k;
				}
			}

		for (j = 1; j <= M; j++) {
			noknext[i][j] = 0;
			for (p = 1; p <= N; p++)
				if (next[i][j][p] <= M) {
					noknext[i][j]++;
					oknext[i][j][noknext[i][j]] = p;
				}
		}
	}
}

#define QMAX 1048576
#define QMOD 1048575

char inq[NMAX][NMAX];
int cmin[NMAX][NMAX], pi[NMAX][NMAX], pj[NMAX][NMAX];
int qi[QMAX], qj[QMAX], li, ls, niter;
priority_queue<pair<int, pair<int, int> > > pq;
pair<int, pair<int, int> > ptmp;

void ShortestPath() {
	//memset(inq, 0, sizeof(inq));
	while (pq.size() > 0)
		pq.pop();
	
	for (/*li = 0, ls = 0,*/ i = 1; i <= N; i++) {
		for (j = 1; j <= M; j++)
			if (Y[i] - R[j] <= 0) {
				cmin[i][j] = C[j];
				//pi[i][j] = pj[i][j] = 0;
				//inq[i][j] = 1;
				//qi[ls] = i;
				//qj[ls] = j;
				//ls++;
				pq.push(make_pair(-C[j], make_pair(i, j)));
				break;
			} else
				cmin[i][j] = INF;
		for (j++; j <= M; j++)
			cmin[i][j] = INF;
	}

	//niter = 0;
	//while (li != ls) {
	while (pq.size() > 0) {
		//niter++;
		//i = qi[li];
		//j = qj[li];
		//inq[i][j] = 0;
		//li = (li + 1) & QMOD;

		//if (inq[pi[i][j]][pj[i][j]])
		//	continue;

		ptmp = pq.top();
		pq.pop();
		i = ptmp.second.first;
		j = ptmp.second.second;

		if (ptmp.first != -cmin[i][j])
			continue;
		
		if (Y[i] + R[j] >= W)
		//	continue;
			break;

		for (k = 1; k <= noknext[i][j]; k++) {
			p = oknext[i][j][k];
			q = next[i][j][p];
			if (p == i)
				cost = C[q] - C[j];
			else
				cost = C[q];
			
			if (cmin[i][j] + cost < cmin[p][q]) {
				cmin[p][q] = cmin[i][j] + cost;
				pi[p][q] = i; pj[p][q] = j;

				//if (!inq[p][q]) {
				//	inq[p][q] = 1;
				//	qi[ls] = p;
				//	qj[ls] = q;
				//	ls = (ls + 1) & QMOD;
				//}
	
				pq.push(make_pair(-cmin[p][q], make_pair(p, q)));
			}
		}
	}
}

int answer;

void FindAnswer() {
	answer = INF;
	for (i = 1; i <= N; i++)
		for (j = 1; j <= M; j++) {
			if (Y[i] + R[j] >= W && cmin[i][j] < answer)
				answer = cmin[i][j];
		}
}

int main() {
	int tstart = clock();

	//freopen("y.txt", "r", stdin);
	int T;
	scanf("%d", &T);

	while (T--) {
		scanf("%d %d %lld", &N, &M, &W);
		for (i = 1; i <= N; i++)
			scanf("%d %d", &X[i], &Y[i]);
		for (i = 1; i <= M; i++)
			scanf("%d %d", &R[i], &C[i]);
		ProcessDiskTypes();
		ConstructGraph();
		ShortestPath();
		FindAnswer();

		if (answer == INF)
			printf("impossible\n");
		else
			printf("%d\n", answer);
		//fprintf(stderr, "Duration so far=%.3lf sec (niter=%d)\n", (double) (clock() - tstart) / CLOCKS_PER_SEC, niter);
	}

	return 0;
}
