#include <stdio.h>
#include <string.h>
#include <algorithm>
#include <vector>
#include <set>

using namespace std;

#define DEBUG 0

#define LOGMAX 19

int bit[LOGMAX];

void ComputeBits() {
	int i;

	bit[0] = 1;
	for (i = 1; i < LOGMAX; i++)
		bit[i] = bit[i - 1] * 2;
}

#define LMAX 211111

char A[LMAX], B[LMAX];
int LA, LB;

#define NMAX 411111
char S[NMAX];
int LS, L;

void ComputeABUnion() {
	int i;

	LS = 0;
	for (i = 1; i <= LA; i++) {
		LS++;
		S[LS] = A[i];
	}

	LS++;
	S[LS] = 126;

	for (i = 1; i <= LB; i++) {
		LS++;
		S[LS] = B[i];
	}
}

int group[LOGMAX + 1][NMAX], vs[NMAX], nbefore[NMAX];
vector<int> g[NMAX], gtmp[NMAX];

void SuffixArray(char S[], int LS) {
	int i, j, r, ng = 0;
	char c;

	for (i = 0; i <= LS; i++)
		g[i].clear();
	
	for (i = 1; i <= LS; i++)
		g[S[i]].push_back(i);

	for (c = 'a'; c <= 126; c++) {
		if (g[c].size() > 0) {
			if (c == ng) {
				for (j = 0; j < g[c].size(); j++)
					group[0][g[c][j]] = ng;
				ng++;
				continue;
			}

			for (j = 0; j < g[c].size(); j++) {
				g[ng].push_back(g[c][j]);
				group[0][g[c][j]] = ng;
			}

			g[c].clear();
			ng++;
		}
	}

	for (r = 0; r < LOGMAX; r++) {
		nbefore[0] = 0;
		for (i = 0; i < ng; i++) {
			nbefore[i + 1] = nbefore[i] + g[i].size();
			for (j = 0; j < g[i].size(); j++) {
				if (g[i][j] + bit[r] <= LS)
					gtmp[group[r][g[i][j] + bit[r]]].push_back(g[i][j]);
				else
					gtmp[ng].push_back(g[i][j]);
				//fprintf(stderr, "r=%d nbefore=%d i=%d/%d elem=%d\n", r, nbefore[i], i, ng, g[i][j]);
			}
			g[i].clear();
		}

		for (i = 0; i <= ng; i++) {
			for (j = 0; j < gtmp[i].size(); j++) {
				vs[nbefore[group[r][gtmp[i][j]]] + 1] = gtmp[i][j];
				nbefore[group[r][gtmp[i][j]]]++;
			}
			gtmp[i].clear();
		}

		ng = 0;
		g[0].push_back(vs[1]);
		group[r + 1][vs[1]] = 0;

		for (i = 2; i <= LS; i++) {
			if (group[r][vs[i]] != group[r][vs[i - 1]] ||
				vs[i] + bit[r] > LS ||
				vs[i - 1] + bit[r] > LS ||
				group[r][vs[i] + bit[r]] != group[r][vs[i - 1] + bit[r]])
				ng++;
			g[ng].push_back(vs[i]);
			group[r + 1][vs[i]] = ng;
		}

		ng++;
	}

	if (DEBUG) {
		fprintf(stderr, "ng=%d LS=%d\nSuffix order:", ng, LS);
		for (i = 0; i < ng; i++)
			for (j = 0; j < g[i].size(); j++)
				fprintf(stderr, " %d", g[i][j]);
		fprintf(stderr, "\n");
	}
}

vector<pair<int, int> > vps;
vector<pair<pair<int, int>, int> > vps2;

void SuffixArray2(char S[], int LS) {
	int i, j, r, ng = 0;
	char c;

	vps.resize(LS);
	for (i = 1; i <= LS; i++) {
		vps[i - 1].first = (int) S[i];
		vps[i - 1].second = i;
	}

	sort(vps.begin(), vps.end());
	ng = 0;
	group[0][vps[0].second] = 0;
	vs[1] = vps[0].second;
	for (i = 1; i < LS; i++) {
		if (vps[i].first != vps[i - 1].first)
			ng++;
		group[0][vps[i].second] = ng;
		vs[i + 1] = vps[i].second;
	}
	ng++;

	vps2.resize(LS);
	for (r = 0; r < LOGMAX; r++) {
		for (i = 1; i <= LS; i++) {
			vps2[i - 1].first.first = group[r][vs[i]];
			j = vs[i] + bit[r];
			if (j <= LS)
				vps2[i - 1].first.second = group[r][j];
			else
				vps2[i - 1].first.second = ng;
			vps2[i - 1].second = vs[i];
		}

		sort(vps2.begin(), vps2.end());
		for (i = 0; i < LS; i++)
			vs[i + 1] = vps2[i].second;

		ng = 0;
		group[r + 1][vs[1]] = 0;

		for (i = 2; i <= LS; i++) {
			if (group[r][vs[i]] != group[r][vs[i - 1]] ||
				vs[i] + bit[r] > LS ||
				vs[i - 1] + bit[r] > LS ||
				group[r][vs[i] + bit[r]] != group[r][vs[i - 1] + bit[r]])
				ng++;
			group[r + 1][vs[i]] = ng;
		}

		ng++;
	}
}

int lcp[NMAX];

void ComputeLCPs(int LS) {
	int i, j, p, q;

	for (i = 2; i <= LS; i++) {
		p = vs[i];
		q = vs[i - 1];
		lcp[i] = 0;

		for (j = LOGMAX - 1; j >= 0 && p <= LS && q <= LS; j--) {
			if (group[j][p] == group[j][q]) {
				lcp[i] += bit[j];
				p += bit[j];
				q += bit[j];
			}
		}
	}

	if (DEBUG) {
		for (i = 2; i <= LS; i++)
			fprintf(stderr, "i=%d: LCP(%d,%d)=%d\n", i, vs[i - 1], vs[i], lcp[i]);
	}
}

long long CNT[NMAX], na[NMAX], nb[NMAX];
int poz126;

void DivideEtImpera(int start, int end) {
	if (start == end)
		return;

	int vmin, pmin, mid, i;
	long long cnta, cntb;

	mid = (start + 1 + end) >> 1;
	vmin = lcp[mid]; pmin = mid;

	for (i = mid - 1; i > start && vmin > 0; i--)
		if (lcp[i] < vmin) {
			vmin = lcp[i];
			pmin = i;
		}

	for (i = mid + 1; i <= end && vmin > 0; i++)
		if (lcp[i] < vmin) {
			vmin = lcp[i];
			pmin = i;
		}

	if (DEBUG)
		fprintf(stderr, "[D&I] start=%d end=%d vmin=%d pmin=%d\n", start, end, vmin, pmin);

	if (vmin > 0) {
		cnta = na[pmin - 1] - na[start - 1];
		cntb = nb[end] - nb[pmin - 1];
		CNT[vmin] += cnta * cntb;

		cnta = na[end] - na[pmin - 1];
		cntb = nb[pmin - 1] - nb[start - 1];
		CNT[vmin] += cnta * cntb;
	}

	DivideEtImpera(start, pmin - 1);
	DivideEtImpera(pmin, end);
}

set<pair<int, int> > mins;
set<pair<int, int> >::iterator minit;
set<int> intv;
set<int>::iterator intvit;

void TraverseSuffixes(char S[], int LS) {
	int i, start, end;
	long long cnta, cntb;

	pair<int, int> pamin;

	for (i = 1, poz126 = LS + 1; i <= LS; i++)
		if (S[i] == 126) {
			poz126 = i;
			break;
		}

	na[0] = nb[0] = 0;
	for (i = 1; i <= LS; i++) {
		na[i] = na[i - 1];
		nb[i] = nb[i - 1];

		if (vs[i] < poz126)
			na[i]++;
		else if (vs[i] > poz126)
			nb[i]++;		
	}

	memset(CNT, 0, sizeof(CNT));
	//DivideEtImpera(1, LS);

	mins.clear();
	for (i = 2; i <= LS; i++)
		mins.insert(make_pair(lcp[i], i));
	
	intv.clear();
	intv.insert(1);
	intv.insert(LS + 1);

	for (minit = mins.begin(); minit != mins.end(); minit++) {
		intvit = intv.lower_bound(minit->second);
		if (intvit == intv.end())
			exit(1);

		end = (*intvit) - 1;
		intvit--;
		start = (*intvit);

		//fprintf(stderr, "%d %d: %d - %d\n", minit->first, minit->second, start, end);

		if (minit->first > 0) {			
			cnta = na[minit->second - 1] - na[start - 1];
			cntb = nb[end] - nb[minit->second - 1];
			CNT[minit->first] += cnta * cntb;
	
			cnta = na[end] - na[minit->second - 1];
			cntb = nb[minit->second - 1] - nb[start - 1];
			CNT[minit->first] += cnta * cntb;
		}

		intv.insert(minit->second);
	}
	
	for (i = LS; i > 1; i--)
		CNT[i - 1] += CNT[i];
}

void Solve(char S[], int LS) {
	if (DEBUG)
		fprintf(stderr, "---\n");
	SuffixArray2(S, LS);
	ComputeLCPs(LS);
	TraverseSuffixes(S, LS);
}

int main() {
	int T, i;

	ComputeBits();

	//freopen("x.txt", "r", stdin);
	scanf("%d ", &T);

	while (T--) {
		fgets(A + 1, LMAX - 2, stdin);
		fgets(B + 1, LMAX - 2, stdin);
		
		LA = strlen(A + 1);
		while (A[LA] < 'a' || A[LA] > 'z')
			LA--;
	
		LB = strlen(B + 1);
		while (B[LB] < 'a' || B[LB] > 'z')
			LB--;

		scanf("%d ", &L);

		ComputeABUnion();
		Solve(S, LS);

		for (i = 1; i <= L; i++)
			printf("%lld ", CNT[i]);
		printf("\n");
	}

	return 0;
}
