#include <stdio.h>
#include <string.h>
#include <time.h>
#include <algorithm>
#include <vector>

using namespace std;

#define USE_FILES 0
#define DEBUG 0
#define GENRAND 0
#define QRAND 1000000
#define CHRAND 2
#define RSEED 12345
#define USE_BRUTE_FORCE 0
#define NMAX 1048576
#define LOGMAX 21
#define MOD 1000000007
#define QSIZE 10000000

char S[NMAX], op[NMAX];
int i, Q, L;
int l, j, nactsuff, pmin, pmax, ans, cans;
long long sumlcp, sumactsuff;
int bit[LOGMAX];

void ComputeBits() {
	int i;

	bit[0] = 1;
	for (i = 1; i < LOGMAX; i++)
		bit[i] = bit[i - 1] * 2;
}

int group[LOGMAX + 1][NMAX], vs[NMAX], vsrev[NMAX];
vector<pair<int, int> > vps;
vector<pair<pair<int, int>, int> > vps2;

void SuffixArray() {
	int i, j, r, ng = 0;

	vps.resize(L);
	for (i = 1; i <= L; i++) {
		vps[i - 1].first = (int) S[i];
		vps[i - 1].second = i;
	}

	sort(vps.begin(), vps.end());
	ng = 0;
	group[0][vps[0].second] = 0;
	vs[1] = vps[0].second;
	for (i = 1; i < L; i++) {
		if (vps[i].first != vps[i - 1].first)
			ng++;
		group[0][vps[i].second] = ng;
		vs[i + 1] = vps[i].second;
	}
	ng++;

	vps2.resize(L);
	for (r = 0; r < LOGMAX; r++) {
		for (i = 1; i <= L; i++) {
			vps2[i - 1].first.first = group[r][vs[i]];
			j = vs[i] + bit[r];
			if (j <= L)
				vps2[i - 1].first.second = group[r][j];
			else
				vps2[i - 1].first.second = ng;
			vps2[i - 1].second = vs[i];
		}

		sort(vps2.begin(), vps2.end());
		for (i = 0; i < L; i++)
			vs[i + 1] = vps2[i].second;

		ng = 0;
		group[r + 1][vs[1]] = 0;

		for (i = 2; i <= L; i++) {
			if (group[r][vs[i]] != group[r][vs[i - 1]] ||
				vs[i] + bit[r] > L ||
				vs[i - 1] + bit[r] > L ||
				group[r][vs[i] + bit[r]] != group[r][vs[i - 1] + bit[r]])
				ng++;
			group[r + 1][vs[i]] = ng;
		}

		ng++;
	}

	for (i = 1; i <= L; i++)
		vsrev[vs[i]] = i;

	if (DEBUG) {
		fprintf(stderr, "vsrev:\n");
		for (i = 1; i <= L; i++)
			fprintf(stderr, "  i=%d: vsrev=%d\n", i, vsrev[i]);
	}
}

int lcp[NMAX];

void ComputeLCPs() {
	int i, j, p, q;

	for (i = 2; i <= L; i++) {
		p = vs[i];
		q = vs[i - 1];
		lcp[i - 1] = 0;

		for (j = LOGMAX - 1; j >= 0 && p <= L && q <= L; j--) {
			if (group[j][p] == group[j][q]) {
				lcp[i - 1] += bit[j];
				p += bit[j];
				q += bit[j];
			}
		}
	}

	if (DEBUG) {
		S[L + 1] = 0;
		fprintf(stderr, "S=%s\n", S + 1);
		for (i = 2; i <= L; i++)
			fprintf(stderr, "i=%d: %c LCP(%d,%d)=%d\n", i - 1, S[i - 1], vs[i - 1], vs[i], lcp[i - 1]);
	}
}

void ReadInput() {
	int cl = 0;

	if (USE_FILES && !GENRAND)
		freopen("x.txt", "r", stdin);
	if (!GENRAND) {
		scanf("%d", &Q);
		//if (Q > 10000)
		//	exit(7);
	} else {
		Q = QRAND;
		srand(RSEED);
	}

	for (i = 1, L = 0; i <= Q; i++) {
		if (!GENRAND)
			scanf(" %c", &(op[i]));
		else {
			if (cl <= 1000000 || (rand() % 2 == 0))
				op[i] = '+';
			else
				op[i] = '-';
		}
	
		if (op[i] == '+') {
			L++;
			cl++;
			if (!GENRAND)
				scanf(" %c", &(S[L]));
			else
				S[L] = 'a' + (rand() % CHRAND);
		} else
			cl--;
	}
}

void ReverseS() {
	int i;
	char c;

	for (i = 1; i <= L / 2; i++) {
		c = S[i];
		S[i] = S[L - i + 1];
		S[L - i + 1] = c;
	}
}

#define MAX_TRIE_NODES 100000
#define MAX_CHILDREN 26
int next[MAX_TRIE_NODES][MAX_CHILDREN];
int nnodes, node;

int BruteForce() {
	//memset(next, 0, sizeof(next));

	nnodes = 1;
	memset(&(next[1][0]), 0, MAX_CHILDREN * sizeof(int));
	
	for (i = pmin; i <= pmax; i++) {
		node = 1;
		for (j = i; j <= pmax; j++) {
			if (next[node][S[j] - 'a'] == 0) {
				nnodes++;
				if (nnodes >= MAX_TRIE_NODES) {
					//fprintf(stderr, "too many trie nodes!\n");
					//exit(88);
					return -1;
				}
				memset(&(next[nnodes][0]), 0, MAX_CHILDREN * sizeof(int));
				next[node][S[j] - 'a'] = nnodes;
			}
			node = next[node][S[j] - 'a'];
		}
	}

	return nnodes - 1;
}

int li[2 * NMAX], ls[2 * NMAX];

void InitTree() {
	int i, lson;
	li[1] = 1; ls[1] = NMAX;
	for (i = 1; i < NMAX; i++) {
		lson = i << 1;
		li[lson] = li[i];
		ls[lson] = (li[i] + ls[i]) >> 1;
		li[lson + 1] = ls[lson] + 1;
		ls[lson + 1] = ls[i];
	}
}

int cnt[2 * NMAX];

int rmq[2][NMAX][LOGMAX], logidx[NMAX];

void PrecomputeRMQ() {
	int i, j;

	for (i = 1; i < L; i++) {
		rmq[0][i][0] = rmq[1][i][0] = lcp[i];
		for (j = 1; j < LOGMAX && i - bit[j] >= 0; j++) {
			rmq[0][i][j] = rmq[0][i - bit[j - 1]][j - 1];
			if (rmq[0][i][j - 1] < rmq[0][i][j])
				rmq[0][i][j] = rmq[0][i][j - 1];
		}
	}

	for (i = L - 1; i >= 1; i--) {
		for (j = 1; j < LOGMAX && i + bit[j] <= L; j++) {
			rmq[1][i][j] = rmq[1][i + bit[j - 1]][j - 1];
			if (rmq[1][i][j - 1] < rmq[1][i][j])
				rmq[1][i][j] = rmq[1][i][j - 1];
		}
	}

	logidx[1] = 0;
	for (i = 2, j = 1; i < L; i++) {
		if (i == bit[j + 1])
			j++;
		logidx[i] = j;
	}
}

int qans, qmin, qmax;
void QueryMinRMQ() {
	int idx = logidx[qmax - qmin + 1];
	qans = rmq[0][qmax][idx];
	if (rmq[1][qmin][idx] < qans)
		qans = rmq[1][qmin][idx];
}

int minlcp[2 * NMAX];

void ComputeMinLCP(int node) {
	if (node >= NMAX)
		minlcp[node] = lcp[node - NMAX + 1];
	else {
		int lson = node << 1;
		ComputeMinLCP(lson);
		ComputeMinLCP(lson + 1);
		minlcp[node] = minlcp[lson];
		if (minlcp[lson + 1] < minlcp[node])
			minlcp[node] = minlcp[lson + 1];
	}
}

void QueryMinLCP(int node) {
	if (qmin <= li[node] && ls[node] <= qmax) {
		if (minlcp[node] < qans)
			qans = minlcp[node];
		return;
	}

	int lson = node << 1;
	if (qmin <= ls[lson])
		QueryMinLCP(lson);
	if (qmax >= li[lson + 1])
		QueryMinLCP(lson + 1);
}

void UpdateCnt(int idx, int v) {
	sumactsuff += v * vs[idx];
	nactsuff += v;
	idx = NMAX + idx - 1;
	while (idx >= 1) {
		cnt[idx] += v;
		idx >>= 1;
	}
}

int GetPred(int idx) {
	idx = NMAX + idx - 1;
	int p;
	while (idx > 1) {
		p = idx >> 1;
		if ((idx & 1) == 1 && cnt[p] > cnt[idx]) {
			idx--;
			break;
		}
		idx = p;
	}

	if (idx == 1)
		return 0;

	while (idx < NMAX) {
		idx <<= 1;
		if (cnt[idx + 1] >= 1)
			idx++;
	}

	return idx - NMAX + 1;
}

int GetSucc(int idx) {
	idx = NMAX + idx - 1;
	int p;
	while (idx > 1) {
		p = idx >> 1;
		if ((idx & 1) == 0 && cnt[p] > cnt[idx]) {
			idx++;
			break;
		}
		idx = p;
	}

	if (idx == 1)
		return 0;

	while (idx < NMAX) {
		idx <<= 1;
		if (cnt[idx] == 0)
			idx++;
	}

	return idx - NMAX + 1;
}

int qactsuff[QSIZE], nextqactsuff[QSIZE], startactsuff[NMAX], nq;
int idxmax[NMAX];

int UpdateLCPPairState(int s1, int s2, int coef) {
	int idx, result = 2;
	qmin = s1; qmax = s2 - 1;
	QueryMinRMQ();

	if (qans == 0)
		return 0;

	s1 = vs[s1]; s2 = vs[s2];
	if (s1 > s2) {
		idx = s1; s1 = s2; s2 = idx;
		result = 1;
	}
	
	idx = s2 + qans - 1;
	if (idx > pmax)
		return result;

	sumlcp += coef * qans;
	if (coef == 1) {
		if (result == 2)
			qmin = qmax + 1;
		if (idx > idxmax[qmin]) {
			nq++;
			if (nq >= QSIZE) {
				fprintf(stderr, "!! QSIZE exceeded\n");
				exit(1);
			}
			qactsuff[nq] = qmin;
			nextqactsuff[nq] = startactsuff[idx];
			startactsuff[idx] = nq;
			idxmax[qmin] = idx;
		}
	}
	return 0;
}

void UpdateLCPState(int s, int coef) {
	int pred, succ, p, res, removed;

	pred = GetPred(s);
	succ = GetSucc(s);	

	if (DEBUG)
		fprintf(stderr, "  suffix: s=%d pred=%d succ=%d\n", s, pred, succ);

	if (coef == -1) {
		if (pred > 0 && succ > 0)
			UpdateLCPPairState(pred, succ, 1);
		if (pred > 0)
			UpdateLCPPairState(pred, s, -1);
		if (succ > 0)
			UpdateLCPPairState(s, succ, coef);
	} else {
		if (pred > 0 && succ > 0)
			UpdateLCPPairState(pred, succ, -1);

		removed = 0;
		while (pred > 0) {
			res = UpdateLCPPairState(pred, s, 1);
			if (!res)
				break;
			if (res == 1) {
				UpdateCnt(pred, -1);
				p = GetPred(pred);
				if (p > 0)
					UpdateLCPPairState(p, pred, -1);
				pred = p;
			} else {
				removed = 1;
				break;
			}
		}

		if (!removed) {
			while (succ > 0) {
				res = UpdateLCPPairState(s, succ, coef);
				if (!res)
					break;
				if (res == 2) {
					UpdateCnt(succ, -1);
					p = GetSucc(succ);
					if (p > 0)
						UpdateLCPPairState(succ, p, -1);
					succ = p;
				} else {
					removed = 1;
					break;
				}
			}
		}
		
		if (removed) {
			pred = GetPred(s);
			succ = GetSucc(s);
			if (pred > 0 && succ > 0)
				UpdateLCPPairState(pred, succ, 1);
		}
	}
}

void ProcessOperations() {
	int opidx, s, bfans, k;

	sumlcp = sumactsuff = 0;
	l = nactsuff = ans = 0;
	pmin = L + 1;
	pmax = L;

	for (opidx = 1; opidx <= Q; opidx++) {
		if (op[opidx] == '+') {
			pmin--;
			l++;
			UpdateCnt(vsrev[pmin], 1);
			UpdateLCPState(vsrev[pmin], 1);
		} else {
			if (cnt[NMAX + vsrev[pmax] - 1] > 0) {
				UpdateCnt(vsrev[pmax], -1);			
				UpdateLCPState(vsrev[pmax], -1);
			}

			for (k = startactsuff[pmax]; k > 0; k = nextqactsuff[k]) {
				s = qactsuff[k];
				if (cnt[NMAX + s - 1] > 0) {
					UpdateCnt(s, -1);			
					UpdateLCPState(s, -1);
				}
			}

			pmax--;
			l--;
		}

		cans = ((long long) (pmax + 1) * (long long) nactsuff) % MOD;
		cans = (cans + MOD - (sumactsuff % MOD)) % MOD;
		cans = (cans + MOD - (sumlcp % MOD)) % MOD;
		ans = (ans + cans) % MOD;

		if (DEBUG) {
			fprintf(stderr, "## op=%c: pmin=%d pmax=%d sumlcp=%lld nactsuff=%d sumactsuff=%lld cans=%d ans=%d\n", op[opidx], pmin, pmax, sumlcp, nactsuff, sumactsuff, cans, ans);
			fprintf(stderr, "Active suffixes:");
			for (i = 1; i <= L; i++)
				if (cnt[NMAX + i - 1] > 0)
					fprintf(stderr, " %d", i);
			fprintf(stderr, "\n");
		}

		if (USE_BRUTE_FORCE) {
			bfans = BruteForce();
			if (bfans >= 0 && bfans != cans) {
				fprintf(stderr, "!! bfans=%d cans=%d\n", bfans, cans);
				exit(2);
			}
		}
	}

	printf("%d\n", ans);
}

int main() {
	int tstart = clock();

	ReadInput();
	ReverseS();
	ComputeBits();
	SuffixArray();
	ComputeLCPs();
	InitTree();
	//ComputeMinLCP(1);
	PrecomputeRMQ();
	ProcessOperations();

	fprintf(stderr, "Duration=%.3lf sec\n", (double) (clock() - tstart) / CLOCKS_PER_SEC);
	return 0;
}
