#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <algorithm>
#include <vector>

using namespace std;

#define DEBUG 0
#define GENRAND 0
#define INF 1e30
#define eps 1e-7
#define NMAX 111

vector<pair<double, double> > xy;
int N;

void ReadInput() {
	int i;

	if (GENRAND) {
		srand(876);
		N = 100;
	} else {
		//freopen("x.txt", "r", stdin);
		scanf("%d", &N);
	}

	xy.resize(N);
	for (i = 0; i < N; i++) {
		if (GENRAND) {
			xy[i].first = (rand() % 10000) / 10.0;
			xy[i].second = (rand() % 10000) / 10.0;
		} else
			scanf("%lf %lf", &(xy[i].first), &(xy[i].second));
	}
	sort(xy.begin(), xy.end());
}

vector<int> vidx[2];
int s1[NMAX], s2[NMAX];
double xavg[2], yavg[2], answer;

double SolveForSetOfPoints(int sidx) {
	double x2, y2, xyp, dx, dy, b, c, delta, l1, l2;
	int i;

	if (vidx[sidx].size() <= 2)
		return 0.0;

	xavg[sidx] /= vidx[sidx].size();
	yavg[sidx] /= vidx[sidx].size();

	for (i = 0, x2 = y2 = xyp = 0.0; i < vidx[sidx].size(); i++) {
		dx = xy[vidx[sidx][i]].first - xavg[sidx];
		dy = xy[vidx[sidx][i]].second - yavg[sidx];
		x2 += dx * dx;
		y2 += dy * dy;
		xyp += dx * dy;
	}

	b = -x2 - y2;
	c = x2 * y2 - xyp * xyp;
	delta = sqrt(b * b - 4.0 * c);
	return 0.5 * (-b - delta);
/*
	l1 = 0.5 * (-b - delta);
	l2 = 0.5 * (-b + delta);

	if (DEBUG) {
		fprintf(stderr, "### Set of points");
		for (i = 0; i < vidx[sidx].size(); i++)
			fprintf(stderr, " %d(%lf,%lf)", vidx[sidx][i], xy[vidx[sidx][i]].first, xy[vidx[sidx][i]].second);
		fprintf(stderr, ": xavg=%lf yavg=%lf x2=%lf y2=%lf xyp=%lf l1=%lf l2=%lf\n", xavg[sidx], yavg[sidx], x2, y2, xyp, l1, l2);
	}
	
	if (l1 < l2)
		return l1;
	return l2;
*/
}

void SolveForTwoLines() {
	int i;

	vidx[0].clear();
	vidx[1].clear();
	xavg[0] = xavg[1] = yavg[0] = yavg[1] = 0.0;
	
	for (i = 0; i < N; i++) {
		if (s1[i] * s2[i] > 0) {
			vidx[0].push_back(i);
			xavg[0] += xy[i].first;
			yavg[0] += xy[i].second;
		} else {
			vidx[1].push_back(i);
			xavg[1] += xy[i].first;
			yavg[1] += xy[i].second;
		}
	}

	double s = SolveForSetOfPoints(0) + SolveForSetOfPoints(1);
	if (s < answer)
		answer = s;
}

vector<pair<double, int> > c2sort;

void SolveForOneLine(double a1, double b1, double c1, int pozidx, int negidx) {
	double a2, b2, c2, s;
	int i;
	
	for (i = 0; i < N; i++) {
		if (i == pozidx)
			s1[i] = 1;
		else if (i == negidx)
			s1[i] = -1;
		else {
			s = a1 * xy[i].first + b1 * xy[i].second + c1;
			if (s > 0 || fabs(s) < eps)
				s1[i] = 1;
			else
				s1[i] = -1;
		}
	}
	
	a2 = -b1;
	b2 = a1;
	c2sort.clear();
	
	for (i = 0; i < N; i++) {
		c2 = -a2 * xy[i].first - b2 * xy[i].second;
		c2sort.push_back(make_pair(c2, i));
	}

	sort(c2sort.begin(), c2sort.end());
	for (i = 0; i < N; i++)
		s2[i] = 1;

	SolveForTwoLines();
	for (i = 0; i < c2sort.size(); i++) {
		s2[c2sort[i].second] = -1;
		SolveForTwoLines();
	}
}

void Solve() {
	int i, j;
	double a, b, c;

	answer = INF;
	SolveForOneLine(0.0, 1.0, 1.0, -1, -1);
	
	for (i = 0; i < N; i++)
		for (j = i + 1; j < N; j++) {
			a = xy[i].second - xy[j].second;
			b = xy[j].first - xy[i].first;
			c = xy[i].first * xy[j].second - xy[j].first * xy[i].second;
			SolveForOneLine(a, b, c, i, j);
			SolveForOneLine(a, b, c, j, i);
		}
}

void WriteOutput() {
	if (fabs(answer) < eps)
		answer = 0.0;
	printf("%.9lf\n", answer / N);
}

int main() {
	int tstart = clock();

	ReadInput();
	Solve();
	WriteOutput();

	fprintf(stderr, "Duration=%.3lf sec\n", (double) (clock() - tstart) / CLOCKS_PER_SEC);
	return 0;
}
