// CEOI 2013 - Task: Watering - Checker

// Checker to be used by evaluator.
//
// Usage: [checker] [input] [official_output] [contestant_output]
//
// (For testing purposes, [official_output] could be anything
// because it doesn't matter.)

#include <cassert>
#include <cstdlib>
#include <cstring>
#include <fstream>
#include <iostream>
#include <string>

using namespace std;

// official input
int R, S;
char a[605][605];

// contestants data
char b[605][605];

// za flood fill (dfs)
int vrijeme;
int bio[605][605];
int u[] = {0,0,1,-1};
int v[] = {1,-1,0,0};

int flood_fill(int x, int y) {
  if (bio[x][y] == vrijeme) return 0;
  bio[x][y] = vrijeme;
  int ret = 1, nx, ny;
  for (int i = 0; i < 4; ++i) {
    nx = x + u[i];
    ny = y + v[i];
    line:;
    if (nx < 0 || ny < 0 || nx >= R * 6 - 1 || ny >= S * 6 - 1) continue;
    if (b[nx][ny] == b[x][y])
      ret += flood_fill(nx, ny);
    else if (b[nx][ny] == '_') {
      nx += u[i];
      ny += v[i];
      goto line;
    }
  }
  return ret;
}

int main(int argc, char *argv[])
{
  assert(argc == 4);

  ifstream finput(argv[1]);
  ifstream fofficial(argv[2]);
  ifstream foutput(argv[3]);

  assert(!finput.fail() && !foutput.fail());

  // Read official input
  if (!(finput >> R >> S)) {
    cout << 0 << endl << "Error reading input file: cannot read R, C." << endl;
    exit(0);
  }
  for (int i = 0; i < 6 * R - 1; ++i)
    if (!(finput >> a[i])) {
      cout << 0 << endl << "Error reading input file: cannot read line #" << i + 1 << endl;
      exit(0);
    }

  // Read contestant's output
  for (int i = 0; i < 6 * R - 1; ++i) {
    if (!(foutput >> b[i])) {
      cout << 0 << endl << "Error reading output file: cannot read line #" << i + 1 << endl;
      exit(0);
    }
    if (strlen(b[i]) != 6 * S - 1) {
      cout << 0 << endl << "Error reading output file: line #" << i + 1 << " has wrong length." << endl;
      exit(0);
    }
  }
  string crap;
  if (foutput >> crap) {
    cout << 0 << endl << "Error reading output file: too much data." << endl;
    exit(0);
  }

  // Check the solution.
  int broj_srusenih = 0;
  for (int i = 0; i < 6 * R - 1; ++i)
    for (int j = 0; j < 6 * S - 1; ++j)
    {
      if (i % 6 == 5 && j % 6 == 5) {
        if (b[i][j] == '+') continue;
        cout << 0 << endl << "Incorrect: expected '+' at position " << i + 1 << ", " << j + 1 << "." << endl;
        exit(0);
      }
      if (i % 6 == 5) {
        if (b[i][j] == '_') { ++broj_srusenih; continue; }
        if (b[i][j] == '-') continue;
        cout << 0 << endl << "Incorrect: expected '-' or '_' at position " << i + 1 << ", " << j + 1 << "." << endl;
        exit(0);
      }
      if (j % 6 == 5) {
        if (b[i][j] == '_') { ++broj_srusenih; continue; }
        if (b[i][j] == '|') continue;
        cout << 0 << endl << "Incorrect: expected '|' or '_' at position " << i + 1 << ", " << j + 1 << "." << endl;
        exit(0);
      }
      if (b[i][j] == '.') {
        cout << 0 << endl << "Incorrect: board not completely filled." << endl;
        exit(0);
      }
      if (a[i][j] == '#') {
        if (b[i][j] == '#') continue;
        cout << 0 << endl << "Incorrect: expected '#' at position " << i + 1 << ", " << j + 1 << endl;
        exit(0);
      }
      if (b[i][j] < 'a' || b[i][j] > 'z') {
        cout << 0 << endl << "Incorrect: expected letter (a-z) at position " << i + 1 << ", " << j + 1 << endl;
        exit(0);
      }
    }
  for (int i = 0; i < 6 * R - 1; ++i)
    for (int j = 0; j < 6 * S - 1; ++j) {
      if (i % 6 == 5 || j % 6 == 5 || a[i][j] == '#') continue;
      ++vrijeme;
      if (flood_fill(i, j) != 3) {
        cout << 0 << endl << "Incorrect: sprinkler does not water 3 cells near position " << i + 1 << " " << j + 1 << endl;
        exit(0);
      }
    }

  if (broj_srusenih > R * S) {
    cout << 0.5 << endl << "Correct, but number of holes is greater than R * C." << endl;
    exit(0);
  }
  cout << 1 << endl << "Correct!" << endl;
  return 0;
}
