// BEGIN CUT HERE
/*
PROBLEM STATEMENT
You are interested in purchasing items from a number of stores in a 
local market. The market is composed of N stores numbered from 0 to 
N-1. The stores with numbers from 0 to M-1 are interesting to you 
and all the other stores are not interesting. Some pairs of stores 
are connected by roads.

You are given a vector <string> interestingStores which contains M 
elements and describes the interesting stores. The i-th element 
corresponds to store i and is formatted "OPEN CLOSE DURATION" 
(quotes for clarity), where OPEN is the opening time (in seconds), 
CLOSE is the closing time (in seconds) and DURATION is the time (in 
seconds) required to make a purchase in this store. You can initiate 
a purchase from a store at any time T between OPEN and CLOSE, 
inclusive. In order to do so, you need to arrive to the store at 
time T (or earlier). The purchase will be finalized at time T + 
DURATION and you need to stay at the store for the entire duration 
of your purchase. Note that it is possible for a purchase to end 
when the store is already closed. You cannot make multiple purchases 
in the same store.

The roads are given by the vector <string> roads. Each element of 
roads describes a single bidirectional road and is formatted "A B 
LENGTH" (quotes for clarity). Here A and B are the numbers of stores 
connected by the road and LENGTH is the time (in seconds) required 
to move from A to B (or from B to A) using this road.

Your start at time 0 at the location of store N-1. Return the 
maximum number of purchases in interesting stores that you can make.

DEFINITION
Class:TravellingPurchasingMan
Method:maxStores
Parameters:int, vector <string>, vector <string>
Returns:int
Method signature:int maxStores(int N, vector <string> 
interestingStores, vector <string> roads)


NOTES
-You are allowed to wait for any amount of time at any location.


CONSTRAINTS
-N will be between 1 and 50, inclusive.
-roads will contain between 1 and 50 elements, inclusive.
-Each element of roads will be formatted "A B LENGTH" (quotes for 
clarity), where A, B and LENGTH are integers with no unnecessary 
leading zeros.
-In each road, A and B will each be between 0 and N-1, inclusive.
-In each road, A and B will be distinct.
-In each road, LENGTH will be between 1 and 604,800, inclusive.
-There will exist at most one road between each pair of stores.
-interestingStores will contain between 1 and min{16, N} elements, 
inclusive,
-Each element of interestingStores will be formatted "OPEN CLOSE 
DURATION" (quotes for clarity), where OPEN, CLOSE and DURATION are 
integers with no unnecessary leading zeros.
-In each store, OPEN will be between 0 and 604,800, inclusive.
-In each store, CLOSE will be between OPEN+1 and 604,800, inclusive.
-In each store, DURATION will be between 1 and 604,800, inclusive.


EXAMPLES

0)
3
{"1 10 10" , "1 55 31", "10 50 100" }
{"1 2 10"}

Returns: 1

It is not possible to make more than one purchase:
If you decide to make the purchase at store 2: You need to wait 10 
seconds until it opens. Then wait until time = 110 seconds for the 
purchase to finish. At 110 seconds, all the other stores will be 
closed.
If you instead decide to make the purchase at store 1: You first 
need travel through the road and arrive store 1 at time = 10. The 
purchase finishes at time = 41. After you travel back to store 2, 
the time will be 51 seconds and store 2 will be closed.
There is no way to reach store 0 when store 2 is the starting point.


1)
3
{"1 10 10" , "1 55 30", "10 50 100" }
{"1 2 10"}

Returns: 2

This time we can travel to store 1, make the purchase and return to 
store 2 exactly at time = 50 to make two purchases in total.


2)
5
{"0 1000 17"}
{"2 3 400", "4 1 500", "4 3 300", "1 0 700", "0 2 400"}

Returns: 0

It is not possible to reach store 0 before it closes.


*/
// END CUT HERE

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <iostream>
#include <sstream>
#include <queue>
#include <string>
#include <vector>
#include <set>
#include <map>

using namespace std;

#define NMAX 55
#define INF 1000000000

int A[NMAX][NMAX], open[NMAX], cls[NMAX], d[NMAX];
int M;

void RoyFloyd(int N) {
	int i, j, k;
	for (k = 0; k < N; k++)
		for (i = 0; i < N; i++)
			for (j = 0; j < N; j++)
				if (A[i][k] + A[k][j] < A[i][j])
					A[i][j] = A[i][k] + A[k][j];
}

#define MMAX 16

int tmin[1 << MMAX][MMAX];
char inq[1 << MMAX][MMAX];
queue<pair<int, int> > q;

int MaxPath(int N) {
	int i, j, k, conf, answer;

	// Init.
	for (conf = 0; conf < 1 << M; conf++)
		for (i = 0; i < M; i++)
			tmin[conf][i] = INF;

	memset(inq, 0, sizeof(inq));
	for (i = 0; i < M; i++)
		if (A[N - 1][i] <= cls[i]) {
			tmin[1 << i][i] = max(A[N - 1][i], open[i]) + d[i];
			q.push(make_pair(1 << i, i));
			inq[1 << i][i] = 1;
		}

	while (q.size() > 0) {
		pair<int, int> p = q.front();
		q.pop();
		inq[p.first][p.second] = 0;
		
		for (i = 0; i < M; i++)
			if ((p.first & (1 << i)) == 0) {
				conf = p.first | (1 << i);
				j = tmin[p.first][p.second] + A[p.second][i];
				if (j <= cls[i]) {
					j = max(j, open[i]) + d[i];
					if (j < tmin[conf][i]) {
						tmin[conf][i] = j;
						if (!inq[conf][i]) {
							q.push(make_pair(conf, i));
							inq[conf][i] = 1;
						}
					}
				}
			}
	}
	
	// Answer.
	answer = 0;
	for (conf = 0; conf < 1 << M; conf++)
		for (i = 0; i < M; i++)
			if (tmin[conf][i] < INF) {
				j = 0;
				for (k = 0; k < M; k++)
					if ((conf & (1 << k)) > 0)
						j++;
				answer = max(answer, j);
			}

	return answer;
}

class TravellingPurchasingMan { 
	public: 
	int maxStores(int N, vector <string> interestingStores, vector <string> roads)  { 
		int i, j, k, c;

		for (i = 0; i < N; i++)
			for (j = 0; j < N; j++)
				A[i][j] = INF;
		for (i = 0; i < N; i++)
			A[i][i] = 0;

		for (k = 0; k < roads.size(); k++) {
			sscanf(roads[k].c_str(), "%d %d %d", &i, &j, &c);
			A[i][j] = A[j][i] = c;
		}

		M = interestingStores.size();
		for (i = 0; i < M; i++)
			sscanf(interestingStores[i].c_str(), "%d %d %d", &open[i], &cls[i], &d[i]);

		RoyFloyd(N);
		return MaxPath(N);
	}
	
// BEGIN CUT HERE
	public:
	void run_test(int Case) { if ((Case == -1) || (Case == 0)) test_case_0(); if ((Case == -1) || (Case == 1)) test_case_1(); if ((Case == -1) || (Case == 2)) test_case_2(); }
	private:
	template <typename T> string print_array(const vector<T> &V) { ostringstream os; os << "{ "; for (typename vector<T>::const_iterator iter = V.begin(); iter != V.end(); ++iter) os << '\"' << *iter << "\","; os << " }"; return os.str(); }
	void verify_case(int Case, const int &Expected, const int &Received) { cerr << "Test Case #" << Case << "..."; if (Expected == Received) cerr << "PASSED" << endl; else { cerr << "FAILED" << endl; cerr << "\tExpected: \"" << Expected << '\"' << endl; cerr << "\tReceived: \"" << Received << '\"' << endl; } }
	void test_case_0() { int Arg0 = 3; string Arr1[] = {"1 10 10" , "1 55 31", "10 50 100" }; vector <string> Arg1(Arr1, Arr1 + (sizeof(Arr1) / sizeof(Arr1[0]))); string Arr2[] = {"1 2 10"}; vector <string> Arg2(Arr2, Arr2 + (sizeof(Arr2) / sizeof(Arr2[0]))); int Arg3 = 1; verify_case(0, Arg3, maxStores(Arg0, Arg1, Arg2)); }
	void test_case_1() { int Arg0 = 3; string Arr1[] = {"1 10 10" , "1 55 30", "10 50 100" }; vector <string> Arg1(Arr1, Arr1 + (sizeof(Arr1) / sizeof(Arr1[0]))); string Arr2[] = {"1 2 10"}; vector <string> Arg2(Arr2, Arr2 + (sizeof(Arr2) / sizeof(Arr2[0]))); int Arg3 = 2; verify_case(1, Arg3, maxStores(Arg0, Arg1, Arg2)); }
	void test_case_2() { int Arg0 = 5; string Arr1[] = {"0 1000 17"}; vector <string> Arg1(Arr1, Arr1 + (sizeof(Arr1) / sizeof(Arr1[0]))); string Arr2[] = {"2 3 400", "4 1 500", "4 3 300", "1 0 700", "0 2 400"}; vector <string> Arg2(Arr2, Arr2 + (sizeof(Arr2) / sizeof(Arr2[0]))); int Arg3 = 0; verify_case(2, Arg3, maxStores(Arg0, Arg1, Arg2)); }

// END CUT HERE
 
}; 

// BEGIN CUT HERE
int main() {
	TravellingPurchasingMan ___test; 
	___test.run_test(-1); 
	return 0;
} 
// END CUT HERE
