// BEGIN CUT HERE
/*
PROBLEM STATEMENT
This problem statement contains superscripts and/or subscripts. 
These may not display properly outside the applet.

Manao is studying graph theory and simple cycles in particular. A 
simple cycle of length L ? 3 in graph G is a sequence of vertices 
(v0, v1, ..., vL-1) such that all v0, v1, ..., vL-1 are pairwise 
distinct and for each i=0..L-1, an edge between vi and v(i+1) mod L 
exists in G.

Manao is interested in graphs formed by connecting two trees. The 
connection process is as follows. Manao takes two trees composed of 
N vertices each. The vertices in each tree are labeled from 0 to N - 
1. Then, he generates a permutation P of numbers from 0 to N - 1 
uniformly at random. Finally, the graph is formed by connecting 
vertex i of the first tree to vertex P[i] of the second tree, for 
each i from 0 to N - 1. To remove ambiguity, the vertices of the 
first tree within the graph are referred to as A0, A1, ..., AN-1 and 
the vertices of the second graph are referred to as B0, B1, ..., BN-
1. Manao wants to know the expected number of simple cycles of 
length K in the resulting graph.

You are given two vector <string>s, tree1 and tree2. Merge the 
elements of tree1 to obtain a single string formatted as a space-
separated list of N - 1 integers describing the first tree. Let x[i] 
denote the i-th integer (0-based index) in the list. Then, for each 
i, we have 0 ? x[i] ? i and in our tree the vertices x[i] and i+1 
are connected by an edge. tree2 describes the second tree in the 
same fashion.
Compute and return the expected number of simple cycles of length K 
in the graph formed by connecting the two given trees as described 
above. Two simple cycles are equal if one of them can be cyclically 
shifted, or reversed and cyclically shifted, to coincide with the 
second. According to this definition, (1, 2, 3, 4), (2, 3, 4, 1) and 
(3, 2, 1, 4) are all equal.

DEFINITION
Class:TreeUnion
Method:expectedCycles
Parameters:vector <string>, vector <string>, int
Returns:double
Method signature:double expectedCycles(vector <string> tree1, vector 
<string> tree2, int K)


NOTES
-The returned value must have an absolute or relative error less 
than 1e-9.


CONSTRAINTS
-The concatenation of elements of tree1 will be formatted as a space-
separated list of N - 1 integers, where N is between 2 and 300, 
inclusive.
-tree1 will contain between 1 and 50 elements, inclusive.
-Each element of tree1 will be between 1 and 50 characters long, 
inclusive.
-For each i, the i-th integer (0-based index) in the concatenation 
of elements of tree1 will be between 0 and i, inclusive, and will 
have no extra leading zeros.
-The concatenation of elements of tree2 will be formatted as a space-
separated list of N - 1 integers, where N is between 2 and 300, 
inclusive.
-tree2 will contain between 1 and 50 elements, inclusive.
-Each element of tree2 will be between 1 and 50 characters long, 
inclusive.
-For each i, the i-th integer (0-based index) in the concatenation 
of elements of tree2 will be between 0 and i, inclusive, and will 
have no extra leading zeros.
-K will be between 3 and 7, inclusive.


EXAMPLES

0)
{"0"}
{"0"}
4

Returns: 1.0

Manao has two trees with two vertices each. He can connect them in 
two ways:



Either way, the resulting graph is a single cycle of length 4.

1)
{"0 1"}
{"0 1"}
4

Returns: 1.3333333333333333

Manao has two chains composed of three vertices each. There are 6 
possible permutations which result in the following graphs:



Each of the two graphs shown in the topmost row contains two cycles 
of length 4.
(Note that in each case the two cycles share the edge A1B1.)
Each of the other four graphs only contains one cycle of length 4.
Thus the expected number of cycles of length 4 is (2+2+1+1+1+1)/6 = 
8/6 = 1.3333333333.

2)
{"0 1"}
{"0 1"}
6

Returns: 0.3333333333333333

These are the same trees as in the previous example, but this time 
Manao is looking for simple cycles with 6 vertices. Only the topmost 
two graphs contain a cycle of length 6, thus the expected number of 
such cycles for a random permutation P is 1/3.

3)
{"0 ", "1 1 1"}
{"0 1 0 ", "1"}
5

Returns: 4.0

The corresponding trees are these:




4)
{"0 1 2 0 1 2 0 1 2 5 6 1", "0 11", " 4"}
{"0 1 1 0 2 3 4 3 4 6 6", " 10 12 12"}
7

Returns: 13.314285714285713

Do not forget to concatenate the elements of the lists first.

*/
// END CUT HERE

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <set>
#include <map>

using namespace std;

#define NMAX 333

int p1[NMAX], p2[NMAX], l1[NMAX], l2[NMAX];
int M, N;

void GetParents(string s, int p[], int l[], int& N) {
	int i, j, cnt, L, poz;
	l[0] = 0;
	N = 1;
	poz = 0;
	L = s.length();

	while (poz < L) {
		sscanf(s.c_str() + poz, "%d%n", &(p[N]), &cnt);
		poz += cnt;
		l[N] = 1 + l[p[N]];
		N++;
	}
}

int np1[NMAX], np2[NMAX];

void GetPairs(int p[], int l[], int N, int np[]) {
	int i, j, ti, tj, d;
	memset(np, 0, (N + 1) * sizeof(int));

	for (i = 0; i < N; i++)
		for (j = i + 1; j < N; j++) {
			d = 0; ti = i; tj = j;
			while (ti != tj) {
				if (l[ti] >= l[tj]) {
					ti = p[ti];
					d++;
				} else {
					tj = p[tj];
					d++;
				}
			}
			np[d]++;
		}
}

class TreeUnion { 
	public: 
	double expectedCycles(vector <string> tree1, vector <string> tree2, int K)  { 
		string st1 = "", st2 = "";
		int i;
		
		for (i = 0; i < tree1.size(); i++)
			st1 += tree1[i];
		GetParents(st1, p1, l1, M);
		GetPairs(p1, l1, M, np1);

		for (i = 0; i < tree2.size(); i++)
			st2 += tree2[i];
		GetParents(st2, p2, l2, N);
		GetPairs(p2, l2, N, np2);
		
		double answer = 0.0;

		for (i = 1; i < K - 2; i++)
			answer += (double) (2.0 * np1[i] * np2[K - 2 - i]) / (N * (N - 1));
		
		return answer;
	}
	
// BEGIN CUT HERE
	public:
	void run_test(int Case) { if ((Case == -1) || (Case == 0)) test_case_0(); if ((Case == -1) || (Case == 1)) test_case_1(); if ((Case == -1) || (Case == 2)) test_case_2(); if ((Case == -1) || (Case == 3)) test_case_3(); if ((Case == -1) || (Case == 4)) test_case_4(); }
	private:
	template <typename T> string print_array(const vector<T> &V) { ostringstream os; os << "{ "; for (typename vector<T>::const_iterator iter = V.begin(); iter != V.end(); ++iter) os << '\"' << *iter << "\","; os << " }"; return os.str(); }
	void verify_case(int Case, const double &Expected, const double &Received) { cerr << "Test Case #" << Case << "..."; if (Expected == Received) cerr << "PASSED" << endl; else { cerr << "FAILED" << endl; cerr << "\tExpected: \"" << Expected << '\"' << endl; cerr << "\tReceived: \"" << Received << '\"' << endl; } }
	void test_case_0() { string Arr0[] = {"0"}; vector <string> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); string Arr1[] = {"0"}; vector <string> Arg1(Arr1, Arr1 + (sizeof(Arr1) / sizeof(Arr1[0]))); int Arg2 = 4; double Arg3 = 1.0; verify_case(0, Arg3, expectedCycles(Arg0, Arg1, Arg2)); }
	void test_case_1() { string Arr0[] = {"0 1"}; vector <string> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); string Arr1[] = {"0 1"}; vector <string> Arg1(Arr1, Arr1 + (sizeof(Arr1) / sizeof(Arr1[0]))); int Arg2 = 4; double Arg3 = 1.3333333333333333; verify_case(1, Arg3, expectedCycles(Arg0, Arg1, Arg2)); }
	void test_case_2() { string Arr0[] = {"0 1"}; vector <string> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); string Arr1[] = {"0 1"}; vector <string> Arg1(Arr1, Arr1 + (sizeof(Arr1) / sizeof(Arr1[0]))); int Arg2 = 6; double Arg3 = 0.3333333333333333; verify_case(2, Arg3, expectedCycles(Arg0, Arg1, Arg2)); }
	void test_case_3() { string Arr0[] = {"0 ", "1 1 1"}; vector <string> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); string Arr1[] = {"0 1 0 ", "1"}; vector <string> Arg1(Arr1, Arr1 + (sizeof(Arr1) / sizeof(Arr1[0]))); int Arg2 = 5; double Arg3 = 4.0; verify_case(3, Arg3, expectedCycles(Arg0, Arg1, Arg2)); }
	void test_case_4() { string Arr0[] = {"0 1 2 0 1 2 0 1 2 5 6 1", "0 11", " 4"}; vector <string> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); string Arr1[] = {"0 1 1 0 2 3 4 3 4 6 6", " 10 12 12"}; vector <string> Arg1(Arr1, Arr1 + (sizeof(Arr1) / sizeof(Arr1[0]))); int Arg2 = 7; double Arg3 = 13.314285714285713; verify_case(4, Arg3, expectedCycles(Arg0, Arg1, Arg2)); }

// END CUT HERE
 
}; 

// BEGIN CUT HERE
int main() {
	TreeUnion ___test; 
	___test.run_test(-1); 
	return 0;
} 
// END CUT HERE
