// BEGIN CUT HERE
/*
PROBLEM STATEMENT
This problem statement contains superscripts and/or subscripts. 
These may not display properly outside the applet.

Gobble City has a tree topology: There are N intersections, numbered 
0 through N-1. The intersections are connected by N-1 bidirectional 
roads in such a way that it is possible to travel between any two 
intersections. More precisely, for each pair of intersections there 
is a unique path (a sequence of roads; see Notes for a formal 
definition) that connects them. You are given a vector <int> roads 
that describes the roads: for each i (0 <= i <= N-2), we have 0 <= 
roads[i] <= i and there is a road between intersections numbered 
roads[i] and i+1.

On each road there is a single lamp. The lamps are numbered 0 
through N-2. For each i, lamp i is on the road that connects roads
[i] and i+1. You are given a string initState that describes the 
initial states of all lamps: initState[i]='1' means that lamp i is 
initially on, and initState[i]='0' means that it is off.

Now you are at the Control Center for these lamps. The only way in 
which you can operate the lamps looks as follows: You enter the 
numbers of two intersections (X and Y) into the Control Center 
computer, and the computer toggles the state of all lamps on the 
path between X and Y. (Toggling the state of a lamp means that if 
the lamp was off it is now on, and vice versa.) You can perform 
arbitrarily many such operations, one after another.

Some of the lamps are important to you. You are given this 
information in the string isImportant: isImportant[i]='1' means that 
lamp i is important, and isImportant[i]='0' means that it is not.

Your goal is to turn on all important lamps at the same time. (We do 
not care about the state of the remaining lamps.) Return the minimum 
number of operations needed to achieve the goal.

DEFINITION
Class:TurnOnLamps
Method:minimize
Parameters:vector <int>, string, string
Returns:int
Method signature:int minimize(vector <int> roads, string initState, 
string isImportant)


NOTES
-In your solution some lamps may be toggled multiple times.
-A path between two intersection a and b is a sequence of 
intersections (v0=a, v1, v2, ..., vk=b) such that all v0, v1, ..., 
vk are pairwise distinct and for each i=0..k-1, there exists a road 
between vi and vi+1.


CONSTRAINTS
-roads will contain N-1 elements, where N is between 2 and 50, 
inclusive.
-For each i, the i-th (0-based index) element of roads will be 
between 0 and i, inclusive.
-initState will be N-1 characters long.
-initState will only contain the characters '0' and '1'.
-isImportant will be N-1 characters long.
-isImportant will only contain the characters '0' and '1'.


EXAMPLES

0)
{0,0,1,1}
"0001"
"0111"

Returns: 1

The figure below illustrates this test case and its optimal solution.

Red roads are the important ones. The optimal solution chooses the 
intersections 2 and 3. The path that corresponds to this choice is 
enclosed by the dashed line. All lamps on this path are toggled from 
0 to 1.

1)
{0,0,1,1}
"0000"
"0111"

Returns: 2

This is almost the same test case as Example 0, but now the initial 
state of the lamp 3 (i.e., the lamp on the road that connects 
intersections 1 and 4) is now 0. As this is an important lamp, we 
now need two operations in order to turn all important lamps to 1.


2)
{0,0,1,1,4,4}
"000100"
"111111"

Returns: 2


Note that lamp on the road between intersections 1 and 4 was toggled 
twice: first from 1 to 0, and then from 0 back to 1.

3)
{0,0,1,1,4,4}
"100100"
"011101"

Returns: 2



4)
{0,0,2,2,3,1,6,3,1}
"010001110"
"000110100"

Returns: 1



5)
{0,0,1,2,4,4,6,1,2,5,2,8,8,3,6,4,14,7,18,14,11,7,1,12,7,5,18,23,0,14,11,10,2,2,6,1,30,11,9,12,5,35,25,11,23,17,14,45,15}

"0000000000010000000000000010000010100000000000000"
"1010111111111011011111000110111111111111111110111"

Returns: 14



*/
// END CUT HERE

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <set>
#include <map>

using namespace std;

#define NMAX 55
#define INF 1000000000

vector<pair<int, int> > vec[NMAX];
vector<pair<int, int> > sons[NMAX];
char visited[NMAX];
int c[NMAX][2], N;

void DFS(int x) {
	int k, p, y, e, yy, ee, sum, cnt;
	visited[x] = 1;
	
	for (k = 0; k < vec[x].size(); k++) {
		y = vec[x][k].first;
		e = vec[x][k].second;
		
		if (!visited[y]) {
			DFS(y);
			sons[x].push_back(make_pair(y, e));
		}
	}

	sum = cnt = 0;
	for (k = 0; k < sons[x].size(); k++) {
		y = sons[x][k].first;
		e = sons[x][k].second;
		if (e == 1)
			sum += c[y][1];
		else if (e == 0) {
			sum += 1 + min(c[y][0], c[y][1]);
			cnt++;
		} else {
			if (c[y][0] < c[y][1]) {
				sum += 1 + c[y][0];
				cnt++;
			} else {
				sum += c[y][1];
			}
		}
	}
	
	c[x][1] = sum - (cnt / 2);
	c[x][0] = c[x][1];

	for (k = 0; k < sons[x].size(); k++) {
		y = sons[x][k].first;
		e = sons[x][k].second;

		if (e == 1)
			continue;
		
		sum = cnt = 0;
		for (p = 0; p < sons[x].size(); p++)
			if (p != k) {
				yy = sons[x][p].first;
				ee = sons[x][p].second;
				//if (N == 5)
				//fprintf(stderr, " !! x=%d k=%d p=%d ee=%d yy=%d\n", x, k, p, ee, yy);
				if (ee == 1)
					sum += c[yy][1];
				else if (ee == 0) {
					sum += 1 + min(c[yy][0], c[yy][1]);
					cnt++;
				} else {
					if (c[yy][0] < c[yy][1]) {
						sum += 1 + c[yy][0];
						cnt++;
					} else {
						sum += c[yy][1];
					}
				}
			}

		sum -= (cnt / 2);
		sum += c[y][0];
		if (sum < c[x][0]) {
			c[x][0] = sum;
			//if (N == 5)
			//	fprintf(stderr, " ! x=%d: sum=%d cnt=%d\n", x, sum, cnt);
		}
	}

	//if (N == 5)
	//fprintf(stderr, "x=%d: %d %d\n", x, c[x][0], c[x][1]);
}

class TurnOnLamps { 
	public: 
	int minimize(vector <int> roads, string initState, string isImportant)  { 
		int i, j, k, e;
		N = roads.size() + 1;

		for (i = 0; i < N; i++) {
			vec[i].clear();
			sons[i].clear();
		}
	
		for (k = 0; k < N - 1; k++) {
			i = k + 1;
			j = roads[k];
			if (initState[k] == '1' && isImportant[k] == '1')
				e = 1;
			else if (initState[k] == '0' && isImportant[k] == '1')
				e = 0;
			else
				e = -1;

			//fprintf(stderr, "%d-%d %d\n", i, j, e);
			vec[i].push_back(make_pair(j, e));
			vec[j].push_back(make_pair(i, e));
		}

		memset(visited, 0, sizeof(visited));
		DFS(0);
		
		return c[0][1];
	}
	
// BEGIN CUT HERE
	public:
	void run_test(int Case) { if ((Case == -1) || (Case == 0)) test_case_0(); if ((Case == -1) || (Case == 1)) test_case_1(); if ((Case == -1) || (Case == 2)) test_case_2(); if ((Case == -1) || (Case == 3)) test_case_3(); if ((Case == -1) || (Case == 4)) test_case_4(); if ((Case == -1) || (Case == 5)) test_case_5(); }
	private:
	template <typename T> string print_array(const vector<T> &V) { ostringstream os; os << "{ "; for (typename vector<T>::const_iterator iter = V.begin(); iter != V.end(); ++iter) os << '\"' << *iter << "\","; os << " }"; return os.str(); }
	void verify_case(int Case, const int &Expected, const int &Received) { cerr << "Test Case #" << Case << "..."; if (Expected == Received) cerr << "PASSED" << endl; else { cerr << "FAILED" << endl; cerr << "\tExpected: \"" << Expected << '\"' << endl; cerr << "\tReceived: \"" << Received << '\"' << endl; } }
	void test_case_0() { int Arr0[] = {0, 1, 0, 3, 0, 5, 0, 7}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); string Arg1 = "00000000"; string Arg2 = "11111101"; int Arg3 = 1; verify_case(0, Arg3, minimize(Arg0, Arg1, Arg2)); }
	void test_case_1() { int Arr0[] = {0,0,1,1}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); string Arg1 = "0000"; string Arg2 = "0111"; int Arg3 = 2; verify_case(1, Arg3, minimize(Arg0, Arg1, Arg2)); }
	void test_case_2() { int Arr0[] = {0,0,1,1,4,4}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); string Arg1 = "000100"; string Arg2 = "111111"; int Arg3 = 2; verify_case(2, Arg3, minimize(Arg0, Arg1, Arg2)); }
	void test_case_3() { int Arr0[] = {0,0,1,1,4,4}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); string Arg1 = "100100"; string Arg2 = "011101"; int Arg3 = 2; verify_case(3, Arg3, minimize(Arg0, Arg1, Arg2)); }
	void test_case_4() { int Arr0[] = {0,0,2,2,3,1,6,3,1}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); string Arg1 = "010001110"; string Arg2 = "000110100"; int Arg3 = 1; verify_case(4, Arg3, minimize(Arg0, Arg1, Arg2)); }
	void test_case_5() { int Arr0[] = {0,0,1,2,4,4,6,1,2,5,2,8,8,3,6,4,14,7,18,14,11,7,1,12,7,5,18,23,0,14,11,10,2,2,6,1,30,11,9,12,5,35,25,11,23,17,14,45,15}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); string Arg1 = "0000000000010000000000000010000010100000000000000"; string Arg2 = "1010111111111011011111000110111111111111111110111"; int Arg3 = 14; verify_case(5, Arg3, minimize(Arg0, Arg1, Arg2)); }

// END CUT HERE
 
}; 

// BEGIN CUT HERE
int main() {
	TurnOnLamps ___test; 
	___test.run_test(-1); 
	return 0;
} 
// END CUT HERE
