// BEGIN CUT HERE
/*
PROBLEM STATEMENT
Let A be a sequence of integers.
The LISNumber of A is the smallest positive integer L such that A 
can be obtained by concatenating L strictly increasing sequences.
For example, the LISNumber of A = {1, 4, 4, 2, 6, 3} is 4, since we 
can obtain A as {1, 4} + {4} + {2, 6} + {3}, and there is no way to 
create A by concatenating 3 (or fewer) strictly increasing sequences.
The LISNumber of a strictly increasing sequence is 1.

You have N types of cards.
For each i, 0 <= i < N, you have cardsnum[i] cards of the i-th type.
Each card of the i-th type contains the number i.

You are given the vector <int> cardsnum and an int K.
You want to arrange all the cards you have into a row in such a way 
that the resulting sequence of integers has LISNumber K.
Note that you must use all the cards you have, you can only choose 
their order.

Let X be the number of different valid sequences you can produce.
Compute and return the number X, modulo 1,000,000,007.

DEFINITION
Class:LISNumber
Method:count
Parameters:vector <int>, int
Returns:int
Method signature:int count(vector <int> cardsnum, int K)


CONSTRAINTS
-cardsnum will contain between 1 and 36 elements, inclusive.
-Each element of cardsnum will be between 1 and 36, inclusive.
-K will be between 1 and 1296, inclusive.


EXAMPLES

0)
{1, 1, 1}
2

Returns: 4

In this case, there are 3 types of cards and you have one of each. 
Among the 6 sequences you can make, the following 4 have LISNumber 2:

{0, 2, 1}
{1, 0, 2}
{1, 2, 0}
{2, 0, 1}



1)
{2}
1

Returns: 0

The only sequence you can make is {0, 0} and its LISNumber is 2.

2)
{36, 36, 36, 36, 36}
36

Returns: 1

Only the sequence {0, 1, 2, 3, 4, 0, 1, 2, 3, 4, ... (36 times) ... 
} has LISNumber 36.

3)
{3, 2, 11, 5, 7}
20

Returns: 474640725



4)
{31, 4, 15, 9, 26, 5, 35, 8, 9, 7, 9, 32, 3, 8, 4, 6, 26}
58

Returns: 12133719



5)
{27, 18, 28, 18, 28, 4, 5, 9, 4, 5, 23, 5,
 36, 28, 7, 4, 7, 13, 5, 26, 6, 24, 9, 7,
 7, 5, 7, 24, 7, 9, 36, 9, 9, 9, 5, 9}
116

Returns: 516440918



*/
// END CUT HERE

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <set>
#include <map>

using namespace std;

#define MOD 1000000007
#define KMAX 1297
#define CMAX 37

long long nsplit[CMAX][CMAX], comb[KMAX][KMAX];
long long cnt[2][KMAX], tcnt[CMAX], tmp;

class LISNumber { 
	public: 
	int count(vector <int> cardsnum, int K)  {
		int i, j, jj, k, l, p, sum, curr, prev, N = cardsnum.size();
		//int tstart = clock();
		
		memset(nsplit, 0, sizeof(nsplit));
		nsplit[0][0] = 1;
		for (i = 1; i < CMAX; i++) {
			nsplit[i][1] = 1;
			for (j = 2; j <= i; j++)
				for (k = 1; k <= i; k++)
					nsplit[i][j] = (nsplit[i][j] + nsplit[i - k][j - 1]) % MOD;
		}

		memset(comb, 0, sizeof(comb));
		comb[0][0] = comb[1][0] = comb[1][1] = 1;
		for (i = 2; i < KMAX; i++) {
			comb[i][0] = 1;
			for (j = 1; j <= i; j++)
				comb[i][j] = (comb[i - 1][j - 1] + comb[i - 1][j]) % MOD;
		}

		memset(cnt[0], 0, sizeof(cnt[0]));
		cnt[0][0] = 1;

		for (i = N - 1, curr = sum = 0; i >= 0; i--) {
			prev = curr;
			curr = 1 - curr;
			memset(cnt[curr], 0, sizeof(cnt[curr]));

			for (j = 0; j <= K; j++)
				if (cnt[prev][j] > 0) {
					//if (N == 3)
					//	fprintf(stderr, "%d %d %lld : %d\n", i + 1, j, cnt[prev][j], sum + 1 - j);
					memset(tcnt, 0, sizeof(tcnt));
					for (k = 0; k <= sum + 1 - j && k <= cardsnum[i]; k++)
						for (l = k; l <= cardsnum[i]; l++) {
							//if (N == 3 && i == 0)
							//	fprintf(stderr, " -- %d %d %d: %lld %lld\n", sum + 1 - j, k, l, comb[sum + 1 - j][k], nsplit[l][k]);
							tcnt[l] = (tcnt[l] + comb[sum + 1 - j][k] * nsplit[l][k]) % MOD;
						}

					for (k = 0; k <= j && k <= cardsnum[i]; k++)
						for (l = k; l <= cardsnum[i]; l++) {
							tmp = (comb[j][k] * nsplit[l][k]) % MOD;
							//if (N == 3 && i == 0)
							//	fprintf(stderr, "  j=%d k=%d l=%d nsplit[l][k]=%lld comb[j][k]=%lld tcnt[cardsnum[i]-l]=%lld\n", j, k, l, nsplit[l][k], comb[j][k], tcnt[l]);
							jj = j + cardsnum[i] - k;
							tmp = (tmp * tcnt[cardsnum[i] - l]) % MOD;
							tmp = (tmp * cnt[prev][j]) % MOD;
							cnt[curr][jj] = (cnt[curr][jj] + tmp) % MOD;
						}
				}

			sum += cardsnum[i];
		}

		//fprintf(stderr, "duration=%.3lf\n", (double) (clock() - tstart) / CLOCKS_PER_SEC);
		return (int) (cnt[curr][K]);
	}
	
// BEGIN CUT HERE
	public:
	void run_test(int Case) { if ((Case == -1) || (Case == 0)) test_case_0(); if ((Case == -1) || (Case == 1)) test_case_1(); if ((Case == -1) || (Case == 2)) test_case_2(); if ((Case == -1) || (Case == 3)) test_case_3(); if ((Case == -1) || (Case == 4)) test_case_4(); if ((Case == -1) || (Case == 5)) test_case_5(); }
	private:
	template <typename T> string print_array(const vector<T> &V) { ostringstream os; os << "{ "; for (typename vector<T>::const_iterator iter = V.begin(); iter != V.end(); ++iter) os << '\"' << *iter << "\","; os << " }"; return os.str(); }
	void verify_case(int Case, const int &Expected, const int &Received) { cerr << "Test Case #" << Case << "..."; if (Expected == Received) cerr << "PASSED" << endl; else { cerr << "FAILED" << endl; cerr << "\tExpected: \"" << Expected << '\"' << endl; cerr << "\tReceived: \"" << Received << '\"' << endl; } }
	void test_case_0() {
	int Arr0[] = {36}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0])));
	int Arg1 = 35; int Arg2 = 4; verify_case(0, Arg2, count(Arg0, Arg1)); }
	void test_case_1() {
int Arr0[] = {
36, 36, 36, 36, 36, 36, 36, 36, 36,
36, 36, 36, 36, 36, 36, 36, 36, 36,
36, 36, 36, 36, 36, 36, 36, 36, 36,
36, 36, 36, 36, 36, 36, 36, 36, 36,
}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0])));
int Arg1 = 1111; int Arg2 = 0; verify_case(1, Arg2, count(Arg0, Arg1)); }
	void test_case_2() { int Arr0[] = {36, 36, 36, 36, 36}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arg1 = 36; int Arg2 = 1; verify_case(2, Arg2, count(Arg0, Arg1)); }
	void test_case_3() { int Arr0[] = {3, 2, 11, 5, 7}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arg1 = 20; int Arg2 = 474640725; verify_case(3, Arg2, count(Arg0, Arg1)); }
	void test_case_4() { int Arr0[] = {31, 4, 15, 9, 26, 5, 35, 8, 9, 7, 9, 32, 3, 8, 4, 6, 26}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arg1 = 58; int Arg2 = 12133719; verify_case(4, Arg2, count(Arg0, Arg1)); }
	void test_case_5() { int Arr0[] = {27, 18, 28, 18, 28, 4, 5, 9, 4, 5, 23, 5,
 36, 28, 7, 4, 7, 13, 5, 26, 6, 24, 9, 7,
 7, 5, 7, 24, 7, 9, 36, 9, 9, 9, 5, 9}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arg1 = 116; int Arg2 = 516440918; verify_case(5, Arg2, count(Arg0, Arg1)); }

// END CUT HERE
 
}; 

// BEGIN CUT HERE
int main() {
	LISNumber ___test; 
	___test.run_test(-1); 
	return 0;
} 
// END CUT HERE
