// BEGIN CUT HERE
/*
PROBLEM STATEMENT
There are some cities and some roads connecting them together.
The road network has the topology of a perfect binary tree (see 
below for a picture), in which the cities are nodes and the roads 
are edges.

You are given the int treeHeight giving the height of the tree.
(The height of a perfect binary tree is the number of edges on the 
path between the root node and any leaf node.)
Thus, there are 2^(treeHeight+1)-1 cities and 2^(treeHeight+1)-2 
roads in total.

The picture below shows how the road network looks like when 
treeHeight = 2.



We want to send some cars into the road network.
Each car will be traveling from its starting city to its destination 
city without visiting the same city twice.
(Note that the route of each car is uniquely determined by its 
starting and its destination city.)
It is possible for the starting city to be equal to the destination 
city, in that case the car only visits that single city.

Our goal is to send out the cars in such a way that each city will 
be visited by exactly one car.
Let X be the smallest number of cars we need in order to do so.
Compute and return the value X modulo 1,000,000,007.

DEFINITION
Class:TrafficCongestion
Method:theMinCars
Parameters:int
Returns:int
Method signature:int theMinCars(int treeHeight)


CONSTRAINTS
-treeHeight will be between 0 and 1,000,000, inclusive.


EXAMPLES

0)
1

Returns: 1

In this case, one car can visit all the cities.



1)
2

Returns: 3

Here is one way to visit all cities exactly once by three cars:




2)
3

Returns: 5



3)
585858

Returns: 548973404



*/
// END CUT HERE

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <set>
#include <map>

using namespace std;

#define MOD 1000000007
#define NMAX 1100001

int cnt[NMAX], sum, i;

class TrafficCongestion { 
	public: 
	int theMinCars(int N)  {
		cnt[0] = cnt[1] = 1;
		for (i = 2, sum = 1; i <= N; i++) {
			cnt[i] = (1 + 2 * sum) % MOD;
			sum = (sum + cnt[i - 1]) % MOD;
		}

		return cnt[N];
	}

// BEGIN CUT HERE
	public:
	void run_test(int Case) { if ((Case == -1) || (Case == 0)) test_case_0(); if ((Case == -1) || (Case == 1)) test_case_1(); if ((Case == -1) || (Case == 2)) test_case_2(); if ((Case == -1) || (Case == 3)) test_case_3(); }
	private:
	template <typename T> string print_array(const vector<T> &V) { ostringstream os; os << "{ "; for (typename vector<T>::const_iterator iter = V.begin(); iter != V.end(); ++iter) os << '\"' << *iter << "\","; os << " }"; return os.str(); }
	void verify_case(int Case, const int &Expected, const int &Received) { cerr << "Test Case #" << Case << "..."; if (Expected == Received) cerr << "PASSED" << endl; else { cerr << "FAILED" << endl; cerr << "\tExpected: \"" << Expected << '\"' << endl; cerr << "\tReceived: \"" << Received << '\"' << endl; } }
	void test_case_0() { int Arg0 = 1000000; int Arg1 = 1; verify_case(0, Arg1, theMinCars(Arg0)); }
	void test_case_1() { int Arg0 = 2; int Arg1 = 3; verify_case(1, Arg1, theMinCars(Arg0)); }
	void test_case_2() { int Arg0 = 3; int Arg1 = 5; verify_case(2, Arg1, theMinCars(Arg0)); }
	void test_case_3() { int Arg0 = 585858; int Arg1 = 548973404; verify_case(3, Arg1, theMinCars(Arg0)); }

// END CUT HERE
 
}; 

// BEGIN CUT HERE
int main() {
	TrafficCongestion ___test; 
	___test.run_test(-1); 
	return 0;
} 
// END CUT HERE
