// BEGIN CUT HERE
/*
PROBLEM STATEMENT
In this problem, all strings consist of uppercase English letters 
only.
That is, there are 26 distinct letters.

The weight of a string S can be computed as follows: 
for each letter that occurs at least once in S, its leftmost and 
rightmost occurrences L and R are found and the weight is increased 
by R-L.

For example, if S="ABCACAZ", the weight of S is (5-0) + (1-1) + (4-
2) + (6-6) = 7.
(The leftmost occurrence of 'A' is at the index L=0, the rightmost 
one is at R=5, so 'A' contributes 5-0 = 5 to the weight of S. The 
only 'B' contributes 0, the pair of 'C's adds 2, and the only 'Z' 
adds 0.)

A string is S called light if no other string of the same length has 
a smaller weight.

You are given a vector <int> L.
Manao is going to choose some light strings.
The elements of L specify the lengths of these strings.
For example, if L = { 3, 42, 1 }, Manao will first choose a light 
string of length 3, then a light string of length 42, and finally a 
light string of length 1.

Then, Manao is going to concatenate all of the chosen strings, in 
the given order.
Compute and return the smallest possible weight of a string Manao 
may obtain at the end.

DEFINITION
Class:StringWeight
Method:getMinimum
Parameters:vector <int>
Returns:int
Method signature:int getMinimum(vector <int> L)


CONSTRAINTS
-L will contain between 1 and 50 elements, inclusive.
-Each element of L will be between 1 and 100, inclusive.


EXAMPLES

0)
{1}

Returns: 0

Every string of length 1 has weight 0.

1)
{1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
1, 1, 1, 1, 1}

Returns: 1

Manao is going to concatenate 27 strings of length 1. If Manao takes 
25 distinct strings and 2 equal strings and places the equal strings 
side by side, the weight of the resulting string will be 1.

2)
{26, 2, 2}

Returns: 8

One possible concatenation of minimum weight is "ABC...XYZ"+"YZ"+"YZ".

3)
{25, 25, 25, 25}

Returns: 1826



4)
{14, 6, 30, 2, 5, 61}

Returns: 1229



*/
// END CUT HERE

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <set>
#include <map>
#include <algorithm>

using namespace std;

#define NMAX 55
#define CMAX 27
#define INF 1000000000

int wmin[NMAX][CMAX][CMAX], nlet[NMAX];

class StringWeight { 
	public: 
	int getMinimum(vector <int> L)  {
		int i, j, k, sl, N = L.size();

		for (i = 1; i <= N; i++)
			nlet[i] = min(L[i - 1], 26);
		
		for (i = 0; i <= N; i++)
			for (j = 0; j < CMAX; j++)
				for (k = 0; k < CMAX; k++)
					wmin[i][j][k] = INF;

		wmin[0][0][0] = 0;
		sl = 0;

		int nopen, nclosed, nclosenow, nmiddle, nextraopen, nopenclosenow, wc;

		for (i = 1; i <= N; i++) {			
			for (nopen = 0; nopen <= 26; nopen++)
				for (nclosed = 0; nclosed <= nopen; nclosed++)
					if (wmin[i - 1][nopen][nclosed] < INF) {
						for (nclosenow = 0; nclosenow + nclosed <= nopen && nclosenow <= nlet[i]; nclosenow++) {
							nmiddle = nlet[i] - nclosenow;
							if (nmiddle > nopen - nclosed - nclosenow) {
								nextraopen = nmiddle - (nopen - nclosed - nclosenow);
								nmiddle = nlet[i] - nclosenow - nextraopen;
							} else
								nextraopen = 0;
							
							if (nopen + nextraopen > 26)
								continue;
							
							wc = wmin[i - 1][nopen][nclosed] + sl * nclosenow + nclosenow * (nclosenow - 1) / 2;
							wc -= ((sl + L[i - 1]) * nextraopen - nextraopen * (nextraopen + 1) / 2);
							
							if (nmiddle == 0) {
								wc += (L[i - 1] - nclosenow - nextraopen);
							}

							for (nopenclosenow = 0; nopenclosenow <= nextraopen; nopenclosenow++) {
								if (wc < wmin[i][nopen + nextraopen][nclosed + nclosenow + nopenclosenow]) {
									wmin[i][nopen + nextraopen][nclosed + nclosenow + nopenclosenow] = wc;
									if (0 && (nopen == 14 || nopen == 26) && (nclosed == 0))
										fprintf(stderr, "upd %d %d %d: %d => %d %d %d: %d (%d %d %d %d : %d)\n", i - 1, nopen, nclosed, wmin[i - 1][nopen][nclosed],
														i, nopen + nextraopen, nclosed + nclosenow + nopenclosenow, wc,
														nclosenow, nmiddle, nextraopen, nopenclosenow, nlet[i]);
								}
								wc += (sl + L[i - 1] - (nextraopen - nopenclosenow));
							}
						}
					}

			sl += L[i - 1];
		}

		int answer = INF, besti = -1;
		for (i = 0; i <= 26; i++)
			if (wmin[N][i][i] < answer) {
				answer = wmin[N][i][i];
				besti = i;
			}
		//fprintf(stderr, "besti=%d\n", besti);
		return answer;
	}
	
// BEGIN CUT HERE
	public:
	void run_test(int Case) { if ((Case == -1) || (Case == 0)) test_case_0(); if ((Case == -1) || (Case == 1)) test_case_1(); if ((Case == -1) || (Case == 2)) test_case_2(); if ((Case == -1) || (Case == 3)) test_case_3(); if ((Case == -1) || (Case == 4)) test_case_4(); }
	private:
	template <typename T> string print_array(const vector<T> &V) { ostringstream os; os << "{ "; for (typename vector<T>::const_iterator iter = V.begin(); iter != V.end(); ++iter) os << '\"' << *iter << "\","; os << " }"; return os.str(); }
	void verify_case(int Case, const int &Expected, const int &Received) { cerr << "Test Case #" << Case << "..."; if (Expected == Received) cerr << "PASSED" << endl; else { cerr << "FAILED" << endl; cerr << "\tExpected: \"" << Expected << '\"' << endl; cerr << "\tReceived: \"" << Received << '\"' << endl; } }
	void test_case_0() { int Arr0[] = {1}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arg1 = 0; verify_case(0, Arg1, getMinimum(Arg0)); }
	void test_case_1() { int Arr0[] = {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arg1 = 1; verify_case(1, Arg1, getMinimum(Arg0)); }
	void test_case_2() { int Arr0[] = {26, 2, 2}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arg1 = 8; verify_case(2, Arg1, getMinimum(Arg0)); }
	void test_case_3() { int Arr0[] = {25, 25, 25, 25}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arg1 = 1826; verify_case(3, Arg1, getMinimum(Arg0)); }
	void test_case_4() { int Arr0[] = {14, 6, 30, 2, 5, 61}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arg1 = 1229; verify_case(4, Arg1, getMinimum(Arg0)); }

// END CUT HERE
 
}; 

// BEGIN CUT HERE
int main() {
	StringWeight ___test; 
	___test.run_test(-1); 
	return 0;
} 
// END CUT HERE
