// BEGIN CUT HERE
/*
PROBLEM STATEMENT
You have a set of cards.
There is one non-negative integer on each card.
Different cards may contain the same integer.
For each i, the number written on the i-th card (0-based index) is 
number[i].
Your friend wants to select a subset of those cards such that the 
bitwise xor of the selected cards is less than or equal to limit.



You are given the vector<long long> number and the long long limit.
Count the number of ways in which your friend can select the subset 
of cards.
Two subsets count as different if they differ as sets of cards (even 
if the corresponding sets of numbers are the same, see Example 4).

DEFINITION
Class:XorCards
Method:numberOfWays
Parameters:vector<long long>, long long
Returns:long long
Method signature:long long numberOfWays(vector<long long> number, 
long long limit)


NOTES
-XOR (exclusive or) is a binary operation, performed on two numbers 
in binary notation. First, the shorter number is prepended with 
leading zeroes until both numbers have the same number of digits (in 
binary). Then, the result is calculated as follows: for each bit 
where the numbers differ the result has 1 in its binary 
representation. It has 0 in all other positions.
-For example 42 XOR 7 is performed as follows. First, the numbers 
are converted to binary: 42 is 101010 and 7 is 111. Then the shorter 
number is prepended with leading zeros until both numbers have the 
same number of digits. This means 7 becomes 000111. Then 101010 XOR 
000111 = 101101 (the result has ones only in the positions where the 
two numbers differ). Then the result can be converted back to 
decimal notation. In this case 101101 = 45, so 42 XOR 7 = 45.
-If your friend decides to select zero cards, the bitwise xor of 
numbers on selected cards is zero.
-If your friend decides to select a single card, the bitwise xor of 
numbers on selected cards is the number on the selected card.


CONSTRAINTS
-number will contain between 1 and 50 elements, inclusive.
-Each element in number will be between 0 and 1,000,000,000,000,000 
(10^15), inclusive.
-limit will be between 0 and 1,000,000,000,000,000 (10^15), inclusive.


EXAMPLES

0)
{1,2}
2

Returns: 3

This set of cards has four possible subsets.
Here they are, along with the corresponding bitwise xors:
{} => 0, {1} => 1, {2} => 2, and {1,2} => 3.
Note that limit=2.
Out of these four subsets, for the first three the bitwise xor of 
selected numbers is at most equal to limit.

1)
{5,5}
3

Returns: 2

The two good subsets are {} and {5,5}.

2)
{1,2,3,4,5,6,7}
5

Returns: 96



3)
{123, 456, 789, 147, 258, 369, 159, 357}
500

Returns: 125



4)
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
1000000000000000

Returns: 4294967296



5)
{1000000000000000, 999999999999999}
65535

Returns: 2



*/
// END CUT HERE

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <set>
#include <map>

using namespace std;

#define NMAX 55
#define BMAX 63

long long bit[BMAX];

void ComputeBits() {
	bit[0] = 1;
	for (int i = 1; i < BMAX; i++)
		bit[i] = bit[i - 1] << 1;
}

long long A[BMAX];

long long Gauss(const vector<long long>& number, long long result, int minbit) {
	int i, j, k, rank, N = number.size();
	long long tmp;
	
	memset(A, 0, sizeof(A));
	for (i = BMAX - 1; i >= minbit; i--) {
		for (j = 0; j < N; j++)
			if ((number[j] & bit[i]) > 0)
				A[i] |= bit[j];
		if ((result & bit[i]) > 0)
			A[i] |= bit[N];
	}

	for (i = BMAX - 1, rank = k = 0; i >= minbit; i--) {
		while (k < N) {
			for (j = i; j >= minbit; j--)
				if ((A[j] & bit[k]) > 0)
					break;
			if (j < minbit)
				k++;
			else
				break;
		}
		
		if (k >= N)
			break;

		rank++;
		if (j != i) {
			tmp = A[i]; A[i] = A[j]; A[j] = tmp;
		}
		
		for (j = i - 1; j >= minbit; j--)
			if ((A[j] & bit[k]) > 0)
				A[j] ^= A[i];
	}
	
	for (; i >= minbit; i--)
		if (A[i] != 0)
			return 0;

	return bit[N - rank];
}

class XorCards { 
	public: 
	long long numberOfWays(vector<long long> number, long long limit)  { 
		int i, j, k;
		long long num, answer = 0;

		ComputeBits();
		
		for (i = BMAX - 1, num = 0; i >= 0; i--) {
			if ((limit & bit[i]) > 0) {
				answer += Gauss(number, num, i);
				num |= bit[i];
			}
		}

		answer += Gauss(number, limit, 0);
		return answer;
	}
	
// BEGIN CUT HERE
	public:
	void run_test(int Case) { if ((Case == -1) || (Case == 0)) test_case_0(); if ((Case == -1) || (Case == 1)) test_case_1(); if ((Case == -1) || (Case == 2)) test_case_2(); if ((Case == -1) || (Case == 3)) test_case_3(); if ((Case == -1) || (Case == 4)) test_case_4(); if ((Case == -1) || (Case == 5)) test_case_5(); }
	private:
	template <typename T> string print_array(const vector<T> &V) { ostringstream os; os << "{ "; for (typename vector<T>::const_iterator iter = V.begin(); iter != V.end(); ++iter) os << '\"' << *iter << "\","; os << " }"; return os.str(); }
	void verify_case(int Case, const long long &Expected, const long long &Received) { cerr << "Test Case #" << Case << "..."; if (Expected == Received) cerr << "PASSED" << endl; else { cerr << "FAILED" << endl; cerr << "\tExpected: \"" << Expected << '\"' << endl; cerr << "\tReceived: \"" << Received << '\"' << endl; } }
	void test_case_0() { long long Arr0[] = {1,2}; vector<long long> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); long long Arg1 = 2LL; long long Arg2 = 3LL; verify_case(0, Arg2, numberOfWays(Arg0, Arg1)); }
	void test_case_1() { long long Arr0[] = {5,5}; vector<long long> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); long long Arg1 = 3LL; long long Arg2 = 2LL; verify_case(1, Arg2, numberOfWays(Arg0, Arg1)); }
	void test_case_2() { long long Arr0[] = {1,2,3,4,5,6,7}; vector<long long> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); long long Arg1 = 5LL; long long Arg2 = 96LL; verify_case(2, Arg2, numberOfWays(Arg0, Arg1)); }
	void test_case_3() { long long Arr0[] = {123, 456, 789, 147, 258, 369, 159, 357}; vector<long long> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); long long Arg1 = 500LL; long long Arg2 = 125LL; verify_case(3, Arg2, numberOfWays(Arg0, Arg1)); }
	void test_case_4() { long long Arr0[] = {99999999999999LL,99999999999999LL,99999999999999LL,99999999999999LL,99999999999999LL,99999999999999LL,99999999999999LL,99999999999999LL,99999999999999LL,99999999999999LL,99999999999999LL,99999999999999LL,99999999999999LL,99999999999999LL,99999999999999LL,99999999999999LL,99999999999999LL,99999999999999LL,99999999999999LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL,1000000000000000LL}; vector<long long> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); long long Arg1 = 1000000000000000LL; long long Arg2 = 4294967296LL; verify_case(4, Arg2, numberOfWays(Arg0, Arg1)); }
	void test_case_5() { long long Arr0[] = {1000000000000000LL, 999999999999999LL}; vector<long long> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); long long Arg1 = 65535LL; long long Arg2 = 2LL; verify_case(5, Arg2, numberOfWays(Arg0, Arg1)); }

// END CUT HERE
 
}; 

// BEGIN CUT HERE
int main() {
	XorCards ___test; 
	___test.run_test(-1); 
	return 0;
} 
// END CUT HERE
