// BEGIN CUT HERE
/*
PROBLEM STATEMENT
Fox Ciel found two astronomical records.
They both describe the same solar system.
Each planet in the system can be characterized by two parameters 
each being a positive number: planet's size and the distance at 
which it orbits the sun.
All orbital distances are pairwise distinct, but sizes of some 
planets may be the same.



The first record is a vector <int> A with x elements.
These elements give the relative sizes of some x planets in the 
solar system.
Formally, the sizes of those planets have the ratio A[0] : A[1] : 
... : A[x-1].
The planets described by A are ordered by their distance from the sun.
(That is, earlier elements of A correspond to planets that are 
closer to the sun.)



The second record is a vector <int> B with y elements.
These elements give the relative sizes of some y planets in the 
solar system.
Formally, the sizes of those planets have the ratio B[0] : B[1] : 
... : B[y-1].
The planets described by B are ordered by their distance from the sun.



Note that the planets considered by a record do not have to be 
consecutive.
For example, if a solar system contains the planets P, Q, R, S, T, 
and U, it is possible that the first record compares P, R, and S, 
and the second record compares Q, R, T, and U.



We assume that both records are correct.
Return the smallest possible total number of planets in the solar 
system.

DEFINITION
Class:AstronomicalRecords
Method:minimalPlanets
Parameters:vector <int>, vector <int>
Returns:int
Method signature:int minimalPlanets(vector <int> A, vector <int> B)


CONSTRAINTS
-A will contain between 2 and 50 elements, inclusive.
-B will contain between 2 and 50 elements, inclusive.
-Each element in A will be between 1 and 1,000,000,000, inclusive.
-Each element in B will be between 1 and 1,000,000,000, inclusive.


EXAMPLES

0)
{1,2,1,2,1}
{2,1,2,1,2}

Returns: 6

There have to be at least 5 planets, because each record compares 5 
of them.
There cannot be exactly 5 planets, because the first one would have 
to be both smaller than and larger than the second one.
(Their ratio would have to be both 1:2 and 2:1, which is impossible.)
There can be exactly 6 planets with relative sizes 1,2,1,2,1,2.

1)
{1,2,3,4}
{2,4,6,8}

Returns: 4

There can be only 4 planets because 1:2:3:4 = 2:4:6:8.

2)
{2,3,2,3,2,3,2}
{600,700,600,700,600,700,600}

Returns: 10



3)
{1,2,3,4,5,6,7,8,9}
{6,7,8,9,10,11,12}

Returns: 12



4)
{100000000,200000000}
{200000000,100000000}

Returns: 3



*/
// END CUT HERE

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <set>
#include <map>

using namespace std;

#define NMAX 111

long long sa[NMAX], sb[NMAX];
int mc[NMAX][NMAX], M, N;

int MaxCommonPlanets(int p, int q, int dir) {
	int i, j;
	long long x, y;
	
	memset(mc, 0, sizeof(mc));

	if (dir == 1) {
		for (i = p + 1; i <= M; i++)
			for (j = q + 1; j <= N; j++) {
				mc[i][j] = max(mc[i - 1][j], mc[i][j - 1]);
				x = sa[p] * sb[j];
				y = sb[q] * sa[i];
				//if (p == 1 && q == 1 && x == y)
				//	fprintf(stderr, "   i=%d j=%d: %lld %lld\n", i, j, sa[i], sb[j]);
				if (x == y)
					mc[i][j] = max(mc[i][j], mc[i - 1][j - 1] + 1);
			}
		//if (p == 1 && q == 1)
		//	fprintf(stderr, "  mc=%d\n", mc[M][N]);
		return mc[M][N];
	} else {
		for (i = p - 1; i >= 1; i--)
			for (j = q - 1; j >= 1; j--) {
				mc[i][j] = max(mc[i + 1][j], mc[i][j + 1]);
				x = sa[p] * sb[j];
				y = sb[q] * sa[i];
				if (x == y)
					mc[i][j] = max(mc[i][j], mc[i + 1][j + 1] + 1);
			}
		return mc[1][1];
	}
}

class AstronomicalRecords { 
	public: 
	int minimalPlanets(vector <int> A, vector <int> B)  {
		M = A.size(); N = B.size();
		int i, j, p, q, ans, cans;

		for (i = 0; i < M; i++)
			sa[i + 1] = A[i];
		for (j = 0; j < N; j++)
			sb[j + 1] = B[j];
	
		ans = M + N;
		for (p = 1; p <= M; p++)
			for (q = 1; q <= N; q++) {
				// sa[p] and sb[q] are the same planet.
				cans = M + N - 1 - MaxCommonPlanets(p, q, -1) - MaxCommonPlanets(p, q, +1);
				if (cans < ans)
					ans = cans;
			}
	
		return ans;
	}
	
// BEGIN CUT HERE
	public:
	void run_test(int Case) { if ((Case == -1) || (Case == 0)) test_case_0(); if ((Case == -1) || (Case == 1)) test_case_1(); if ((Case == -1) || (Case == 2)) test_case_2(); if ((Case == -1) || (Case == 3)) test_case_3(); if ((Case == -1) || (Case == 4)) test_case_4(); }
	private:
	template <typename T> string print_array(const vector<T> &V) { ostringstream os; os << "{ "; for (typename vector<T>::const_iterator iter = V.begin(); iter != V.end(); ++iter) os << '\"' << *iter << "\","; os << " }"; return os.str(); }
	void verify_case(int Case, const int &Expected, const int &Received) { cerr << "Test Case #" << Case << "..."; if (Expected == Received) cerr << "PASSED" << endl; else { cerr << "FAILED" << endl; cerr << "\tExpected: \"" << Expected << '\"' << endl; cerr << "\tReceived: \"" << Received << '\"' << endl; } }
	void test_case_0() { int Arr0[] = {1,2,1,2,1}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arr1[] = {2,1,2,1,2}; vector <int> Arg1(Arr1, Arr1 + (sizeof(Arr1) / sizeof(Arr1[0]))); int Arg2 = 6; verify_case(0, Arg2, minimalPlanets(Arg0, Arg1)); }
	void test_case_1() { int Arr0[] = {999999999, 999999999, 999999999, 8, 8, 8}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arr1[] = {2, 2, 2}; vector <int> Arg1(Arr1, Arr1 + (sizeof(Arr1) / sizeof(Arr1[0]))); int Arg2 = 4; verify_case(1, Arg2, minimalPlanets(Arg0, Arg1)); }
	void test_case_2() { int Arr0[] = {2,3,2,3,2,3,2}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arr1[] = {600,700,600,700,600,700,600}; vector <int> Arg1(Arr1, Arr1 + (sizeof(Arr1) / sizeof(Arr1[0]))); int Arg2 = 10; verify_case(2, Arg2, minimalPlanets(Arg0, Arg1)); }
	void test_case_3() { int Arr0[] = {1,2,3,4,5,6,7,8,9}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arr1[] = {6,7,8,9,10,11,12}; vector <int> Arg1(Arr1, Arr1 + (sizeof(Arr1) / sizeof(Arr1[0]))); int Arg2 = 12; verify_case(3, Arg2, minimalPlanets(Arg0, Arg1)); }
	void test_case_4() { int Arr0[] = {100000000,200000000}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arr1[] = {200000000,100000000}; vector <int> Arg1(Arr1, Arr1 + (sizeof(Arr1) / sizeof(Arr1[0]))); int Arg2 = 3; verify_case(4, Arg2, minimalPlanets(Arg0, Arg1)); }

// END CUT HERE
 
}; 

// BEGIN CUT HERE
int main() {
	AstronomicalRecords ___test; 
	___test.run_test(-1); 
	return 0;
} 
// END CUT HERE
