#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <string>
#include <vector>

using namespace std;

//#define COMPETITION_MODE

#define INF 1000000000
#define NMAX 128

#define TIME_LIMIT 9.8

int A[NMAX][NMAX];

int delta_row[4] = { -1, 0, +1, 0};
int delta_col[4] = { 0, +1, 0, -1};

class FragileMirrors {
public:
	int N, M, touch_idx, tmp1, tmp2;
	vector<int> result, best_result;
	
	FragileMirrors() {
		touch_idx = 0;
	}
	
	#define BMAX 31
	int bit[BMAX], bmask[BMAX];
	
	void ComputeBits() {
		int i;
		bit[0] = 1;
		for (i = 1; i < BMAX; i++)
			bit[i] = bit[i - 1] << 1;
		
		bmask[0] = ~0;
		for (i = 1; i < BMAX; i++)
			bmask[i] = bmask[i - 1] ^ bit[i - 1];
	}

	int EvaluateRay(int srow, int scol, int drow, int dcol, vector<string>& board, int do_moves) {
		int score = 0, i, j, k, p, nvec;

		touch_idx++;

		srow += drow;
		scol += dcol;
		while (srow >= 0 && srow < N && scol >= 0 && scol < N) {
			if (board[srow][scol] == '-' || A[srow][scol] == touch_idx) {
				A[srow][scol] = touch_idx;
				if (do_moves)
					board[srow][scol] = '-';
				srow += drow;
				scol += dcol;
				continue;
			}
			
			A[srow][scol] = touch_idx;
			score++;

			if (board[srow][scol] == 'L') {
				tmp1 = drow; tmp2 = dcol;
				drow = -tmp2; dcol = -tmp1;
			} else if (board[srow][scol] == 'R') {
				tmp1 = drow; tmp2 = dcol;
				drow = tmp2; dcol = tmp1;
			} else {
				fprintf(stderr, "unexpected character in the board: %c!\n", board[srow][scol]);
				fflush(stderr);
				exit(2);
			}
	
			if (do_moves) {
				board[srow][scol] = '-';
				M--;
			}

			srow += drow;
			scol += dcol;
		}
	
		if (0 && score > 0) {
			for (i = BMAX - 1; i >= 0; i--)
				if ((score & bit[i]) != 0)
					break;
			i -= 2;
			if (i < 0)
				i = 0;
			score &= bmask[i];
		}
		
		return score;
	
		//score = 0;
		for (i = 0; i < N; i++)
			for (j = 0; j < N; j++)
				if (board[i][j] != '-' && A[i][j] != touch_idx) {
					for (k = j + 1; k < N; k++)
						if (board[i][j] != '-' && A[i][j] != touch_idx)
							break;
					
					if (k == N) {
						score--;
						break;
					}
					
					for (p = i + 1; p < N; p++)
						if (board[p][j] != '-' && A[p][j] != touch_idx)
							break;

					if (p == N) {
						score--;
						continue;
					}
					
					if (board[p][k] == '-' || A[p][k] == touch_idx) {
						score--;
						continue;
					}
					
					if (board[i][j] == 'L' && board[i][k] == 'R' &&
						board[p][j] == 'R' && board[p][k] == 'L') {
						score--;
						continue;
					}

					if (board[i][j] == 'R' && board[i][k] == 'L' &&
						board[p][j] == 'L' && board[p][k] == 'R') {
						score--;
						continue;
					}
				}
	
		return score;
	}

	vector<int> bestiv, bestjv, drowv, dcolv;
	
	vector<int> destroy(vector<string> board) {
		int i, j, k, score, best_score, min_score, max_moves, drow, dcol;
		int	tstart = clock(), tend;
		tend = (int) (tstart + TIME_LIMIT * CLOCKS_PER_SEC);

		ComputeBits();
		
		best_result.clear();
		N = board.size();
		max_moves = N;
	
		srand(12345);
	
		while (clock() < tend) {
			vector<string> board_copy;
			for (i = 0; i < N; i++)
				board_copy.push_back(board[i]);
			
			result.clear();
	
			M = N * N;
			while (M > 0) {
				best_score = -INF;
				bestiv.clear();
				bestjv.clear();
				drowv.clear();
				dcolv.clear();

				//if (result.size() < 2 * max_moves) {
				//	min_score = 2 * M / (2 * max_moves - result.size());
				//	if (min_score < N)
				//		min_score = N;
				//} else
					min_score = INF;

				// Top row.
				for (j = 0; j < N; j++) {
					score = EvaluateRay(-1, j, +1, 0, board_copy, 0);
					if (score > best_score && best_score < min_score) {
						best_score = score;
						bestiv.clear();
						bestjv.clear();
						drowv.clear();
						dcolv.clear();
					}
					
					if (score == best_score || score >= min_score) {
						bestiv.push_back(-1);
						bestjv.push_back(j);
						drowv.push_back(+1);
						dcolv.push_back(0);
					}
				}

				// Bottom row.
				for (j = 0; j < N; j++) {
					score = EvaluateRay(N, j, -1, 0, board_copy, 0);
					if (score > best_score && best_score < min_score) {
						best_score = score;
						bestiv.clear();
						bestjv.clear();
						drowv.clear();
						dcolv.clear();
					}

					if (score == best_score || score >= min_score) {
						bestiv.push_back(N);
						bestjv.push_back(j);
						drowv.push_back(-1);
						dcolv.push_back(0);
					}
				}

				// Left column.
				for (i = 0; i < N; i++) {
					score = EvaluateRay(i, -1, 0, +1, board_copy, 0);
					if (score > best_score && best_score < min_score) {
						best_score = score;
						bestiv.clear();
						bestjv.clear();
						drowv.clear();
						dcolv.clear();
					}

					if (score == best_score || score >= min_score) {
						bestiv.push_back(i);
						bestjv.push_back(-1);
						drowv.push_back(0);
						dcolv.push_back(+1);
					}
				}
	
				// Right column.
				for (i = 0; i < N; i++) {
					score = EvaluateRay(i, N, 0, -1, board_copy, 0);
					if (score > best_score && best_score < min_score) {
						best_score = score;
						bestiv.clear();
						bestjv.clear();
						drowv.clear();
						dcolv.clear();
					}

					if (score == best_score || score >= min_score) {
						bestiv.push_back(i);
						bestjv.push_back(N);
						drowv.push_back(0);
						dcolv.push_back(-1);
					}
				}

				if (best_score == -INF) {
					fprintf(stderr, "unexpected best_score!\n");
					fflush(stderr);
					exit(1);
				}

				if (0 && best_score < min_score) {
					fprintf(stderr, "M=%d min_score=%d best_score=%d\n", M, min_score, best_score);
					fflush(stderr);
					break;
				}
			
				k = rand() % bestiv.size();
				
				result.push_back(bestiv[k]);
				result.push_back(bestjv[k]);
	
				if (best_result.size() > 0 && result.size() >= best_result.size())
					break;
				
				EvaluateRay(bestiv[k], bestjv[k], drowv[k], dcolv[k], board_copy, 1);
			}

			if (M == 0 && (best_result.size() == 0 || result.size() < best_result.size())) {
				best_result = result;
				fprintf(stderr, "new best result=%u\n", best_result.size() / 2);
				fflush(stderr);
			}
		}

		return best_result;
	}
};

#ifndef COMPETITION_MODE

char line[1000];

int main() {
	int i, N;
	scanf("%d", &N);

	vector<string> board;
	for (i = 0; i < N; i++) {
		scanf("%s", line);
		board.push_back(line);
	}

	FragileMirrors fm;
	vector<int> result = fm.destroy(board);

	printf("%u\n", result.size());
    for (i = 0; i < result.size(); i++)
		printf("%d\n", result[i]);

	fflush(stdout);
	return 0;
}

#endif
