#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <algorithm>
#include <vector>

using namespace std;

#define TIME_LIMIT 9.80

double TEND;
const int NCAND = 1, NANGLES = 100;

#define pb push_back
#define mp make_pair

//#define VISUALIZER_MODE
//#define LOCAL_MODE

#ifdef LOCAL_MODE
#include <time.h>
#else
#ifdef VISUALIZER_MODE
#include <time.h>
#else
#include <sys/time.h>
#endif
#endif

double GetTime() {
#ifdef LOCAL_MODE
    return ((double) clock() / CLOCKS_PER_SEC);
#else
#ifdef VISUALIZER_MODE
    return ((double) clock() / CLOCKS_PER_SEC);
#else
    timeval t;
    gettimeofday(&t,NULL);
    return t.tv_sec + t.tv_usec * 1e-6;
#endif
#endif
}


#define NUM_LOCAL_TESTS 10

#define USE_GEOM 1

#define NMAX 501
#define eps 1e-8
#define INF 1e9

vector<double> answer, msort;
double best_score, pi, cosu, sinu;
double dx[NMAX], dy[NMAX], xinit[NMAX], yinit[NMAX];
double cx[NMAX], cy[NMAX], r[NMAX], m[NMAX];
char selected[NMAX];
double u, u2, d, dmin, tmp1, tmp2, msum, step = 5e-5, dc, dd;
int N, i;

class CirclesSeparation {
public:
	void UpdateAnswer(const char* msg) {
		double cscore = 0.0;
		int i;
		for (i = 0; i < N; i++)
			cscore += m[i] * sqrt((cx[i] - xinit[i]) * (cx[i] - xinit[i]) +
							      (cy[i] - yinit[i]) * (cy[i] - yinit[i]));
	
		#ifdef LOCAL_MODE
		fprintf(stderr, "%s: %.9lf\n", msg, cscore);
		#endif
		if (cscore < best_score) {
			best_score = cscore;
			answer.clear();
			for (i = 0; i < N; i++) {
				answer.pb(cx[i]);
				answer.pb(cy[i]);
			}
		}
	}

	vector<pair<double, int> > v;
	vector<int> vcand, vsel;
	vector<pair<double, double> > covered;

	void RandomGreedy() {
		int i, j, k, niter = 0, niter2, angle, keep_going, changed;
		double score, dbest, li, ls, finalcx, finalcy, mid, du, dv, drange, TCURR;
		
		srand(12345);

		while (GetTime() < TEND) {
			niter++;
			score = 0.0;

			memset(selected, 0, sizeof(selected));
			vsel.clear();

			for (k = 0; k < N && GetTime() < TEND; k++) {
				v.clear();
				for (i = 0; i < N; i++)
					if (!selected[i])
						v.pb(mp(m[i], i));
				sort(v.begin(), v.end());
				vcand.clear();
				for (i = v.size() - 1; i >= 0 && vcand.size() < NCAND; i--)
					vcand.pb(v[i].second);

				i = vcand[rand() % vcand.size()];
				selected[i] = 1;
				
				// Find the best position for circle i.
				dbest = INF;
				finalcx = xinit[i];
				finalcy = yinit[i];
				
				for (angle = 0, keep_going = 1; angle < NANGLES && keep_going; angle++) {
					u = pi * (rand() % 10000) / 10000.0;
					cosu = cos(u);
					sinu = sin(u);

					if (USE_GEOM) {
						covered.clear();
						for (j = 0; j < k; j++) {
							tmp1 = cy[vsel[j]] - yinit[i];
							tmp2 = cx[vsel[j]] - xinit[i];
							u2 = atan2(tmp1, tmp2);
							while (u2 < u)
								u2 += 2.0 * pi;
							d = sqrt(tmp1 * tmp1 + tmp2 * tmp2);
							du = d * cos(u2 - u);
							dv = fabs(d * sin(u2 - u));
							if (dv >= r[vsel[j]] + r[i] + eps)
								continue;
							drange = sqrt((r[vsel[j]] + r[i] + eps) * (r[vsel[j]] + r[i] + eps) - dv * dv);
							covered.pb(mp(du - drange - eps, du + drange + eps));
							//fprintf(stderr, "  i=%d (%lf,%lf %lf):  angle=%d  j=%d: %lf - %lf\n", i, xinit[i], yinit[i], r[i], angle, vsel[j], du - drange, du + drange);
						}
	
						covered.pb(mp(INF, INF));						
						sort(covered.begin(), covered.end());
	
						du = -INF;
						for (j = 0; j < covered.size(); j++) {
							if (covered[j].first <= du)
								du = max(du, covered[j].second);
							else {
								if (du < 0.0 && covered[j].first > 0.0) {
									dbest = 0.0;
									finalcx = xinit[i];
									finalcy = yinit[i];
									keep_going = 0;
									break;
								}
							
								if (fabs(du) < dbest) {
									dbest = fabs(du);
									finalcx = xinit[i] + du * cosu;
									finalcy = yinit[i] + du * sinu;
								}
								
								if (fabs(covered[j].first) < dbest) {
									dbest = fabs(covered[j].first);
									finalcx = xinit[i] + covered[j].first * cosu;
									finalcy = yinit[i] + covered[j].first * sinu;
								}
								
								du = covered[j].second;
							}
						}
					} else {
						li = ls = 0.0;
						while (li < dbest) {
							cx[i] = xinit[i] + ls * cosu;
							cy[i] = yinit[i] + ls * sinu;
							for (j = 0; j < k; j++) {
								tmp1 = cx[i] - cx[vsel[j]];
								tmp2 = cy[i] - cy[vsel[j]];
								d = tmp1 * tmp1 + tmp2 * tmp2;
								dmin = r[i] + r[vsel[j]];
								dmin *= dmin;
								if (d < dmin + eps)
									break;
							}
							
							if (j == k) {
								if (ls < dbest) {
									dbest = ls;
									finalcx = cx[i];
									finalcy = cy[i];
								}
								break;
							}
		
							li = ls;
							ls += 0.05;
						}
		
						while (fabs(ls - li) >= eps) {
							if (li >= dbest)
								break;
							mid = 0.5 * (li + ls);
							cx[i] = xinit[i] + mid * cosu;
							cy[i] = yinit[i] + mid * sinu;
		
							for (j = 0; j < k; j++) {
								tmp1 = cx[i] - cx[vsel[j]];
								tmp2 = cy[i] - cy[vsel[j]];
								d = tmp1 * tmp1 + tmp2 * tmp2;
								dmin = r[i] + r[vsel[j]];
								dmin *= dmin;
								if (d < dmin + eps)
									break;
							}
							
							if (j == k) {
								if (mid < dbest) {
									dbest = mid;
									finalcx = cx[i];
									finalcy = cy[i];
								}
								ls = mid;
							} else {
								li = mid;
							}
						}
					}
				}
				
				cx[i] = finalcx;
				cy[i] = finalcy;

				// Move the circle closer to the destination if possible.
				while (GetTime() < TEND) {
					u = atan2(yinit[i] - cy[i], xinit[i] - cx[i]);
					dc = sqrt((cx[i] - xinit[i]) * (cx[i] - xinit[i]) +
							  (cy[i] - yinit[i]) * (cy[i] - yinit[i]));

					finalcx = cx[i] + step * cos(u);
					finalcy = cy[i] + step * sin(u);

					changed = 1;
					niter2 = 0;
					while (changed && niter2 < 10) {
						niter2++;
						changed = 0;
						dx[i] = dy[i] = 0;
						for (j = 0; j < k; j++) {
							tmp1 = finalcx - cx[vsel[j]];
							tmp2 = finalcy - cy[vsel[j]];
							d = tmp1 * tmp1 + tmp2 * tmp2;
							tmp1 = r[i] + r[vsel[j]];
							dmin = tmp1 * tmp1;
							if (d < dmin + eps) {
								changed = 1;
								d = tmp1 + eps - sqrt(d);
								if (d < step)
									d = step;
								u = atan2(cy[vsel[j]] - finalcy, cx[vsel[j]] - finalcx);
								dx[i] -= d * cos(u);
								dy[i] -= d * sin(u);
							}
						}
					
						finalcx += dx[i];
						finalcy += dy[i];
					}
					
					if (changed)
						break;
					dd = sqrt((finalcx - xinit[i]) * (finalcx - xinit[i]) +
							  (finalcy - yinit[i]) * (finalcy - yinit[i]));
					if (dd >= dc - eps)
						break;
					cx[i] = finalcx;
					cy[i] = finalcy;
				}
		
				//fprintf(stderr, "  niter=%d: k=%d i=%d dbest=%lf cx=%lf cy=%lf r=%lf\n", niter, k, i, dbest, cx[i], cy[i], r[i]);
				vsel.pb(i);
				score += m[i] * sqrt((cx[i] - xinit[i]) * (cx[i] - xinit[i]) +
							         (cy[i] - yinit[i]) * (cy[i] - yinit[i]));
				if (score >= best_score)
					break;
			}
		
			fprintf(stderr, "  random greedy: niter=%d k=%d/%d score=%.9lf\n", niter, k, N, score);

			if (k == N) {
				UpdateAnswer("random greedy");
				//break;
			}
		}
	}
	
	vector <double> minimumWork(vector <double> x, vector <double> y, vector <double> radius, vector <double> mass) {
		TEND = GetTime() + TIME_LIMIT;

		pi = 2.0 * acos(0.0);
		
		N = x.size();
		best_score = INF;

		for (i = 0; i < N; i++) {
			xinit[i] = x[i];
			yinit[i] = y[i];
			r[i] = radius[i];
			m[i] = mass[i];
		}

		RandomGreedy();
		
		if (answer.size() != 2 * N) {
			fprintf(stderr, "Incorrect answer size!\n");
			exit(1);
		}

		return answer;
	}
};

#ifdef LOCAL_MODE

int main() {
	int test, i, j, N;
	double total_score = 0.0, t, maxR;

	for (test = 1; test <= NUM_LOCAL_TESTS; test++) {
		srand(test * 100);
		t = (double) (rand() % 10001) / 10000.0;
		N = (int) (300 + floor(201.0 * t * t));
		maxR = sqrt(1.0 / N) + ((rand() % 10001) / 10000.0) * (sqrt(5.0 / N) - sqrt(1.0 / N));

		vector<double> x(N), y(N), m(N), r(N);
		for (i = 0; i < N; i++) {
			x[i] = (rand() % 10001) / 10000.0;
			y[i] = (rand() % 10001) / 10000.0;
			m[i] = (rand() % 10001) / 10000.0;
			r[i] = ((rand() % 10001) / 10000.0) * maxR;
		}
	
		fprintf(stderr, "### TEST %d: N=%d (t=%.4lf) maxR=%.4lf\n", test, N, t, maxR);

		CirclesSeparation cs;
		int tstart = clock();
		vector<double> result = cs.minimumWork(x, y, r, m);
		double duration = (double) (clock() - tstart) / CLOCKS_PER_SEC;

		double score = 0.0;
		for (i = 0; i < N; i++) {
			double fx = result[2 * i], fy = result[2 * i + 1];
			if (fx < -99 || fx > 99 || fy < -99 || fy > 99) {
				fprintf(stderr, "Final coordinates are outside the range!");
				exit(1);
			}
			score += m[i] * sqrt((x[i] - fx) * (x[i] - fx) + (y[i] - fy) * (y[i] - fy));
		}

		for (i = 0; i < N; i++) {
			double fxi = result[2 * i], fyi = result[2 * i + 1];
			for (j = i + 1; j < N; j++) {
				double fxj = result[2 * j], fyj = result[2 * j + 1];
				d = sqrt((fxi - fxj) * (fxi - fxj) + (fyi - fyj) * (fyi - fyj));
				dmin = r[i] + r[j];
				if (d < dmin + eps) {
					fprintf(stderr, "Circles %d (%lf,%lf,%lf) and %d (%lf,%lf,%lf) overlap! d=%lf dmax=%lf\n", i, fxi, fyi, r[i], j, fxj, fyj, r[j], d, dmin);
					exit(2);
				}
			}
		}
		
		total_score += score;
		fprintf(stderr, "SCORE=%.9lf TOTAL SCORE=%.9lf Duration=%.3lf sec\n\n", score, total_score, duration);
	}
}
#endif

#ifdef VISUALIZER_MODE
int main() {
	int i, N;
	scanf("%d", &N);

	vector<double> x(N), y(N), r(N), m(N);
	for (i = 0; i < N; i++)
		scanf("%lf", &x[i]);
	for (i = 0; i < N; i++)
		scanf("%lf", &y[i]);
	for (i = 0; i < N; i++)
		scanf("%lf", &r[i]);
	for (i = 0; i < N; i++)
		scanf("%lf", &m[i]);
	CirclesSeparation cs;
	vector<double> ret = cs.minimumWork(x, y, r, m);
	for (i = 0; i < 2 * N; i++)
		printf("%.9lf\n", ret[i]);
	fflush(stdout);
	return 0;
}
#endif
