// BEGIN CUT HERE
/*
PROBLEM STATEMENT
You are given ints n and m: the dimensions of a matrix.
A matrix M with n rows and m columns is called a mountain if it 
satisfies the following conditions:

All elements of M are positive integers.
There are integers a and b with the following properties:
  
  0 <= a <= n-1 and 0 <= b <= m-1.
  For each row i, M[i][0] < M[i][1] < ... < M[i][b-1] < M[i][b] > M
[i][b+1] > ... > M[i][m-1].
  For each column j, M[0][j] < M[1][j] < ... < M[a-1][j] < M[a][j] > 
M[a+1][j] > ... > M[n-1][j].
  



In addition to n and m, you are given some elements of M.
More precisely, you are given the vector <int>s rowIndex, 
columnIndex and element.
For each valid k, M[rowIndex[k]][columnIndex[k]] has to be element[k].

Your task is to fill in the missing values so that M becomes a 
mountain.
Find and return the smallest possible sum of all elements in a 
mountain M that matches the given constraints.
If there is no mountain that matches the given constraints, return 
-1 instead.

DEFINITION
Class:TheMountain
Method:minSum
Parameters:int, int, vector <int>, vector <int>, vector <int>
Returns:int
Method signature:int minSum(int n, int m, vector <int> rowIndex, 
vector <int> columnIndex, vector <int> element)


CONSTRAINTS
-n and m will each be between 1 and 200, inclusive.
-rowIndex will contain between 1 and min(n*m, 50) elements, inclusive.
-columnIndex and element will contain the same number of elements as 
rowIndex.
-Each element of rowIndex will be between 0 and n-1, inclusive.
-Each element of columnIndex will be between 0 and m-1, inclusive.
-Each element of element will be between 1 and 1,000, inclusive.
-No two indices represented by rowIndex and columnIndex will be the 
same.


EXAMPLES

0)
2
3
{0, 0, 0, 1, 1, 1}
{0, 1, 2, 0, 1, 2}
{4, 6, 9, 1, 3, 6}

Returns: 29

We are already given all elements of this matrix.
It looks as follows:

[4 6 9]
[1 3 6]

We can easily verify that this is a mountain for a=0 and b=2.
The sum of all elements is 4+6+9+1+3+6=29.

1)
2
3
{1, 0, 1}
{2, 2, 0}
{5, 7, 6}

Returns: 40

The matrix looks as follows.

[? ? 7]
[6 ? 5]

The optimal solution is as follows. The sum is 7+8+7+6+7+5=40.

[7 8 7]
[6 7 5]


2)
3
3
{0, 0, 2, 2}
{0, 2, 2, 0}
{1, 1, 1, 1}

Returns: 15

The matrix looks as follows:

[1 ? 1]
[? ? ?]
[1 ? 1]

The optimal solution is as follows.

[1 2 1]
[2 3 2]
[1 2 1]


3)
2
2
{0, 0, 1, 1}
{0, 1, 1, 0}
{5, 8, 5, 8}

Returns: -1

The given matrix is not a mountain.

[5 8]
[8 5]


4)
1
3
{0}
{1}
{1}

Returns: -1

This matrix looks as follows:

[? 1 ?]

It is not possible to make a mountain out of it, because in a 
mountain all elements have to be positive.
(Note that 0 is not positive.)

5)
123
45
{2, 3, 5, 7, 11}
{13, 17, 19, 23, 29}
{100, 200, 300, 400, 500}

Returns: 367047



6)
200
200
{5}
{8}
{666}

Returns: 5737554



7)
10
10
{0, 8, 7}
{3, 1, 9}
{5, 4, 7}

Returns: 593



*/
// END CUT HERE

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <set>
#include <map>

using namespace std;

#define NMAX 222

int A[NMAX][NMAX], fixed_pos[NMAX][NMAX];
int AA[4][NMAX][NMAX], solpos[4][NMAX][NMAX], S[4][NMAX][NMAX];

class TheMountain { 
	public: 
	int minSum(int n, int m, vector <int> rowIndex, vector <int> columnIndex, vector <int> element)  { 
		int i, j, k;

		memset(fixed_pos, 0, sizeof(fixed_pos));
		memset(S, 0, sizeof(S));
		memset(solpos, 0, sizeof(solpos));
		memset(AA, 0, sizeof(AA));

		for (i = 0; i <= n + 1; i++)
			for (j = 0; j <= m + 1; j++) {
				A[i][j] = 0;
				solpos[0][i][j] = solpos[1][i][j] = solpos[2][i][j] = solpos[3][i][j] = 1;
			}

		for (k = 0; k < rowIndex.size(); k++) {
			A[rowIndex[k] + 1][columnIndex[k] + 1] = element[k];
			fixed_pos[rowIndex[k] + 1][columnIndex[k] + 1] = 1;
		}

		for (i = 1; i <= n; i++) {
			for (j = 1; j <= m; j++) {
				if (!fixed_pos[i][j]) {
					AA[0][i][j] = max(AA[0][i - 1][j], AA[0][i][j - 1]) + 1;
				} else {
					AA[0][i][j] = A[i][j];
					if (AA[0][i][j] <= max(AA[0][i - 1][j], AA[0][i][j - 1]))
						solpos[0][i][j] = 0;
				}
				
				solpos[0][i][j] &= solpos[0][i][j - 1];
				solpos[0][i][j] &= solpos[0][i - 1][j];
				S[0][i][j] = S[0][i - 1][j] + S[0][i][j - 1] - S[0][i - 1][j - 1] + AA[0][i][j];
			}

			for (j = m; j >= 1; j--) {
				if (!fixed_pos[i][j]) {
					AA[1][i][j] = max(AA[1][i - 1][j], AA[1][i][j + 1]) + 1;
				} else {
					AA[1][i][j] = A[i][j];
					if (AA[1][i][j] <= max(AA[1][i - 1][j], AA[1][i][j + 1]))
						solpos[1][i][j] = 0;
				}
				
				solpos[1][i][j] &= solpos[1][i - 1][j];
				solpos[1][i][j] &= solpos[1][i][j + 1];
				S[1][i][j] = S[1][i - 1][j] + S[1][i][j + 1] - S[1][i - 1][j + 1] + AA[1][i][j];
			}
		}		

		for (i = n; i >= 1; i--) {
			for (j = 1; j <= m; j++) {
				if (!fixed_pos[i][j]) {
					AA[2][i][j] = max(AA[2][i + 1][j], AA[2][i][j - 1]) + 1;
				} else {
					AA[2][i][j] = A[i][j];
					if (AA[2][i][j] <= max(AA[2][i + 1][j], AA[2][i][j - 1]))
						solpos[2][i][j] = 0;
				}
				
				solpos[2][i][j] &= solpos[2][i][j - 1];
				solpos[2][i][j] &= solpos[2][i + 1][j];
				S[2][i][j] = S[2][i + 1][j] + S[2][i][j - 1] - S[2][i + 1][j - 1] + AA[2][i][j];
			}

			for (j = m; j >= 1; j--) {
				if (!fixed_pos[i][j]) {
					AA[3][i][j] = max(AA[3][i + 1][j], AA[3][i][j + 1]) + 1;
				} else {
					AA[3][i][j] = A[i][j];
					if (AA[3][i][j] <= max(AA[3][i + 1][j], AA[3][i][j + 1]))
						solpos[3][i][j] = 0;
				}
				
				solpos[3][i][j] &= solpos[3][i][j + 1];
				solpos[3][i][j] &= solpos[3][i + 1][j];
				S[3][i][j] = S[3][i + 1][j] + S[3][i][j + 1] - S[3][i + 1][j + 1] + AA[3][i][j];
			}
		}

		int canswer, answer = -1, sol_possible, a, b;

		for (a = 1; a <= n; a++) {
			for (b = 1; b <= m; b++) {
				sol_possible = solpos[0][a - 1][b - 1] & solpos[1][a - 1][b + 1] & solpos[2][a + 1][b - 1] & solpos[3][a + 1][b + 1];
				canswer = S[0][a - 1][b - 1] + S[1][a - 1][b + 1] + S[2][a + 1][b - 1] + S[3][a + 1][b + 1];

				for (i = 1; i < a && sol_possible; i++) {
					if (!fixed_pos[i][b])
						A[i][b] = max(max(A[i - 1][b], AA[0][i][b - 1]), AA[1][i][b + 1]) + 1;
					else if (A[i][b] <= max(max(A[i - 1][b], AA[0][i][b - 1]), AA[1][i][b + 1]))
						sol_possible = 0;
					canswer += A[i][b];
				}

				for (i = n; i > a && sol_possible; i--) {
					if (!fixed_pos[i][b])
						A[i][b] = max(max(A[i + 1][b], AA[2][i][b - 1]), AA[3][i][b + 1]) + 1;
					else if (A[i][b] <= max(max(A[i + 1][b], AA[2][i][b - 1]), AA[3][i][b + 1]))
						sol_possible = 0;
					canswer += A[i][b];
				}

				for (j = 1; j < b && sol_possible; j++) {
					if (!fixed_pos[a][j])
						A[a][j] = max(max(AA[0][a - 1][j], AA[2][a + 1][j]), A[a][j - 1]) + 1;
					else if (A[a][j] <= max(max(AA[0][a - 1][j], AA[2][a + 1][j]), A[a][j - 1]))
						sol_possible = 0;
					canswer += A[a][j];
				}

				for (j = m; j > b && sol_possible; j--) {
					if (!fixed_pos[a][j])
						A[a][j] = max(max(AA[1][a - 1][j], AA[3][a + 1][j]), A[a][j + 1]) + 1;
					else if (A[a][j] <= max(max(AA[1][a - 1][j], AA[3][a + 1][j]), A[a][j + 1]))
						sol_possible = 0;
					canswer += A[a][j];
				}
				
				if (!fixed_pos[a][b])
					A[a][b] = max(max(A[a - 1][b], A[a + 1][b]), max(A[a][b - 1], A[a][b + 1])) + 1;
				else if (A[a][b] <= max(max(A[a - 1][b], A[a + 1][b]), max(A[a][b - 1], A[a][b + 1])))
					sol_possible = 0;
				canswer += A[a][b];
	
				if (sol_possible && (answer < 0 || canswer < answer))
					answer = canswer;
			}
		}
		
		return answer;
	}
	
// BEGIN CUT HERE
	public:
	void run_test(int Case) { if ((Case == -1) || (Case == 0)) test_case_0(); if ((Case == -1) || (Case == 1)) test_case_1(); if ((Case == -1) || (Case == 2)) test_case_2(); if ((Case == -1) || (Case == 3)) test_case_3(); if ((Case == -1) || (Case == 4)) test_case_4(); if ((Case == -1) || (Case == 5)) test_case_5(); if ((Case == -1) || (Case == 6)) test_case_6(); if ((Case == -1) || (Case == 7)) test_case_7(); }
	private:
	template <typename T> string print_array(const vector<T> &V) { ostringstream os; os << "{ "; for (typename vector<T>::const_iterator iter = V.begin(); iter != V.end(); ++iter) os << '\"' << *iter << "\","; os << " }"; return os.str(); }
	void verify_case(int Case, const int &Expected, const int &Received) { cerr << "Test Case #" << Case << "..."; if (Expected == Received) cerr << "PASSED" << endl; else { cerr << "FAILED" << endl; cerr << "\tExpected: \"" << Expected << '\"' << endl; cerr << "\tReceived: \"" << Received << '\"' << endl; } }
	void test_case_0() { int Arg0 = 2; int Arg1 = 3; int Arr2[] = {0, 0, 0, 1, 1, 1}; vector <int> Arg2(Arr2, Arr2 + (sizeof(Arr2) / sizeof(Arr2[0]))); int Arr3[] = {0, 1, 2, 0, 1, 2}; vector <int> Arg3(Arr3, Arr3 + (sizeof(Arr3) / sizeof(Arr3[0]))); int Arr4[] = {4, 6, 9, 1, 3, 6}; vector <int> Arg4(Arr4, Arr4 + (sizeof(Arr4) / sizeof(Arr4[0]))); int Arg5 = 29; verify_case(0, Arg5, minSum(Arg0, Arg1, Arg2, Arg3, Arg4)); }
	void test_case_1() { int Arg0 = 2; int Arg1 = 3; int Arr2[] = {1, 0, 1}; vector <int> Arg2(Arr2, Arr2 + (sizeof(Arr2) / sizeof(Arr2[0]))); int Arr3[] = {2, 2, 0}; vector <int> Arg3(Arr3, Arr3 + (sizeof(Arr3) / sizeof(Arr3[0]))); int Arr4[] = {5, 7, 6}; vector <int> Arg4(Arr4, Arr4 + (sizeof(Arr4) / sizeof(Arr4[0]))); int Arg5 = 40; verify_case(1, Arg5, minSum(Arg0, Arg1, Arg2, Arg3, Arg4)); }
	void test_case_2() { int Arg0 = 3; int Arg1 = 3; int Arr2[] = {0, 0, 2, 2}; vector <int> Arg2(Arr2, Arr2 + (sizeof(Arr2) / sizeof(Arr2[0]))); int Arr3[] = {0, 2, 2, 0}; vector <int> Arg3(Arr3, Arr3 + (sizeof(Arr3) / sizeof(Arr3[0]))); int Arr4[] = {1, 1, 1, 1}; vector <int> Arg4(Arr4, Arr4 + (sizeof(Arr4) / sizeof(Arr4[0]))); int Arg5 = 15; verify_case(2, Arg5, minSum(Arg0, Arg1, Arg2, Arg3, Arg4)); }
	void test_case_3() { int Arg0 = 2; int Arg1 = 2; int Arr2[] = {0, 0, 1, 1}; vector <int> Arg2(Arr2, Arr2 + (sizeof(Arr2) / sizeof(Arr2[0]))); int Arr3[] = {0, 1, 1, 0}; vector <int> Arg3(Arr3, Arr3 + (sizeof(Arr3) / sizeof(Arr3[0]))); int Arr4[] = {5, 8, 5, 8}; vector <int> Arg4(Arr4, Arr4 + (sizeof(Arr4) / sizeof(Arr4[0]))); int Arg5 = -1; verify_case(3, Arg5, minSum(Arg0, Arg1, Arg2, Arg3, Arg4)); }
	void test_case_4() { int Arg0 = 1; int Arg1 = 3; int Arr2[] = {0}; vector <int> Arg2(Arr2, Arr2 + (sizeof(Arr2) / sizeof(Arr2[0]))); int Arr3[] = {1}; vector <int> Arg3(Arr3, Arr3 + (sizeof(Arr3) / sizeof(Arr3[0]))); int Arr4[] = {1}; vector <int> Arg4(Arr4, Arr4 + (sizeof(Arr4) / sizeof(Arr4[0]))); int Arg5 = -1; verify_case(4, Arg5, minSum(Arg0, Arg1, Arg2, Arg3, Arg4)); }
	void test_case_5() { int Arg0 = 123; int Arg1 = 45; int Arr2[] = {2, 3, 5, 7, 11}; vector <int> Arg2(Arr2, Arr2 + (sizeof(Arr2) / sizeof(Arr2[0]))); int Arr3[] = {13, 17, 19, 23, 29}; vector <int> Arg3(Arr3, Arr3 + (sizeof(Arr3) / sizeof(Arr3[0]))); int Arr4[] = {100, 200, 300, 400, 500}; vector <int> Arg4(Arr4, Arr4 + (sizeof(Arr4) / sizeof(Arr4[0]))); int Arg5 = 367047; verify_case(5, Arg5, minSum(Arg0, Arg1, Arg2, Arg3, Arg4)); }
	void test_case_6() { int Arg0 = 200; int Arg1 = 200; int Arr2[] = {5}; vector <int> Arg2(Arr2, Arr2 + (sizeof(Arr2) / sizeof(Arr2[0]))); int Arr3[] = {8}; vector <int> Arg3(Arr3, Arr3 + (sizeof(Arr3) / sizeof(Arr3[0]))); int Arr4[] = {666}; vector <int> Arg4(Arr4, Arr4 + (sizeof(Arr4) / sizeof(Arr4[0]))); int Arg5 = 5737554; verify_case(6, Arg5, minSum(Arg0, Arg1, Arg2, Arg3, Arg4)); }
	void test_case_7() { int Arg0 = 10; int Arg1 = 10; int Arr2[] = {0, 8, 7}; vector <int> Arg2(Arr2, Arr2 + (sizeof(Arr2) / sizeof(Arr2[0]))); int Arr3[] = {3, 1, 9}; vector <int> Arg3(Arr3, Arr3 + (sizeof(Arr3) / sizeof(Arr3[0]))); int Arr4[] = {5, 4, 7}; vector <int> Arg4(Arr4, Arr4 + (sizeof(Arr4) / sizeof(Arr4[0]))); int Arg5 = 593; verify_case(7, Arg5, minSum(Arg0, Arg1, Arg2, Arg3, Arg4)); }

// END CUT HERE
 
}; 

// BEGIN CUT HERE
int main() {
	TheMountain ___test; 
	___test.run_test(-1); 
	return 0;
} 
// END CUT HERE
