// BEGIN CUT HERE
/*
PROBLEM STATEMENT
This problem statement contains superscripts and/or subscripts. 
These may not display properly outside the applet.

Manao is very curious about bitwise operations. He decided to learn 
how the bitwise AND works by testing some examples. Manao takes a 
vector <int> X and a int K and performs the operations described by 
the following pseudocode:


  for i := 1 to K
    X := process(X)
	
  process(list X)
    list Y = []
      for i := 0 to length(X) - 1
        for j := i + 1 to length(X) - 1
          Y.append(X[i] & X[j])
    return Y

The function length(X) returns the length of the list X.
The method Y.append(Z) adds the element Z to the end of the list Y.
The elements in a list are indexed starting from 0. The lists allow 
duplicate elements.

Manao faced some problems when his lists grew a little too long. So 
he is asking for help. He has several queries of the type "After the 
entire pseudocode is executed, will N occur in X?". You are given a 
vector <int> queries containing M elements. Return a string of 
length M. The i-th character in the returned string should be '+' if 
the number queries[i] will occur in the list X after executing X := 
process(X) K times, and it should be '-' if queries[i] will not 
occur in that list.

DEFINITION
Class:YetAnotherANDProblem
Method:test
Parameters:vector <int>, int, vector <int>
Returns:string
Method signature:string test(vector <int> X, int K, vector <int> 
queries)


CONSTRAINTS
-X will contain between 3 and 16 elements, inclusive.
-Each element of X will be between 1 and 1,000,000,000, inclusive.
-K will be between 1 and 50, inclusive.
-queries will contain between 1 and 50 elements, inclusive.
-Each element of queries will be between 0 and 1,000,000,000, 
inclusive.


EXAMPLES

0)
{10, 14, 7}
1
{10, 14, 2, 4}

Returns: "+-+-"

Manao's initial list is [10, 14, 7]. He performs the process of 
taking all pairwise bitwise ANDs of the numbers once. The binary 
representations of 10, 14 and 7 are "1010", "1110" and "0111" 
correspondingly. So Manao obtains:

1010 & 1410 = 10102 & 11102 = 10102 = 1010
1010 & 710 = 10102 & 01112 = 00102 = 210
1410 & 710 = 11102 & 01112 = 01102 = 610

Thus, the new list contains 10 and 2, but does not contain 14 or 4.

1)
{30, 29, 27, 23, 15}
2
{28, 9, 16, 0, 12}

Returns: "-++-+"

The binary representations of the numbers in X:

11110
11101
11011
10111
01111


Note that 2810 = 111002 was in the list after the first iteration, 
but does not appear in the final list. 910 (001012), 1610 (100002) 
and 1210 (011002) occur in the final list, and 0 does not.

2)
{5, 5, 5, 5, 5}
50
{0, 1, 2, 3, 4, 5, 6, 7, 8, 9}

Returns: "-----+----"

Remember that duplicates are kept in the list.

3)
{71258, 1257, 2592588, 2885851, 999928, 123456, 59881, 99999}
4
{72, 91, 10, 0, 27, 64, 8}

Returns: "+--+-++"



*/
// END CUT HERE

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <set>
#include <map>

using namespace std;

set<int> ands;
int cmin, cmax, tand, nsel;

void GenSubsets(int idx, const vector<int> & X) {
	if (idx >= X.size()) {
		if (nsel >= cmin && nsel <= cmax)
			ands.insert(tand);
	} else {
		int oldtand = tand;
		tand &= X[idx];
		nsel++;
		GenSubsets(idx + 1, X);
		nsel--;
		tand = oldtand;
		GenSubsets(idx + 1, X);
	}
}

class YetAnotherANDProblem { 
	public: 
	string test(vector <int> X, int K, vector <int> queries)  { 
		int i;
		
		cmin = cmax = 1;
		for (i = 1; i <= K; i++) {
			cmin++;
			cmax *= 2;
			if (cmin > 3)
				cmin = 3;
			if (cmax > X.size())
				cmax = X.size();
		}
		
		tand = (1 << 30) - 1;
		ands.clear();
		nsel = 0;
		GenSubsets(0, X);

		string answer = "";
		for (i = 0; i < queries.size(); i++)
			if (ands.find(queries[i]) != ands.end())
				answer += "+";
			else
				answer += "-";
		
		return answer;
	}
	
// BEGIN CUT HERE
	public:
	void run_test(int Case) { if ((Case == -1) || (Case == 0)) test_case_0(); if ((Case == -1) || (Case == 1)) test_case_1(); if ((Case == -1) || (Case == 2)) test_case_2(); if ((Case == -1) || (Case == 3)) test_case_3(); }
	private:
	template <typename T> string print_array(const vector<T> &V) { ostringstream os; os << "{ "; for (typename vector<T>::const_iterator iter = V.begin(); iter != V.end(); ++iter) os << '\"' << *iter << "\","; os << " }"; return os.str(); }
	void verify_case(int Case, const string &Expected, const string &Received) { cerr << "Test Case #" << Case << "..."; if (Expected == Received) cerr << "PASSED" << endl; else { cerr << "FAILED" << endl; cerr << "\tExpected: \"" << Expected << '\"' << endl; cerr << "\tReceived: \"" << Received << '\"' << endl; } }
	void test_case_0() { int Arr0[] = {23, 59, 70, 16}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arg1 = 1; int Arr2[] = {0, 1, 2, 3, 4, 5, 6, 7, 8}; vector <int> Arg2(Arr2, Arr2 + (sizeof(Arr2) / sizeof(Arr2[0]))); string Arg3 = "+-+-"; verify_case(0, Arg3, test(Arg0, Arg1, Arg2)); }
	void test_case_1() { int Arr0[] = {30, 29, 27, 23, 15}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arg1 = 2; int Arr2[] = {28, 9, 16, 0, 12}; vector <int> Arg2(Arr2, Arr2 + (sizeof(Arr2) / sizeof(Arr2[0]))); string Arg3 = "-++-+"; verify_case(1, Arg3, test(Arg0, Arg1, Arg2)); }
	void test_case_2() { int Arr0[] = {5, 5, 5, 5, 5}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arg1 = 50; int Arr2[] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9}; vector <int> Arg2(Arr2, Arr2 + (sizeof(Arr2) / sizeof(Arr2[0]))); string Arg3 = "-----+----"; verify_case(2, Arg3, test(Arg0, Arg1, Arg2)); }
	void test_case_3() { int Arr0[] = {71258, 1257, 2592588, 2885851, 999928, 123456, 59881, 99999}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arg1 = 4; int Arr2[] = {72, 91, 10, 0, 27, 64, 8}; vector <int> Arg2(Arr2, Arr2 + (sizeof(Arr2) / sizeof(Arr2[0]))); string Arg3 = "+--+-++"; verify_case(3, Arg3, test(Arg0, Arg1, Arg2)); }

// END CUT HERE
 
}; 

// BEGIN CUT HERE
int main() {
	YetAnotherANDProblem ___test; 
	___test.run_test(-1); 
	return 0;
} 
// END CUT HERE
