// BEGIN CUT HERE
/*
PROBLEM STATEMENT
There are some deer in the zoo.
Each deer has two antlers.
You are given vector <int>s antler1 and antler2.
These two vector <int>s will contain the same number of elements.
For each index i, antler1[i] and antler2[i] are the weights of the 
two antlers of one of the deer.


You are also given an int capacity.
A deer is unbalanced if the weight difference between his antlers is 
strictly more than capacity.


You decided to perform some operations on the deer. 
Your goal is to make all deer balanced.
In each operation, you can choose some two antlers (each on a 
different deer) and swap them.


Return the minimal number of operations required to make all deer 
balanced.
If this is impossible, return -1 instead.

DEFINITION
Class:AntlerSwapping
Method:getmin
Parameters:vector <int>, vector <int>, int
Returns:int
Method signature:int getmin(vector <int> antler1, vector <int> 
antler2, int capacity)


CONSTRAINTS
-antler1 and antler2 will contain the same number of elements.
-antler1 and antler2 will contain between 1 and 16 elements, 
inclusive.
-Each element of antler1 and antler2 will be between 1 and 
1,000,000, inclusive.
-capacity will be between 0 and 1,000,000, inclusive.


EXAMPLES

0)
{3, 2, 2}
{3, 5, 5}
0

Returns: 1

There are three deer in the zoo.
As capacity=0, a deer is only balanced if he has two antlers of 
exactly equal weight.
Currently, deer 0 is balanced and the other two are not.
We can fix that in a single operation.
For example, we can swap deer 1's antler 1 and deer 2's antler 2.
After this operation, deer 1 will have two antlers that weigh 5 
each, and deer 2 will have two antlers that weigh 2 each.

1)
{4, 2, 6, 4, 8, 5, 2, 3}
{3, 4, 5, 2, 8, 5, 7, 6}
1

Returns: 2

One of the optimal ways is as follows:

Swap deer 1's antler with weight 2 and deer 3's antler with weight 4.
Swap deer 6's antler with weight 7 and deer 7's antler with weight 3.


2)
{12, 34, 56, 78}
{1234, 2345, 3456, 4567}
100

Returns: -1

If it is impossible to achieve the goal, return -1.

3)
{47, 58, 2013}
{49, 55, 2013}
3

Returns: 0



4)
{4, 1, 7, 5, 7, 8, 2, 1, 3, 1, 7, 5, 9, 4, 9, 1}
{10, 6, 5, 3, 1, 8, 4, 4, 4, 7, 1, 4, 6, 5, 10, 10}
1

Returns: 7



*/
// END CUT HERE

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <set>
#include <map>

using namespace std;

#define NMAX 16
#define INF 1000000000

vector<int> v;
char sel1[NMAX], sel2[NMAX];
int a[NMAX][2], cmin[1 << NMAX], sel1idx[NMAX], nsel1;
int N, cap, mask1, mask2, tmp, i;

void GenSubsets2(int idx) {
	if (idx >= nsel1) {
		if (mask2 > 0 && mask2 < mask1) {
			tmp = cmin[mask2] + cmin[mask1 ^ mask2];
			if (tmp < cmin[mask1]) {
				cmin[mask1] = tmp;
			}
		}
	} else {
		mask2 |= (1 << sel1idx[idx]);
		GenSubsets2(idx + 1);
		mask2 ^= (1 << sel1idx[idx]);
		GenSubsets2(idx + 1);
	}
}

void TestSubset(int mask) {
	int i;

	v.clear();
	for (i = 0; i < nsel1; i++) {
		v.push_back(a[sel1idx[i]][0]);
		v.push_back(a[sel1idx[i]][1]);
	}
	
	sort(v.begin(), v.end());
	for (i = 0; i < nsel1; i++) {
		if (v[2 * i + 1] - v[2 * i] > cap)
			break;
	}
	
	if (i == nsel1)
		cmin[mask] = nsel1 - 1;
}

void GenSubsets1(int idx) {
	if (idx >= N) {
		if (mask1 == 0) {
			cmin[mask1] = 0;
			return;
		}
		cmin[mask1] = INF;
		TestSubset(mask1);
		mask2 = 0;
		GenSubsets2(0);
	} else {
		sel1[idx] = 0;
		GenSubsets1(idx + 1);

		sel1[idx] = 1;
		sel1idx[nsel1++] = idx;
		mask1 |= (1 << idx);
		GenSubsets1(idx + 1);
		mask1 ^= (1 << idx);
		nsel1--;
	}
}

class AntlerSwapping { 
	public: 
	int getmin(vector <int> antler1, vector <int> antler2, int capacity)  { 
		N = antler1.size();
		cap = capacity;
		for (i = 0; i < N; i++) {
			a[i][0] = antler1[i];
			a[i][1] = antler2[i];
		}

		mask1 = nsel1 = 0;
		GenSubsets1(0);
		
		int answer = cmin[(1 << N) - 1];
		if (answer == INF)
			answer = -1;
		return answer;
	}
	
// BEGIN CUT HERE
	public:
	void run_test(int Case) { if ((Case == -1) || (Case == 0)) test_case_0(); if ((Case == -1) || (Case == 1)) test_case_1(); if ((Case == -1) || (Case == 2)) test_case_2(); if ((Case == -1) || (Case == 3)) test_case_3(); if ((Case == -1) || (Case == 4)) test_case_4(); }
	private:
	template <typename T> string print_array(const vector<T> &V) { ostringstream os; os << "{ "; for (typename vector<T>::const_iterator iter = V.begin(); iter != V.end(); ++iter) os << '\"' << *iter << "\","; os << " }"; return os.str(); }
	void verify_case(int Case, const int &Expected, const int &Received) { cerr << "Test Case #" << Case << "..."; if (Expected == Received) cerr << "PASSED" << endl; else { cerr << "FAILED" << endl; cerr << "\tExpected: \"" << Expected << '\"' << endl; cerr << "\tReceived: \"" << Received << '\"' << endl; } }
	void test_case_0() { int Arr0[] = {3, 2, 2}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arr1[] = {3, 5, 5}; vector <int> Arg1(Arr1, Arr1 + (sizeof(Arr1) / sizeof(Arr1[0]))); int Arg2 = 0; int Arg3 = 1; verify_case(0, Arg3, getmin(Arg0, Arg1, Arg2)); }
	void test_case_1() { int Arr0[] = {4, 2, 6, 4, 8, 5, 2, 3}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arr1[] = {3, 4, 5, 2, 8, 5, 7, 6}; vector <int> Arg1(Arr1, Arr1 + (sizeof(Arr1) / sizeof(Arr1[0]))); int Arg2 = 1; int Arg3 = 2; verify_case(1, Arg3, getmin(Arg0, Arg1, Arg2)); }
	void test_case_2() { int Arr0[] = {12, 34, 56, 78}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arr1[] = {1234, 2345, 3456, 4567}; vector <int> Arg1(Arr1, Arr1 + (sizeof(Arr1) / sizeof(Arr1[0]))); int Arg2 = 100; int Arg3 = -1; verify_case(2, Arg3, getmin(Arg0, Arg1, Arg2)); }
	void test_case_3() { int Arr0[] = {47, 58, 2013}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arr1[] = {49, 55, 2013}; vector <int> Arg1(Arr1, Arr1 + (sizeof(Arr1) / sizeof(Arr1[0]))); int Arg2 = 3; int Arg3 = 0; verify_case(3, Arg3, getmin(Arg0, Arg1, Arg2)); }
	void test_case_4() { int Arr0[] = {4, 1, 7, 5, 7, 8, 2, 1, 3, 1, 7, 5, 9, 4, 9, 1}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arr1[] = {10, 6, 5, 3, 1, 8, 4, 4, 4, 7, 1, 4, 6, 5, 10, 10}; vector <int> Arg1(Arr1, Arr1 + (sizeof(Arr1) / sizeof(Arr1[0]))); int Arg2 = 1; int Arg3 = 7; verify_case(4, Arg3, getmin(Arg0, Arg1, Arg2)); }

// END CUT HERE
 
}; 

// BEGIN CUT HERE
int main() {
	AntlerSwapping ___test; 
	___test.run_test(-1); 
	return 0;
} 
// END CUT HERE
