// BEGIN CUT HERE
/*
PROBLEM STATEMENT
Two sequences A and B are called similar if they are both non-empty, 
have the same length, and have the following property:
we can obtain two identical sequences by deleting one element of A 
and one element of B.
(The deleted elements do not necessarily have to have the same index.
The order of the remaining elements in each sequence has to be 
preserved.)

You are given a vector <int> seq and a int bound.
Consider all sequences such that each element is an integer between 
1 and bound, inclusive.
Count how many of such sequences are similar to seq.
Return this count modulo 1,000,000,009.

DEFINITION
Class:SimilarSequences
Method:count
Parameters:vector <int>, int
Returns:int
Method signature:int count(vector <int> seq, int bound)


CONSTRAINTS
-seq will contain between 1 and 40 elements, inclusive.
-bound will be between 1 and 1,000,000,000, inclusive.
-Each element in seq will be between 1 and bound, inclusive.


EXAMPLES

0)
{1, 1}
3

Returns: 5

We need to count the number of sequences which consist of elements 
in [1, 3] and are similar to (1, 1). These sequences are (1, 1), (1, 
2), (2, 1), (1, 3) and (3, 1).

1)
{1, 2}
2

Returns: 4

Any valid sequence is similar to (1, 2).

2)
{999999999}
1000000000

Returns: 1000000000

Note that after deleting the elements from the sequences, the 
resulting sequences may be empty.

3)
{1, 2, 3, 4, 5}
5

Returns: 97



4)
{5, 8, 11, 12, 4, 1, 7, 9}
1000000000

Returns: 999999363



*/
// END CUT HERE

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <set>
#include <map>
#include <queue>

using namespace std;

#define MOD 1000000009

set<int> s;
set<int>::iterator sit;

#define MAXNODES1 1111
#define MAXNODES2 1000000

char fnode1[MAXNODES1], fnode2[MAXNODES2];
map<int, int> next1[MAXNODES1], next2[MAXNODES2];
map<int, int>::iterator it;
int nnodes1, nnodes2, node, nfin1, nfin2, bmax;

class SimilarSequences { 
	public: 
	int count(vector <int> seq, int bound)  { 
		int i, j, k, N = seq.size();
		bmax = bound;
	
		s.clear();
		for (i = 0; i < N; i++)
			s.insert(seq[i]);

		nnodes1 = nnodes2 = 1;
		nfin1 = nfin2 = 0;
		next1[nnodes1].clear();
		next2[nnodes2].clear();

		memset(fnode1, 0, sizeof(fnode1));
		memset(fnode2, 0, sizeof(fnode2));
		
		vector<int> v, v2;

		for (i = 0; i < N; i++) {
			// add 1
			for (node = 1, j = 0; j < N; j++) {
				if (i == j)
					continue;
				it = next1[node].find(seq[j]);
				if (it == next1[node].end()) {
					nnodes1++;
					next1[nnodes1].clear();
					next1[node][seq[j]] = nnodes1;
					node = nnodes1;
				} else {
					node = it->second;
				}
			}
			
			if (!fnode1[node]) {
				fnode1[node] = 1;
				nfin1++;
			}

			v.clear();
			for (j = 0; j < N; j++)
				if (i != j)
					v.push_back(seq[j]);
				
			for (sit = s.begin(); sit != s.end(); sit++) {
				for (j = 0; j < N; j++) {
					v2.clear();
					for (k = 0; k < j; k++)
						v2.push_back(v[k]);
					v2.push_back(*sit);
					for (k = j; k < N - 1; k++)
						v2.push_back(v[k]);

					// add 2
					for (node = 1, k = 0; k < N; k++) {
						it = next2[node].find(v2[k]);
						if (it == next2[node].end()) {
							nnodes2++;
							next2[nnodes2].clear();
							next2[node][v2[k]] = nnodes2;
							node = nnodes2;
						} else {
							node = it->second;
						}
					}

					if (!fnode2[node]) {
						fnode2[node] = 1;
						nfin2++;
					}
				}
			}
		}

		long long answer = nfin2;
		long long mult = 0;
		for (i = 0; i < N; i++)
			mult = (mult + (long long) (bmax - s.size())) % MOD;

		fprintf(stderr, "nnodes1=%d nnodes2=%d nfin1=%d nfin2=%d mult=%lld bmax=%d\n", nnodes1, nnodes2, nfin1, nfin2, mult, bmax);

		answer = (answer + mult * (long long) nfin1) % MOD;
		return (int) answer;
	}

// BEGIN CUT HERE
	public:
	void run_test(int Case) { if ((Case == -1) || (Case == 0)) test_case_0(); if ((Case == -1) || (Case == 1)) test_case_1(); if ((Case == -1) || (Case == 2)) test_case_2(); if ((Case == -1) || (Case == 3)) test_case_3(); if ((Case == -1) || (Case == 4)) test_case_4(); }
	private:
	template <typename T> string print_array(const vector<T> &V) { ostringstream os; os << "{ "; for (typename vector<T>::const_iterator iter = V.begin(); iter != V.end(); ++iter) os << '\"' << *iter << "\","; os << " }"; return os.str(); }
	void verify_case(int Case, const int &Expected, const int &Received) { cerr << "Test Case #" << Case << "..."; if (Expected == Received) cerr << "PASSED" << endl; else { cerr << "FAILED" << endl; cerr << "\tExpected: \"" << Expected << '\"' << endl; cerr << "\tReceived: \"" << Received << '\"' << endl; } }
	void test_case_0() { 
		int Arr0[] = {431460730, 103190409, 738139991, 107321069, 45491145, 321917088, 372671422, 219526913, 25633755, 251652765, 431460730, 431879050, 431879050, 357583115, 294861628, 680345489, 680345489, 397431490, 431460730, 680345489, 107321069, 234034260, 251652765, 103190409, 321917088, 738139991, 310652494, 107321069, 738139991, 25633755, 234034260, 45491145, 25633755, 431879050, 357583115, 738139991, 738139991, 25633755, 738139991, 372671422};
		vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0])));
		int Arg1 = 790906595; int Arg2 = 541748554; verify_case(0, Arg2, count(Arg0, Arg1)); }
	void test_case_1() { int Arr0[] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arg1 = 1000000000; int Arg2 = 999983971; verify_case(1, Arg2, count(Arg0, Arg1)); }
	void test_case_2() { int Arr0[] = {999999999}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arg1 = 1000000000; int Arg2 = 1000000000; verify_case(2, Arg2, count(Arg0, Arg1)); }
	void test_case_3() { int Arr0[] = {1, 2, 3, 4, 5}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arg1 = 5; int Arg2 = 97; verify_case(3, Arg2, count(Arg0, Arg1)); }
	void test_case_4() { int Arr0[] = {5, 8, 11, 12, 4, 1, 7, 9}; vector <int> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); int Arg1 = 1000000000; int Arg2 = 999999363; verify_case(4, Arg2, count(Arg0, Arg1)); }

// END CUT HERE
 
}; 

// BEGIN CUT HERE
int main() {
	SimilarSequences ___test; 
	___test.run_test(-1); 
	return 0;
} 
// END CUT HERE
