// BEGIN CUT HERE
/*
PROBLEM STATEMENT
You are organizing a local Rock-Paper-Scissors Tournament, where the 
best N players in your town will compete for fame and prizes.  You 
have already ranked the participants according to their performance 
in the qualifying round, so each one of them will have a seed for 
winning the tournament in the range from 1 to N.

The tournament will be in a knockout format and will consist of 
multiple rounds.  During each round the competitors still cleft in 
the competition are randomly paired up with each other.  The winners 
from each game advance to the cnext round.  The great champion is the 
winner of the final game between the last two remaining players.

The tournament starts soon, and you are preoccupied with some pre-
statistics. You estimate that a player with any seed S can win 
against players with greater seeds, and against ones with lower 
seeds, if the difference between their seeds is not greater than sqrt
(C * S), where C is a constant.  Assuming that the outcome of all 
the played games during the tournament agrees with your estimation, 
a player can win the tournament if there exists an assignment of 
games in each of the rounds such that the player can win each round.

You will be given two ints, rounds, denoting the number of rounds, 
and the constant C. Return the greatest seed of a player that could 
win the tournament.  Note that N, the number of competitors will be 
N = 2rounds.


DEFINITION
Class:RPSTournament2
Method:greatestSeed
Parameters:int, int
Returns:int
Method signature:int greatestSeed(int rounds, int C)


CONSTRAINTS
-rounds will be between 1 and 16, inclusive.
-C will be between 0 and 1500, inclusive.


EXAMPLES

0)
2
0

Returns: 1

There is no room for surprises here.  Since nobody can win against a 
higher-rated opponent, the lowest seed will win the tournament.


1)
3
1

Returns: 6

Player 2 can win against player 1, so both of them can win the final 
if they are to meet in that phase of the competition.  Since players 
3 and 4 can beat player 2, they can win the tournament as long as 
player 2 wins against player 1 somewhere before the final.  Finally, 
players 5 and 6 can beat player 4 in the final, as long as player 4 
wins against player 2 in the semifinals and player 2 eliminates 
player 1 in the first round.  It is impossible for players 7 and 8 
to win the tournament.  We return the greatest seed of those who can 
win, namely 6.


2)
4
1

Returns: 9



3)
7
3

Returns: 50



4)
15
180

Returns: 9755

Watch out for timeout!


*/
// END CUT HERE

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <set>
#include <map>
#include <algorithm>

using namespace std;

#define NMAX 65555

vector<int> players;
char marked[NMAX];
int parent[NMAX], ssize[NMAX], cnext[NMAX];
int cleft[NMAX];
int N, M, i, j;

int Find(int x) {
	int tx = x, px;
	while (parent[tx] > 0)
		tx = parent[tx];

	while (x != tx) {
		px = parent[x];
		parent[x] = tx;
		x = px;
	}
	
	return tx;
}

void Union(int x, int y) {
	if (x <= 0 || y <= 0 || x > N || y > N || !marked[x] || !marked[y])
		return;
	int tx = Find(x), ty = Find(y);
	if (ssize[tx] <= ssize[ty]) {
		parent[tx] = ty;
		ssize[ty] += ssize[tx];
	} else {
		parent[ty] = tx;
		ssize[tx] += ssize[ty];
		cnext[tx] = cnext[ty];
	}
}

int OK(int S) {
	//fprintf(stderr, "Testing %d/%d\n", S, N);

	players.clear();
	players.push_back(S);

	for (i = 1; i <= N; i++)
		marked[i] = 0;
	
	marked[S] = 1;
	parent[S] = 0;
	ssize[S] = 1;
	cnext[S] = S + 1;
	
	while (players.size() < N) {
		sort(players.begin(), players.end());
		M = players.size();
		for (i = 0; i < M; i++) {
			j = cleft[players[i]];
			if (j == players[i])
				j++;
			if (marked[j])
				j = cnext[Find(j)];
			if (j > N)
				return 0;

			//if (N == 8)
			//	fprintf(stderr, "  match %d - %d\n", players[i], j);
			
			marked[j] = 1;
			players.push_back(j);
			parent[j] = 0;
			ssize[j] = 1;
			cnext[j] = j + 1;
			Union(j, j + 1);
			Union(j - 1, j);
		}		
	}

	return 1;
}

class RPSTournament2 { 
	public: 
	int greatestSeed(int rounds, int C)  { 
		double dif;

		N = 1 << rounds;
		for (i = 1; i <= N; i++) {
			dif = sqrt((double) i * C);
			cleft[i] = i - (int) dif;
			if (cleft[i] < 1)
				cleft[i] = 1;
		}

		int li = 2, ls = N, mid, answer = 1;
		
		while (li <= ls) {
			mid = (li + ls) >> 1;
			if (OK(mid)) {
				answer = mid;
				li = mid + 1;
			} else {
				ls = mid - 1;
			}
		}

		return answer;
	}
	
// BEGIN CUT HERE
	public:
	void run_test(int Case) { if ((Case == -1) || (Case == 0)) test_case_0(); if ((Case == -1) || (Case == 1)) test_case_1(); if ((Case == -1) || (Case == 2)) test_case_2(); if ((Case == -1) || (Case == 3)) test_case_3(); if ((Case == -1) || (Case == 4)) test_case_4(); }
	private:
	template <typename T> string print_array(const vector<T> &V) { ostringstream os; os << "{ "; for (typename vector<T>::const_iterator iter = V.begin(); iter != V.end(); ++iter) os << '\"' << *iter << "\","; os << " }"; return os.str(); }
	void verify_case(int Case, const int &Expected, const int &Received) { cerr << "Test Case #" << Case << "..."; if (Expected == Received) cerr << "PASSED" << endl; else { cerr << "FAILED" << endl; cerr << "\tExpected: \"" << Expected << '\"' << endl; cerr << "\tReceived: \"" << Received << '\"' << endl; } }
	void test_case_0() { int Arg0 = 4; int Arg1 = 2; int Arg2 = 1; verify_case(0, Arg2, greatestSeed(Arg0, Arg1)); }
	void test_case_1() { int Arg0 = 3; int Arg1 = 1; int Arg2 = 6; verify_case(1, Arg2, greatestSeed(Arg0, Arg1)); }
	void test_case_2() { int Arg0 = 4; int Arg1 = 1; int Arg2 = 9; verify_case(2, Arg2, greatestSeed(Arg0, Arg1)); }
	void test_case_3() { int Arg0 = 7; int Arg1 = 3; int Arg2 = 50; verify_case(3, Arg2, greatestSeed(Arg0, Arg1)); }
	void test_case_4() { int Arg0 = 15; int Arg1 = 180; int Arg2 = 9755; verify_case(4, Arg2, greatestSeed(Arg0, Arg1)); }

// END CUT HERE
 
}; 

// BEGIN CUT HERE
int main() {
	RPSTournament2 ___test; 
	___test.run_test(-1); 
	return 0;
} 
// END CUT HERE
