// BEGIN CUT HERE
/*
PROBLEM STATEMENT
The job of a search engine is to provide relevant results to search 
queries.  However, most real search queries 
match thousands, if not millions, of websites.  Since some of them 
have to be shown, it is important for 
the search engine to figure out which websites are worth 
displaying.  This is done with several heuristics, 
one of which considers the count of external websites linking to the 
website in question, as well as their 
importance.

An easy way to do this is to assign all websites an initial vote of 
1.  If website A links to 
website B, website A adds all its votes to website B.  As an 
example, consider websites A, 
B and C.  They each have an initial vote of 1.  However, after 
careful evaluation you discover that both A and 
B link to C.  Therefore, A and B each have 1 vote, and C has 3 
votes.  Should a search query ever match all three 
websites, website C will be displayed first.  The task is made more 
complex by websites often exchanging links.  This means that 
if website A links to B, B may also link to A.  To prevent 
artificial inflation of website importance, all votes from website A 
to website B must not count toward B's votes if website B directly 
or indirectly links to A.

You are given a vector <string> votes and string website.  Each 
element of votes consists of a website name, followed by a single 
space, followed by a single space separated list of all the websites 
that link to it.  All website names contain only uppercase letters.  
Count and return the 
votes that website has, following the strategy outlined above.


DEFINITION
Class:WebsiteRank2
Method:countVotes
Parameters:vector <string>, string
Returns:long long
Method signature:long long countVotes(vector <string> votes, string 
website)


CONSTRAINTS
-votes will contain between 1 and 50 elements, inclusive.
-Each element of votes will contain between 1 and 50 characters, 
inclusive.
-Each element of votes will contain only uppercase letters ('A'-'Z') 
and spaces (' ').
-Each element of votes will contain no leading or trailing spaces, 
and no consecutive spaces.
-website will contain between 1 and 50 characters, inclusive.
-website will contain only uppercase letters ('A'-'Z').
-website will be contained in votes either as a linking website or a 
website that is linked to.
-A website will never directly link back to itself.
-All elements of votes will start with different website names.
-Each element of votes will have a distinct first website name.  
Within a single element of votes no website name will be repeated.


EXAMPLES

0)
{"C A B"}
"C"

Returns: 3

The example from the problem statement.


1)
{"A B C D", "B C D", "C D"}
"A"

Returns: 8

B has 4 votes, C has 2 votes, and D has 1 vote. Initially A has 1 
vote, which changes to 8 after the strategy is applied.


2)
{"A"}
"A"

Returns: 1

Unfortunately, no websites currently link to A.


3)
{"A B", "B A"}
"A"

Returns: 1

B's votes to A and A's votes to B are ignored, leaving just one vote 
for each.


4)
{"A B C D E F", "B A", "C B", "D B"}
"A"

Returns: 3

Not only does A directly link to B, but also indirectly to C and D.  
This leaves just 3 votes for A.


5)
{"MYSITE OTHERSITE ANOTHERSITE THIRDSITE"}
"MYSITE"

Returns: 4



*/
// END CUT HERE

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <set>
#include <map>

using namespace std;

map<string, int> mws;
map<string, int>::iterator it;
map<int, int> mwbase;
map<int, int>::iterator wit;
vector<int> vin[55];
set<string> wbase;
set<int> wbaseidx, tss;
int vbase[55], ts[2555];
long long nv[2555];
char wname[200];
int A[55][55];

class WebsiteRank2 { 
	public: 
	long long countVotes(vector <string> votes, string website)  { 
		int N = votes.size(), M = 0, T = 0, i, j, k, p, q, L;

		mws.clear();
		mwbase.clear();
		wbase.clear();
		wbaseidx.clear();
		tss.clear();
		
		for (i = 0; i < N; i++) {
			vin[i].clear();

			int poz = 0, num;
			sscanf(votes[i].c_str(), "%s%n", wname, &num);
			poz += num;
			it = mws.find(wname);
			if (it != mws.end())
				j = it->second;
			else {
				mws[wname] = j = M;
				M++;
			}
			
			vbase[i] = j;
			wbase.insert(wname);
			wbaseidx.insert(j);
			mwbase[j] = i;
			
			L = votes[i].length();
			while (poz < L) {
				sscanf(votes[i].c_str() + poz, "%s%n", wname, &num);
				poz += num;
				it = mws.find(wname);
				if (it != mws.end())
					k = it->second;
				else {
					mws[wname] = k = M;
					M++;
				}
			
				vin[i].push_back(k);
			}
		}

		if (wbase.find(website) == wbase.end())
			return 1;

		memset(A, 0, sizeof(A));

		for (i = 0; i < N; i++) {
			nv[i] = 1;
			for (j = 0; j < vin[i].size(); j++) {
				k = vin[i][j];
				wit = mwbase.find(k);
				if (wit != mwbase.end())
					A[wit->second][i] = 1;
				else if (tss.find(k) == tss.end()) {
					ts[T] = k;
					T++;
					tss.insert(k);
				}
			}
		}

		// Strongly connected components.
		for (k = 0; k < N; k++)
			for (i = 0; i < N; i++)
				for (j = 0; j < N; j++)
					if (A[i][k] && A[k][j])
						A[i][j] = 1;

		// Topo sort.
		if (T != M - N)
			exit(1);

		while (T < M) {
			for (i = 0; i < N; i++)
				if (tss.find(vbase[i]) == tss.end()) {
					for (j = 0; j < N; j++)
						if (j != i && tss.find(vbase[j]) == tss.end() &&
							A[j][i] && !A[i][j])
							break;
					if (j == N) {
						ts[T] = vbase[i];
						T++;
						tss.insert(vbase[i]);
					}
				}
		}

		memset(nv, 0, sizeof(nv));
		for (i = 0; i < M; i++)
			nv[i] = 1;
		
		for (i = 0; i < M; i++) {
			if (wbaseidx.find(ts[i]) != wbaseidx.end()) {
				j = mwbase[ts[i]];
				for (k = 0; k < vin[j].size(); k++) {
					p = vin[j][k];
					if (wbaseidx.find(p) == wbaseidx.end())
						nv[ts[i]] += nv[p];
					else {
						q = mwbase[p];
						if (!A[j][q])
							nv[ts[i]] += nv[p];
					}
				}
			}
		}
		
		return nv[mws[website]];
	}
	
// BEGIN CUT HERE
	public:
	void run_test(int Case) { if ((Case == -1) || (Case == 0)) test_case_0(); if ((Case == -1) || (Case == 1)) test_case_1(); if ((Case == -1) || (Case == 2)) test_case_2(); if ((Case == -1) || (Case == 3)) test_case_3(); if ((Case == -1) || (Case == 4)) test_case_4(); if ((Case == -1) || (Case == 5)) test_case_5(); }
	private:
	template <typename T> string print_array(const vector<T> &V) { ostringstream os; os << "{ "; for (typename vector<T>::const_iterator iter = V.begin(); iter != V.end(); ++iter) os << '\"' << *iter << "\","; os << " }"; return os.str(); }
	void verify_case(int Case, const long long &Expected, const long long &Received) { cerr << "Test Case #" << Case << "..."; if (Expected == Received) cerr << "PASSED" << endl; else { cerr << "FAILED" << endl; cerr << "\tExpected: \"" << Expected << '\"' << endl; cerr << "\tReceived: \"" << Received << '\"' << endl; } }
	void test_case_0() { string Arr0[] = {"C A B"}; vector <string> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); string Arg1 = "C"; long long Arg2 = 3LL; verify_case(0, Arg2, countVotes(Arg0, Arg1)); }
	void test_case_1() { string Arr0[] = {"A B C D", "B C D", "C D"}; vector <string> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); string Arg1 = "A"; long long Arg2 = 8LL; verify_case(1, Arg2, countVotes(Arg0, Arg1)); }
	void test_case_2() { string Arr0[] = {"A"}; vector <string> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); string Arg1 = "A"; long long Arg2 = 1LL; verify_case(2, Arg2, countVotes(Arg0, Arg1)); }
	void test_case_3() { string Arr0[] = {"A B", "B A"}; vector <string> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0]))); string Arg1 = "A"; long long Arg2 = 1LL; verify_case(3, Arg2, countVotes(Arg0, Arg1)); }
	void test_case_4() { string Arr0[] = 
	{"AAA AAB AAC AAD AAE AAF AAG AAH AAI AAJ", 
     "BAA BAB BAC BAD BAE BAF BAG BAH BAI BAJ",
     "CAA CAB CAC CAD CAE CAF CAG CAH CAI CAJ",
     "DAA DAB DAC DAD DAE DAF DAG DAH DAI DAJ",
     "EAA EAB EAC EAD EAE EAF EAG EAH EAI EAJ",
     "FAA FAB FAC FAD FAE FAF FAG FAH FAI FAJ",
     "GAA GAB GAC GAD GAE GAF GAG GAH GAI GAJ",
     "HAA HAB HAC HAD HAE HAF HAG HAH HAI HAJ",
     "IAA IAB IAC IAD IAE IAF IAG IAH IAI IAJ",
     "JAA JAB JAC JAD JAE JAF JAG JAH JAI JAJ",
     "ZZZ AAA BAA CAA DAA EAA FAA GAA HAA IAA JAA",
	 }; vector <string> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0])));
	string Arg1 = "ZZZ"; long long Arg2 = 3LL; verify_case(4, Arg2, countVotes(Arg0, Arg1)); }
	void test_case_5() { string Arr0[] = {
	"A B C D E G",
	"F A J",
	"G F I",
	"H Z G Y",
	"Z A",
	"Y A",
	};
	
	vector <string> Arg0(Arr0, Arr0 + (sizeof(Arr0) / sizeof(Arr0[0])));
	string Arg1 = "H"; long long Arg2 = 4LL; verify_case(5, Arg2, countVotes(Arg0, Arg1)); }

// END CUT HERE
 
}; 

// BEGIN CUT HERE
int main() {
	WebsiteRank2 ___test; 
	___test.run_test(-1); 
	return 0;
} 
// END CUT HERE
